
import bpy

#from . import mesh_setting_module
from . import modifier_module

from bpy.props import *

class POLEMAKEROBJ_OT_BaseObject(bpy.types.Operator, modifier_module.ModifierModule):

    bl_idname = "polemaker.base_object"
    bl_label = "Base Object"
    bl_description = "Description"
    bl_options = {'REGISTER', 'UNDO'}

    action : StringProperty(
        name="action")
    
    AxisType = [("X", "X", "X", "", 1),
                ("Y", "Y", "Y", "", 2)]

    base_axis : EnumProperty(
        name = "Axis", default = "X",
        items = AxisType)

    interval : FloatProperty(
        name = "Interval", default = 20,
        min = 1)

    number : IntProperty(
        name = "Number", default = 2,
        min = 2, max = 99)

    use_cursor : BoolProperty(
        name = "Use Cursor", default = False)

    def draw(self, context):
        layout = self.layout

        col = layout.column(align = True)
        row = col.row()
        row.prop(self, "base_axis", expand=True)
        col.prop(self, "interval")
        col.prop(self, "number")
        
        row = col.row()
        row.prop(self, "use_cursor")
        

    def execute(self, context):

        interval = self.interval
        number = self.number

        vertices = []
        if (self.base_axis == 'X'):
            ini = -interval/2 * (number-1)
            for i in range(number):
                vertices.append((ini + interval * i, 0, 0))

        if (self.base_axis == 'Y'):
            ini = -interval/2 * (number-1)
            for i in range(number):
                vertices.append((0, ini + interval * i, 0))

        edges=[]
        for ini, fin in zip(range(0, number-1), range(1, number)):
            edges.append((ini, fin))

        print("vertices=", tuple(vertices))
        print("edges=", edges)

        mesh = bpy.data.meshes.new("pole_base")
        mesh.from_pydata(tuple(vertices), edges, [])
        
        obj = self.make_obj(context, mesh.name)

        if (self.use_cursor == True):
            obj.location = context.scene.cursor.location

        bpy.ops.object.select_all(action='DESELECT')
        obj.select_set(True)

        mod_mesh = self.add_GN_modifier(obj, ".GN.ShowVertices")

        bpy.context.view_layer.objects.active = obj

        return {'FINISHED'}

