
import bpy
#import bmesh
#import time

#from bpy.props import *
#import numpy as np
#import math
#from mathutils import Vector

class CurveModule:

    def get_col_to_belong(self, context, use_collection):

        obj = context.object

        col_to_belong = context.scene.collection
        if (use_collection):
            for col in bpy.data.collections:
                for o in col.objects:
                    if o == obj:
                        col_to_belong = col

        return col_to_belong

    def resolution_for_GN(self, resolution):
        return 4 + resolution * 2

    def make_curve(self, name):
        curveData = bpy.data.curves.new(name, type='CURVE')
        curveData.dimensions = '3D'
        curveData.resolution_u = 2
        curveData.use_fill_caps = True
        curveData.bevel_resolution = self.resolution
        return curveData

    def set_point_from3d(self, point, co):
        co_tmp = co.copy();
        co_tmp.resize_4d(); point.co = co_tmp

    def set_poly_from3d(self, curveData, co_0, co_1, tilt = 0, radius = 1):
        polyline = curveData.splines.new('POLY')
        points = polyline.points
        points.add(1)
        points[0].tilt = tilt
        points[1].tilt = tilt
        points[0].radius = radius
        points[1].radius = radius
        self.set_point_from3d(points[0], co_0)
        self.set_point_from3d(points[1], co_1)
        return polyline

    def set_material(self, object, material):
        if (material == None):
            return

        object.data.materials.append(material)
        
        object.material_slots[0].link = "OBJECT"
        object.material_slots[0].material = material
        object.material_slots[0].link = "DATA"
            
    def make_curve_object(self, name, data, material, context):
        
        curveOB = bpy.data.objects.new(name, data)

        use_collection = False
        if hasattr(self, "use_collection"):
            use_collection = self.use_collection

        col_to_belong = self.get_col_to_belong(context, use_collection)
        col_to_belong.objects.link(curveOB)
        
        self.set_material(curveOB, material)
        return curveOB
