    
import bpy

from . import mesh_setting_module

class ModifierModule(mesh_setting_module.MeshSettingModule):

    def add_array_modifier(self, object, count, use_relative_offset = True, use_constant_offset = False):

        mod = object.modifiers.new(name = "Array", type="ARRAY")

        mod.count = count
        mod.use_relative_offset = use_relative_offset
        mod.use_constant_offset = use_constant_offset

        return mod
                

    def add_displace_modifier(self, object, group_name, direction, strength, modifier_name = "Displace", single = False):

        if (single == True):
            if modifier_name in object.modifiers:
                mod = object.modifiers[modifier_name]
            else:
                single = False

        if (single == False):
            mod = object.modifiers.new(name = modifier_name, type="DISPLACE")
        
        mod.direction = direction
        mod.vertex_group = group_name

        mod.mid_level = 0
        mod.strength = strength
        
        return mod

    def add_mask_modifier(self, object, group_name, mod_name = "Mask", single = False):

        if (single == True):
            if modifier_name in object.modifiers:
                mod = object.modifiers[modifier_name]
            else:
                single = False

        if (single == False):
            mod = object.modifiers.new(name = mod_name, type="MASK")

        mod.vertex_group = group_name
        mod.invert_vertex_group = True
        
        return mod

    def add_GN_modifier(self, object, node_name, modifier_name = "GeometryNode"):
        if (modifier_name == "GeometryNode"):
            modifier_name = node_name
                
        mod = object.modifiers.new(name = modifier_name, type="NODES")
        old_node = mod.node_group
        if (old_node != None):
            bpy.data.node_groups.remove(mod.node_group)
        node = self.load_NodeGroup(node_name)
        mod.node_group = node

        return mod

    def make_node_dict(self, geometry_node):

        NodeDict = {}
        node_group = geometry_node.node_group

        if bpy.app.version >= (4,0,0):
            for item in node_group.interface.items_tree:
                if item.in_out == 'INPUT':
                    NodeDict[item.name] = item.identifier
        else:
            for input in node_group.inputs:
                NodeDict[input.name] = input.identifier
                
        return NodeDict

    def set_GN_inputs(self, mod, paramList):

        NodeDict = self.make_node_dict(mod)
        node_group = mod.node_group

        socket_name = {
            "object"    :("NodeSocketObject", "OBJECT"),
            "color"     :("NodeSocketColor" , "COLOR"),
            "vector"    :("NodeSocketVetor" , "VECTOR"),
            "value"     :("NodeSocketFloat" , "VALUE"),
            "int"       :("NodeSocketInt"   , "INT"),
            "bool"      :("NodeSocketBool"  , "BOOLEAN"),
            "collection":("NodeSocketCollection", "COLLECTION"),
        }

        for item in paramList:
            socket_key = NodeDict[item[0]]

            if bpy.app.version >= (4,0,0):
                node_input = node_group.interface.items_tree[item[0]]
                socket_type = node_input.socket_type
                version_index = 0
            else:
                node_input = node_group.inputs[item[0]]
                socket_type = node_input.type
                version_index = 1

            socket_object = socket_name["object"][version_index]
            
            if (not socket_type in [socket_object] and type(item[1]) == str):
                mod[socket_key+"_use_attribute"] = 1
                mod[socket_key+"_attribute_name"] = item[1]
            else:
                #^ႤƃG[ɂȂ̂ňꕔSu
                if socket_type == socket_name["vector"][version_index]:
                    mod[socket_key][0] = float(item[1][0])
                    mod[socket_key][1] = float(item[1][1])
                    mod[socket_key][2] = float(item[1][2])
                elif socket_type == socket_name["value"][version_index]:
                    mod[socket_key] = float(item[1])
                    node_input.min_value = node_input.min_value
                    node_input.max_value = node_input.max_value
                elif socket_type == socket_name["int"][version_index]:
                    mod[socket_key] = int(item[1])
                    node_input.min_value = node_input.min_value
                    node_input.max_value = node_input.max_value
                elif socket_type == socket_name["bool"][version_index]:
                    mod[socket_key] = bool(item[1])
                elif socket_type == socket_name["object"][version_index]:
                    if (type(item[1]) == str):
                        if item[1] in bpy.data.objects:
                            mod[socket_key] = bpy.data.objects[item[1]]
                    else:
                        mod[socket_key] = item[1]
                elif socket_type == socket_name["collection"][version_index]:
                    if (type(item[1]) == str):
                        if (item[1] in bpy.data.collections):
                            mod[socket_key] = bpy.data.collections[item[1]]
                    else:
                        mod[socket_key] = item[1]
                else:
                    mod[socket_key] = item[1]

# make_node_dict:
# fBt@CAŌ\Pbgւ̓͂["Input_4"]ƂĂȂĂ̂
# OŐݒłȂ̂ŁAΉ\

# set_GN_inputs:
# make_node_dict ōXggāAOŃp[^Zbgł
# paramList ͂Ȋ
# paramList = [("Scale", self.scale"),
#              ("Rot", self.rot),
#              ("Col", "Col")]
# Ŏw肵ꍇ́AAgr[g

    def add_GP_offset_modifier(self, object, location):

        mod = object.grease_pencil_modifiers.new(name = "Offset", type="GP_OFFSET")
        mod.location = location

        return mod


    
