# -*- coding: utf-8 -*-
from .download import download_zip,download_file
from pathlib import Path
down_path = Path(__file__).parent
xz_url = "addons_file"+"/"+down_path.name+"/"
download_zip(xz_url + "icons.zip",down_path)
download_zip(xz_url + "textures.zip",down_path)
download_file(xz_url+"PoleMaker.blend",down_path)
download_file(xz_url+"SignalMark.blend",down_path)

bl_info = {
    "name" : "Pole Maker",
    "author" : "Q@StudioPotpourri",
    "version" : (1,2,0),
    "blender" : (3, 3, 0),
    "location" : "Edit Mode > Create > Pole Maker",
    "description" : "Make Poles",
    "warning" : "",
    "doc_url" : "",
    "tracker_url" : "",
    "category" : "Mesh"
}

if "bpy" in locals():
    import imp
    imp.reload(base_module)
    imp.reload(curve_module)
    imp.reload(type_module)
    imp.reload(mesh_setting_module)
    imp.reload(modifier_module)
    imp.reload(catenary_module)
    imp.reload(translate_module)
    imp.reload(category_module)
else:
    from . import base_module
    from . import curve_module
    from . import type_module
    from . import mesh_setting_module
    from . import modifier_module
    from . import catenary_module
    from . import translate_module
    from . import category_module

import bpy
import bmesh
import re
import math
import numpy as np
import mathutils
import os
import random
import time
import bpy.utils.previews

from os import path
from math import sin, cos, sqrt, atan2
from mathutils import Vector, Matrix, noise
from bpy.props import *
from bpy.types import Operator, Panel

dirname = path.dirname(__file__)
addon_name = __package__

preview_collections = {}

#call_backで使うのでglobal関数でないといけない
def get_pcoll_icon_id(pcoll_name, index_or_name):
    if (type(index_or_name) == int):
        return preview_collections[pcoll_name]["%02d" % index_or_name].icon_id
    return preview_collections[pcoll_name][index_or_name].icon_id

def common_type_callback(max, type):
    get = get_pcoll_icon_id
    items = [];
    id = ["DUMMY", "TYPEA", "TYPEB", "TYPEC", "TYPED", "TYPEE", "TYPEF", ]
    name = ["Dummy", "Type A", "Type B", "Type C", "Type D", "Type E", "Type F", ]
    for index in range(1,max+1):
        items.append((id[index], name[index], "", get(type, index), index))
    return items
    
    

class PoleMakerPreferences(bpy.types.AddonPreferences):
    bl_idname = __package__

    category: StringProperty(
        name="Tab Category", default="Create",
        description="Choose a name for the category of the panel",
        update=category_module.update_panel
    )

    use_material : BoolProperty(
        name = "Material", default = True,
        description = "Set default materials")

    max_count : IntProperty(
        name = "Max Object Number", default = 40,
        min = 40, max = 199)

    def draw(self, context):
        
        layout = self.layout

        row = layout.row()
        col = row.column()
        col.label(text="Tab Category:")
        col.prop(self, "category", text="")    
        
        col = layout.column(align = True)
        col.prop(self, 'use_material')
        col.prop(self, 'max_count')

class MarkCommons:
    
    MarkCircleCategoryType = [
        ("LIMITTING", "Limitting", "Limitting", "CUSTOM_LIMITTING", 1),
        ("SPEED", "Speed", "Speed", "CUSTOM_SPEED", 2),
        ("HEIGHTWIDTH", "Height Width Limit", "Height Width Limit", "ARROW_LEFTRIGHT", 3),
        ("ARROWS", "Arrow Mark", "Arrow Mark", "SORT_DESC", 4),
        ("EXCLUSIVE", "Single Purpose", "Single Purpose", "PROP_ON", 5),
        ("FIRE", "Fire Fighting", "Fire Fighting", "MESH_CIRCLE", 6)
    ]

class PoleMakerCommons :

    TrafficLightType = [("FOR_CARS", "For Cars", "For Cars", "", 1),
                        ("FOR_CARS_DOUBLE", "Double-Sided", "Double-Sided", "", 2),
                        ("FOR_WALKERS", "For Walkers", "For Walkers", "", 3)]
    
    TrafficLightDict = {"TYPEA":"Signal.A",
                        "TYPEB":"Signal.B",
                        "TYPEC":"Signal.C",
                        "TYPED":"Signal.D",
                        "TYPEE":"Signal.E",
                        }

    TrafficLightDict_ItemPos = {
        "TYPEA":((0, 1.817, -0.36),     (0, 1.847, 0), (0, 0.4, 0), ),
        "TYPEB":((0, 1.817, -0.721),    (0, 1.847, 0), (0, 0.4, 0), ),
        "TYPEC":((0.03, 4.107, -0.721), (0, 4.107, 0), (0, 2.4, 0), ),
        "TYPED":((-0.16, 1.817, -0.424),(0, 1.847, 0), (0, 0.4, 0), ),
        "TYPEE":((-0.22, 4.107, -0.551),(0, 4.107, 0), (0, 2.4, 0), ),
    }

   
    TrafficLightDoubleDict = {"TYPEA":"Signal.Double.A",
                              "TYPEB":"Signal.Double.B",
                              "TYPEC":"Signal.Double.C",
                              "TYPED":"Signal.Double.D",
                              "TYPEE":"Signal.Double.E"
                              }


    # 複雑な方
    TrafficLightRingDict_FullA = {
        "TYPEA":((0,), ("Full.A.A",)),
        "TYPEB":((0,), ("Full.A.A",)),
        "TYPEC":((0, -1.25), ("Full.A.A", "Full.A.B")),
        "TYPED":((0,), ("Full.A.A",)),
        "TYPEE":((0, -0.16, 2.09), ("Full.A.C", None, "Full.A.D")),
    }

    TrafficLightRingDict_FullB = {
        "TYPEA":((0,), ("Full.B.A",)),
        "TYPEB":((0,), ("Full.B.A",)),
        "TYPEC":((0, -1.25), ("Full.B.A", "Full.B.B")),
        "TYPED":((0,), ("Full.B.A",)),
        "TYPEE":((0, 2.09), ("Full.B.C", "Full.B.D")),
    }

    TrafficLightRingDict_FullC = {
        "TYPEA":((0,), ("Full.C.A",)),
        "TYPEB":((0,), ("Full.C.A",)),
        "TYPEC":((0, -1.25), ("Full.C.A", "Full.C.B")),
        "TYPED":((0,), ("Full.C.A",)),
        "TYPEE":((0, 2.09), ("Full.C.C", "Full.C.D")),
    }
    
    TrafficLightWalkerDictA = {"TYPEA":"Signal.Walker.A.A",
                               "TYPEB":"Signal.Walker.A.B",
                               "TYPEC":"Signal.Walker.A.C",
                               "TYPED":"Signal.Walker.A.D"}

    TrafficLightWalkerDictB = {"TYPEA":"Signal.Walker.B.A",
                               "TYPEB":"Signal.Walker.B.B",
                               "TYPEC":"Signal.Walker.B.C",
                               "TYPED":"Signal.Walker.B.D"}

    TrafficLightWalkerRingDict_FullA = {
        "TYPEA":((0,), ("Full.A.B",)),
        "TYPEB":((0, -0.94), ("Full.A.B", "Full.A.B")),
        "TYPEC":((0,), ("Full.A.B",)),
        "TYPED":((0, -0.94), ("Full.A.B", "Full.A.B")),
    }

    TrafficLightWalkerRingDict_FullB = {
        "TYPEA":((0,), ("Full.B.E",)),
        "TYPEB":((0, -0.94), ("Full.B.E", "Full.B.E")),
        "TYPEC":((0,), ("Full.B.E",)),
        "TYPED":((0, -0.94), ("Full.B.E", "Full.B.E")),
    }
    
    TrafficLightWalkerRingDict_FullC = {
        "TYPEA":((0,), ("Full.C.G",)),
        "TYPEB":((0, -0.94), ("Full.C.G", "Full.C.G")),
        "TYPEC":((0,), ("Full.C.E",)),
        "TYPED":((0, -0.94), ("Full.C.E", "Full.C.E")),
    }
    

    ArrowLightType = [("TYPEA_SINGLE", "TypeA Single", "TypeA Single", "MESH_PLANE", 1),
                      ("TYPEA_TRIPLE", "TypeA Triple", "TypeA Triple", "MESH_PLANE", 2),
                      ("TYPEB_SINGLE", "TypeB Single", "TypeB Single", "MESH_CIRCLE", 3),
                      ("TYPEB_TRIPLE", "TypeB Triple", "TypeB Triple", "MESH_CIRCLE", 4),
                      ]

    TrafficLightArrowDict = {"TYPEA_SINGLE":"Signal.EX.Single.A",
                             "TYPEA_TRIPLE":"Signal.EX.Triple.A",
                             "TYPEB_SINGLE":"Signal.EX.Single.B",
                             "TYPEB_TRIPLE":"Signal.EX.Triple.B",
                             }

    TrafficLightHeadBoardDict = {"TYPEA":"Signal.HeadBoard.A",
                                 "TYPEB":"Signal.HeadBoard.B",
                                 }

    TrafficLightSideBoardDict = {"TYPEA":"Signal.SideBoard.A",
                                 "TYPEB":"Signal.SideBoard.B",
                                 "TYPEC":"Signal.SideBoard.C",
                                 "TYPEA_DOUBLE":"Signal.SideBoard.Double.A",
                                 "TYPEB_DOUBLE":"Signal.SideBoard.Double.B",
                                 "TYPEC_DOUBLE":"Signal.SideBoard.Double.C",
                                 }

    TrafficLightSideBoardWalkerDictA = {"TYPEA":"Signal.SideBoard.W.A.A",
                                        "TYPEB":"Signal.SideBoard.W.A.B",
                                        "TYPEC":"Signal.SideBoard.W.A.C",
                                        "TYPED":"Signal.SideBoard.W.A.D",
                                        }

    TrafficLightSideBoardWalkerDictB = {"TYPEA":"Signal.SideBoard.W.B.A",
                                        "TYPEB":"Signal.SideBoard.W.B.B",
                                        "TYPEC":"Signal.SideBoard.W.B.C",
                                        "TYPED":"Signal.SideBoard.W.B.D",
                                   }
    

    ControllerType = [("BUTTON", "Pelican Button", "Pecican Button", 1),
                      ("BOX", "Control Box", "Control Box", 2)]

    def TrafficLightSubtype_callback(self, context):
        return common_type_callback(5, "TrafficLightType")

    def TrafficLightWalker_callback(self, context):
        return common_type_callback(4, "TrafficLightWalkerType")

    def Pelican_callback(self, context):
        return common_type_callback(3, "PelicanType")

    def ControllerBox_callback(self, context):
        return common_type_callback(4, "ControllerBoxType")

    def Signboard_callback(self, context):
        return common_type_callback(4, "AppendixSignboardType")

    SignboardLayoutType = [("CENTERED", "Centered", "Centered", "", 1),
                           ("OFFSET", "Offset", "Offset", "", 2)]

    SignboardSizeType = [("0.4m", "0.4m", "0.4m", "", 1),
                         ("0.6m", "0.6m", "0.6m", "", 2),
                         ("0.9m", "0.9m", "0.9m", "", 3),
                         ("1.2m", "1.2m", "1.2m", "", 4),]
   

    SignboardMappingDict = {"OV0.4m_Circle":"CIRCLE",
                            "OV0.4m_TriangleA":"TRIANGLEA",
                            "OV0.4m_TriangleB":"TRIANGLEB",
                            "OV0.4m_Rhombus":"RHOMBUS",
                            "OV0.4m_SquareA":"SQUARE",
                            "OV0.4m_SquareB":"SQUARE",
                            "OV0.4m_SquareC":"SQUARE",
                            "CV0.4m_Circle":"CIRCLE",
                            "CV0.4m_TriangleA":"TRIANGLEA",
                            "CV0.4m_TriangleB":"TRIANGLEB",
                            "CV0.4m_Rhombus":"RHOMBUS",
                            "CV0.4m_SquareA":"SQUARE",
                            "CV0.4m_SquareB":"SQUARE",
                            "CV0.4m_SquareC":"SQUARE",
                            "CV0.6m_Circle":"CIRCLE",
                            "CV0.6m_TriangleA":"TRIANGLEA",
                            "CV0.6m_TriangleB":"TRIANGLEB",
                            "CV0.6m_Rhombus":"RHOMBUS",
                            "CV0.6m_SquareA":"SQUARE",
                            "CV0.6m_SquareB":"SQUARE",
                            "CV0.6m_SquareC":"SQUARE",
                            "CH0.6m_Circle":"CIRCLE",
                            "CH0.6m_TriangleA":"TRIANGLEA",
                            "CH0.6m_TriangleB":"TRIANGLEB",
                            "CH0.6m_Rhombus":"RHOMBUS",
                            "CH0.6m_SquareA":"SQUARE",
                            "CH0.6m_SquareB":"SQUARE",
                            "CH0.6m_SquareC":"SQUARE",
                            "CH0.9m_Circle":"CIRCLE",
                            "CH0.9m_TriangleA":"TRIANGLEA",
                            "CH0.9m_TriangleB":"TRIANGLEB",
                            "CH0.9m_Rhombus":"RHOMBUS",
                            "CH0.9m_SquareA":"SQUARE",
                            "CH0.9m_SquareB":"SQUARE",
                            "CH0.9m_SquareC":"SQUARE",
                            "CH1.2m_Circle":"CIRCLE",
                            "CH1.2m_TriangleA":"TRIANGLEA",
                            "CH1.2m_TriangleB":"TRIANGLEB",
                            "CH1.2m_Rhombus":"RHOMBUS",
                            "CH1.2m_SquareA":"SQUARE",
                            "CH1.2m_SquareB":"SQUARE",
                            "CH1.2m_SquareC":"SQUARE",
                            }
    

    # 原点を挟んでの上サイズ、下サイズ
    Signboard_PropDict = {"OV0.4m_Circle":(0.22, 0.23),
                          "OV0.4m_TriangleA":(0.17, 0.25),
                          "OV0.4m_TriangleB":(0.35, 0.15),
                          "OV0.4m_Rhombus":(0.25, 0.25),
                          "OV0.4m_SquareA":(0.07, 0.13),
                          "OV0.4m_SquareB":(0.11, 0.18),
                          "OV0.4m_SquareC":(0.25, 0.33),
                          "CV0.4m_Circle":(0.22, 0.23),
                          "CV0.4m_TriangleA":(0.17, 0.25),
                          "CV0.4m_TriangleB":(0.35, 0.15),
                          "CV0.4m_Rhombus":(0.25, 0.25),
                          "CV0.4m_SquareA":(0.07, 0.13),
                          "CV0.4m_SquareB":(0.11, 0.18),
                          "CV0.4m_SquareC":(0.25, 0.33),
                          "CV0.6m_Circle":(0.365, 0.335),
                          "CV0.6m_TriangleA":(0.25, 0.37),
                          "CV0.6m_TriangleB":(0.53, 0.22),
                          "CV0.6m_Rhombus":(0.38, 0.37),
                          "CV0.6m_SquareA":(0.10, 0.20),
                          "CV0.6m_SquareB":(0.17, 0.27),
                          "CV0.6m_SquareC":(0.37, 0.50),
                          "ADDITIONAL":(0.15, 0.15),
                          "CUSTOM":(0.2, 0.2)}
    
    FootageBoltDict = {"Default":"FootageBolt",}


    ConnectorDict = {1:"ConnectorFull.A.A",
                     2:"ConnectorFull.A.B",
                     3:"ConnectorFull.A.C",
                     4:"ConnectorFull.A.D",
                     5:"ConnectorFull.B.A",
                     6:"ConnectorFull.B.B",
                     7:"ConnectorFull.B.C",
                     8:"ConnectorFull.B.D",
                     9:"ConnectorFull.B.E",
                     10:"ConnectorFull.B.F",
                     11:"ConnectorFull.C.A",
                     12:"ConnectorFull.C.B",
                     13:"ConnectorFull.C.C",
                     14:"ConnectorFull.C.D",
                     15:"ConnectorFull.C.E",
                     15:"ConnectorFull.C.F",
                     16:"ConnectorFull.C.G",
                     }

    GuideSignLargeArmOffsetDict = {"2.8x2.4m":0,
                                   "2.8x2.2m":0,
                                   "2.4x2.0m":-0.2}

    SignboardPoleDict = {"0.04m":(2.7, -0.3, 0.02, 0.02),
                         "0.06m":(3.0, -0.5, 0.03, 0.03),
                         "0.2m":(6.5, -0.8, 0.1, 0.1),
                         "0.3m":(6.5, -0.8, 0.15, 0.15),}

    GuideSignSmallType_Dict = {"TYPEA":"GuideSign.S.A",
                               "TYPEB":"GuideSign.S.B",
                               "TYPEC":"GuideSign.S.C"}

    TypeDict = {"TYPEA":"A",
                "TYPEB":"B",
                "TYPEC":"C",
                "TYPED":"D",
                "TYPEE":"E",
                "TYPEF":"F"}

    def SignboardPoleSignType_callback(self, context):
        type = "PoleSignType"
        get = get_pcoll_icon_id
        items = [("CIRCLE"   , "Circle"    , "", get(type, 1), 1),
                 ("TRIANGLEA", "Triangle A", "", get(type, 2), 2),
                 ("TRIANGLEB", "Triangle B", "", get(type, 3), 3),
                 ("RHOMBUS"  , "Rhombus"   , "", get(type, 4), 4),
                 ("SQUARE"   , "Square"    , "", get(type, 5), 5),
                 ]
        return items

    def get_enum_current_name(self, prop, enum_list):
        dir = {}
        for enum_item in enum_list:
            dir.update({enum_item[0]:enum_item[1]})
        prop_value = getattr(self, prop)
        name = dir[prop_value]
        #print(prop_value, name)

        return name
        

    def PS2(self, col, props, flip = False): #特殊レイアウト
        if (flip == True):
            split = col.split(factor = 0.75)
            col_2 = split.column(align = True)
            split = split.split(factor = 1)
            col_1 = split.column(align = True)
        else:
            split = col.split(factor = 0.25)
            col_1 = split.column(align = True)
            split = split.split(factor = 1)
            col_2 = split.column(align = True)
        for idx, prop in enumerate(props):
            if (idx == 0):
                col_1.template_icon_view(self, prop, scale=3.0, scale_popup=3.0)
            else:
                if "@" in prop: # ラベル
                    prop = prop.replace("@", "")
                    col_2.label(text = prop)
                else:
                    if "#" in prop: # label 無し
                        prop = prop.replace("#", "")
                        col_2.prop(self, prop, text="")
                    else:
                        col_2.prop(self, prop)

    def PS3(self, col, props): #特殊レイアウト
        split = col.split(factor = 0.25)
        col_1 = split.column(align = True)
        split = split.split(factor = 0.33)
        col_2 = split.column(align = True)
        split = split.split(factor = 1)
        col_3 = split.column(align = True)
        for idx, prop in enumerate(props):
            if (idx == 0):
                col_1.template_icon_view(self, prop, scale=3.0, scale_popup=3.0)
            elif (idx == 1):
                col_2.template_icon_view(self, prop, scale=3.0, scale_popup=3.0)
            else:
                if "@" in prop: # ラベル
                    prop = prop.replace("@", "")
                    col_3.label(text = prop)
                else:
                    if "#" in prop: # label 無し
                        prop = prop.replace("#", "")
                        col_3.prop(self, prop, text="")
                    else:
                        col_3.prop(self, prop)
    
    def EnumCustomIconBase(type_list):
        
        icons = preview_collections["icons"]
        dynamic_list = []
        for item in type_list:
            item_list = list(item)
            if 'CUSTOM_' in item_list[3]:
                name = item[3][7:]
                item_list[3] = icons[name].icon_id
            
            dynamic_list.append(tuple(item_list))
        
        return dynamic_list


    def EnumCustomIcon_AppendixType(self, context):
        return PoleMakerCommons.EnumCustomIconBase(PoleMakerCommons.AppendixType)
    
    def make_telegraph_footage(self, context, pole, min_height, pos_z):
        
        m_name = self.FootageBoltDict["Default"]
        col = self.get_asset_collection(context, "PoleParts", context.scene.collection)
        partsOB = self.load_object(context, m_name, col)

        mod_set = self.add_GN_modifier(pole, ".GN.Pole.FootageBolt")
        paramList = [("Object"   , partsOB),
                     ("Top_Z"    , pos_z),
                     ("MinHeight", min_height)]
        self.set_GN_inputs(mod_set, paramList)
        
    def get_signboard_type(self, str_layout, str_size, sign, square_type):

        if sign=="":
            sign = "CIRCLE"
            
        if sign == "CIRCLE":
            signboard_type = str_layout + str_size + "_Circle"

        if sign == "TRIANGLEA":
            signboard_type = str_layout + str_size + "_TriangleA"

        if sign == "TRIANGLEB":
            signboard_type = str_layout + str_size + "_TriangleB"

        if sign == "RHOMBUS":
            signboard_type = str_layout + str_size + "_Rhombus"
                
        if sign == "SQUARE":
            signboard_type = str_layout + str_size + "_Square"
            signboard_type += {"TYPEA":"A",
                               "TYPEB":"B",
                               "TYPEC":"C"}[square_type]
        
        return signboard_type

    def newbox(self, colA):
        colA.separator()

        box = colA.box()
        col = box.column(align = True)
        row = col.row()
        return col, row
   

#Appendix としても付けられるパーツの変数
#
class AppendixCommon(modifier_module.ModifierModule):

    CommonTypeSelection = type_module.TypeModule.CommonTypeSelection
    CommonTypeSelectionWithNone = type_module.TypeModule.CommonTypeSelectionWithNone

    # 不規則にリングを追加(2個までのRingを "AddRingConnector"で追加)
    # Num, offset, interval, ringwidth
    RingInfoDict = {'Lamp.A': [1, 0, 0.25, 0.05],
                    'Lamp.B': [2, 0, 0.25, 0.05],
                    'Lamp.C': [2, 0, 0.25, 0.05],
                    'Trans_30kVA.Single': [2, -0.06, 0.33, 0.05],
                    'Trans_30kVA.Dual':   [2, -0.21, 0.33, 0.05],
                    'Trans_20kVA.Single.A': [0, -0.21, 0.33, 0.05],
                    'Trans_20kVA.Single.B': [0, -0.21, 0.33, 0.05],
                    'Trans_20kVA.Dual': [0, -0.21, 0.12, 0.05],
                    'ControlBox.A' : [2, -0.14, 1.32, 0.05],
                    'ControlBox.B' : [2, 0.42, 0.76, 0.05],
                    'ControlBox.C' : [1, 0.48, 0, 0.05],
                    'ControlBox.D' : [2, -0.16, 0.76, 0.05],
                    'ControlBox.A.mir' : [2, -0.14, 1.32, 0.05],
                    'ControlBox.B.mir' : [2, 0.42, 0.76, 0.05],
                    'ControlBox.C.mir' : [1, 0.48, 0, 0.05],
                    'ControlBox.D.mir' : [2, -0.16, 0.76, 0.05],
                    'ControlButton.A' : [1, 0, 0.25, 0.05],
                    'ControlButton.B' : [2, -0.114, 0.22, 0.05],
                    'ControlButton.C' : [2, -0.11, 0.28, 0.05],
                    'SignBoardBase.A' : [2, -0.73, 0.27, 0.08],
                    'SignBoardBase.B' : [1, 0, 0, 0.08],
                    'SignBoardBase.C' : [2, 0, 1.18, 0.08],
                    'SignBoardBase.D' : [2, -0.09, 0.18, 0.08],
                    }

    TransType = [("NONE", "None", "None", "", 1),
                 ("30KVA_SINGLE", "30kVA Single", "30kVA Single", "", 2),
                 ("30KVA_DUAL", "30kVA Dual", "30kVA Dual", "", 3),
                 ("20KVA_SINGLEA", "20kVA Single A", "30kVA Single A", "", 4),
                 ("20KVA_SINGLEB", "20kVA Single B", "30kVA Single B", "", 5),
                 ("20KVA_DUAL", "20kVA Dual", "20kVA Dual", "", 6),]

    TransDict = {'30KVA_SINGLE':'Trans_30kVA.Single',
                 '30KVA_DUAL':'Trans_30kVA.Dual',
                 '20KVA_SINGLEA':'Trans_20kVA.Single.A',
                 '20KVA_SINGLEB':'Trans_20kVA.Single.B',
                 '20KVA_DUAL':'Trans_20kVA.Dual',
                 }
    
    LampDict = {'TYPEA':"Lamp.A",
                'TYPEB':"Lamp.B",
                'TYPEC':"Lamp.C",}

    GuideSignLargeType = [("2.8x2.4m", "2.8x2.4m", "2.8x2.4m", "", 1),
                          ("2.8x2.2m", "2.8x2.2m", "2.8x2.2m", "", 2),
                          ("2.4x2.0m", "2.4x2.0m", "2.4x2.0m", "", 3)]

    def TransType_callback(self, context):
        type = "TransformerType"
        get = get_pcoll_icon_id
        items = [("NONE"         , "None"          , "", get(type, -1), 1),
                 ("30KVA_SINGLE" , "30kVA Single"  , "", get(type, 1), 2),
                 ("30KVA_DUAL"   , "30kVA Dual"    , "", get(type, 2), 3),
                 ("20KVA_SINGLEA", "20kVA Single A", "", get(type, 3), 4),
                 ("20KVA_SINGLEB", "20kVA Single B", "", get(type, 4), 5),
                 ("20KVA_DUAL"   , "20kVA Dual"    , "", get(type, 5), 6),]
        return items

    def ArmHV_callback(self, context):
        return common_type_callback(6, "ArmHVType")

    def ArmLV_callback(self, context):
        return common_type_callback(5, "ArmLVType")

    def ArmCom_callback(self, context):
        return common_type_callback(4, "ArmComType")

    def AppendixLight_callback(self, context):
        return common_type_callback(3, "AppendixLampType")

  
    telegraph_lamp_type : EnumProperty(
        name = "Type", default = 1,
        items = AppendixLight_callback)

    telegraph_lamp_offset : FloatProperty(
        name = "Offset", default = 0,
        min = -2, max = 2, step = 0.3)
    
    telegraph_make_rings : BoolProperty(
        name = "Make Rings", default = False)

    telegraph_trans_type : EnumProperty(
        name = "Transformer", default = 2,
        items = TransType_callback)

    telegraph_trans_pos : EnumProperty(
        name = "Position", default = "HIGHER",
        items = [("HIGHER", "Higher", "Higher", "", 1),
                 ("LOWER1", "Lower 1", "Lower", "", 2),
                 ("LOWER2", "Lower 2", "Lower", "", 3)])

    # 不規則にリングを追加(1つずつAddRingで追加)
    RingZDict = {'ArmHV.A': (),
                 'ArmHV.B': (),
                 'ArmHV.C': (),
                 'ArmHV.D': (-0.21, -0.10, 0.40, 0.50),
                 'ArmHV.E': (),
                 'ArmHV.F': (),
                 'ArmLV.A': (0, 0.29, 0.505),
                 'ArmLV.B': (),
                 'ArmLV.C': (-0.08, 0.03, 0.36, 0.47),
                 'ArmLV.D': (-0.08, 0.03, 0.36, 0.47),
                 'ArmLV.E': (0,),
                 'ArmCom.A': (0, 0.26),
                 'ArmCom.B': (-0.08, 0.03),
                 'ArmCom.C': (-0.08, 0.03, 0.18, 0.28),
                 'ArmCom.D': (-0.08, 0.03, 0.36, 0.470),
                 'ArmWire.A': (0, ),
                 }

    ArmHVDict = {'TYPEA':("ArmHV.A", 1, 1.5, 0), 
                 'TYPEB':("ArmHV.B", 1, 1.5, 0), 
                 'TYPEC':("ArmHV.C", 1, 1.5, 0),
                 'TYPED':("ArmHV.D", 0, 1.6, 0),
                 'TYPEE':("ArmHV.E", 0, 1.25, 0.25),
                 'TYPEF':("ArmHV.F", 0, 1.25, 0.25),}

    ArmLVDict = {'TYPEA':"ArmLV.A",
                 'TYPEB':"ArmLV.B",
                 'TYPEC':"ArmLV.C",
                 'TYPED':"ArmLV.D",
                 'TYPEE':"ArmLV.E",}

    ArmComDict = {'TYPEA':"ArmCom.A",
                  'TYPEB':"ArmCom.B",
                  'TYPEC':"ArmCom.C",
                  'TYPED':"ArmCom.D",}

    ArmWireDict = {'TYPEA':"ArmWire.A"}

    telegraph_arm_HV_type : EnumProperty(
        name = "Type", default = 1,
        items = ArmHV_callback)

    telegraph_arm_LV0_type : EnumProperty(
        name = "Type", default = 1,
        items = ArmLV_callback)

    telegraph_arm_LV1_type : EnumProperty(
        name = "Type", default = 2,
        items = ArmLV_callback)

    telegraph_arm_com_type : EnumProperty(
        name = "Type", default = 1,
        items = ArmCom_callback)

    telegraph_arm_wire_type : EnumProperty(
        name = "Type", default = "TYPEA",
        items = CommonTypeSelection[0:1])

    telegraph_arm_HV_offset : FloatProperty(
        name = "Offset", default = 1,
        min = 0.2, max = 2, step = 0.3)

    telegraph_arm_LV1_offset : FloatProperty(
        name = "Offset", default = 0,
        min = -2, max = 2, step = 0.3)

    telegraph_arm_com_offset : FloatProperty(
        name = "Offset", default = 0,
        min = -2, max = 2, step = 0.3)

    telegraph_arm_wire_offset : FloatProperty(
        name = "Offset", default = 0,
        min = -6, max = 2, step = 0.3)

    guidesign_large_type : EnumProperty(
        name = "Type", default = "2.8x2.2m",
        items = GuideSignLargeType)

    guidesign_tex_index : IntProperty(
        name = "Tex Index", default = 1,
        min = 0, max = 99)

    traffic_light_type : EnumProperty(
        name = "Type", default = "FOR_CARS",
        items = PoleMakerCommons.TrafficLightType)

    traffic_light_style : EnumProperty(
        name = "Style", default = 1,
        items = CommonTypeSelection[0:3])

    traffic_light_subtype : EnumProperty(
        name = "Type", default = 1,
        items = PoleMakerCommons.TrafficLightSubtype_callback)

    traffic_light_walker_subtype : EnumProperty(
        name = "Type", default = 1,
        items = PoleMakerCommons.TrafficLightWalker_callback)
    
    #traffic_light_offset : FloatProperty(
    #    name = "Offset", default = 0,
    #    min = -2, max = +2, step = 0.2)

    traffic_light_extend : FloatProperty(
        name = "Extend", default = 0,
        min = -1, max = +2, step = 0.2)

    traffic_light_mirror : BoolProperty(
        name = "Mirror", default = False)

    traffic_light_arrow_signal : BoolProperty(
        name = "Arrow Signal", default = False)

    traffic_light_arrow_signal_type : EnumProperty(
        name = "Type", default = "TYPEA_SINGLE",
        items = PoleMakerCommons.ArrowLightType)

    traffic_light_head_board : BoolProperty(
        name = "Text Head Board", default = False)

    traffic_light_head_board_type : EnumProperty(
        name = "Type", default = "TYPEA",
        items = CommonTypeSelection[0:2])

    traffic_light_head_board_offset : FloatProperty(
        name = "Offset", default = 0,
        min = -0.5, max = 0.5, step = 0.1)

    traffic_light_head_board_tex_index : IntProperty(
        name = "Tex Index", default = 1,
        min = 0, max = 99)

    traffic_light_side_board : BoolProperty(
        name = "Text Side Board", default = False)

    traffic_light_side_board_type : EnumProperty(
        name = "Type", default = "TYPEA",
        items = CommonTypeSelection[0:3])

    traffic_light_side_board_tex_index : IntProperty(
        name = "Tex Index", default = 1,
        min = 0, max = 99)

    traffic_light_side_board_double : BoolProperty(
        name = "Double-Sided", default = False)

    traffic_light_support : BoolProperty(
        name = "Support", default = False)

    traffic_light_speaker : BoolProperty(
        name = "Speaker", default = False)

    ControlBoxDict = {"TYPEA":"ControlBox.A",
                      "TYPEB":"ControlBox.B",
                      "TYPEC":"ControlBox.C",
                      "TYPED":"ControlBox.D",}


    ControlButtonDict = {"TYPEA":"ControlButton.A",
                         "TYPEB":"ControlButton.B",
                         "TYPEC":"ControlButton.C",
                         }

    controller_type : EnumProperty(
        name = "Type", default = "BUTTON",
        items = PoleMakerCommons.ControllerType)

    controller_box_type : EnumProperty(
        name = "Type", default = 1,
        items = PoleMakerCommons.ControllerBox_callback)

    controller_box_mark : BoolProperty(
        name = "Mark", default = True)

    controller_box_opt1 : BoolProperty(
        name = "Opt1", default = True)

    controller_box_ext : FloatProperty(
        name = "Extend", default = 0,
        min = -1, max = 1, step = 0.3)

    controller_box_mirror : BoolProperty(
        name = "Mirror", default = False)

    controller_button_type : EnumProperty(
        name = "Type", default = 1,
        items = PoleMakerCommons.Pelican_callback)

    controller_button_ext : FloatProperty(
        name = "Extend", default = 0,
        min = -1, max = 1, step = 0.3)

    #controller_offset : FloatProperty(
    #    name = "Offset", default = 0,
    #    min = -0.5, max = 0.5, step = 0.2)

    def set_radius_displace(self, obj, pole, offset_X = False):

        #radius = self.get_radius_at(pos_z, self.telegraph_type)
        #self.add_displace_modifier(obj, "", "Y", radius)
        offset = {True:Vector((1,0,0)),
                  False:Vector((0,1,0))}[offset_X]
        
        mod_offset = self.add_GN_modifier(obj, ".GN.Pole.OffsetFromRadius")
        paramList = [("ParentPole", pole),
                     ("Offset"    , offset),
                     ("Invert"    , "INVERT")]
        self.set_GN_inputs(mod_offset, paramList)
            
    def make_arm_rings(self, objArm, m_name, pole, width = 0.05, enable = True):

        material = self.load_material(".Pole.Basic")
        rings = self.RingZDict[m_name]
        for r in rings:
            mod_ring = self.add_GN_modifier(objArm, ".GN.Pole.AddRing")
            mod_ring.show_expanded = False
            mod_ring.show_viewport = enable
            mod_ring.show_render = enable
            paramList = [("ParentPole", pole),
                         ("Material", material),
                         ("RingOffset", r),
                         ("RingWidth", width)]
            self.set_GN_inputs(mod_ring, paramList)
            
    def make_telegraph_arm_HV(self, context, pole, type, pos_z, flip):
        
        m_name, offset, trans_offset, self_offset = self.ArmHVDict[type]
        
        pos_z += self_offset
        objArm = self.make_on_pole(context, pole, m_name, pos_z, offset = offset)
        objArm.poleprops.pole_type = "TELEGRAPH_ARM"
        objArm.poleprops.pole_subtype = "HV"
        if (flip == True):
            objArm.rotation_euler.z += 3.14159

        self.make_arm_rings(objArm, m_name, pole, enable = self.telegraph_make_rings)
        
        return objArm, trans_offset

    def make_telegraph_arm_LV(self, context, pole, type, pos_z, flip):
        
        m_name = self.ArmLVDict[type]
        
        offset = 0
        if (type == "TYPEB"):
            offset = 1
            
        objArm = self.make_on_pole(context, pole, m_name, pos_z, offset)
        objArm.poleprops.pole_type = "TELEGRAPH_ARM"
        objArm.poleprops.pole_subtype = "LV"
        if (flip == True):
            objArm.rotation_euler.z += 3.14159

        if (type in ["TYPEA", "TYPEC", "TYPED"]):
            self.set_radius_displace(objArm, pole)

        width = 0.05
        make_rings = self.telegraph_make_rings

        if (type in ["TYPEE"]):
            mod = self.add_GN_modifier(objArm, ".GN.Pole.AdjustParts.Full")
            paramList = [("ParentPole", pole),
                         ("BaseValue", 0.110),
                         ("RingOffset", 0)]
            self.set_GN_inputs(mod, paramList)
            width = 0.038
            make_rings = True

        self.make_arm_rings(objArm, m_name, pole, width=width, enable = make_rings)

        return objArm

    def make_telegraph_arm_com(self, context, pole, type, pos_z, flip):

        m_name = self.ArmComDict[type]
        
        objArm = self.make_on_pole(context, pole, m_name, pos_z, 0)
        objArm.poleprops.pole_type = "TELEGRAPH_ARM"
        objArm.poleprops.pole_subtype = "Com"
        if (flip == True):
            objArm.rotation_euler.z += 3.14159
            
        self.set_radius_displace(objArm, pole)
        self.make_arm_rings(objArm, m_name, pole, enable = self.telegraph_make_rings)

        return objArm
        
    def make_telegraph_arm_wire(self, context, pole, type, pos_z, flip):

        m_name = self.ArmWireDict[type]

        objArm = self.make_on_pole(context, pole, m_name, pos_z, 0)
        objArm.poleprops.pole_type = "TELEGRAPH_ARM"
        objArm.poleprops.pole_subtype = "WIRE"
        if (flip == True):
            objArm.rotation_euler.z += 3.14159
            
        self.set_radius_displace(objArm, pole, offset_X=True)
        
        self.make_arm_rings(objArm, m_name, pole, enable = self.telegraph_make_rings)
        return objArm
        
    def make_telegraph_instance_appendix(self, context, pole, m_name, obj_name, pos_z, rot_z, flip, as_instance = True, offset_vector = Vector((0,1,0))):

        col = self.get_asset_collection(context, "PoleParts", context.scene.collection)
        partsOB = self.load_object(context, m_name, col)
        ring_items  = self.RingInfoDict[m_name].copy()

        obj = self.make_on_pole_instance(context, pole, obj_name, pos_z)

        mod_inst = self.add_GN_modifier(obj, ".GN.Pole.InstanceOffset")
        paramList = [("ParentPole" , pole),
                     ("Object"     , partsOB),
                     ("Offset"     , offset_vector),
                     ("RotateZ"    , rot_z + flip*3.14159),
                     ("As Instance", False)]
        self.set_GN_inputs(mod_inst, paramList)

        material = self.load_material(".Pole.Basic")
        mod_ring = self.add_GN_modifier(obj, ".GN.Pole.AddRingConnector")
        mod_ring.show_expanded = False
        mod_ring.show_viewport = self.telegraph_make_rings
        mod_ring.show_render   = self.telegraph_make_rings
        paramList = [("ParentPole"  , pole),
                     ("Material"    , material),
                     ("RingNum"     , ring_items[0]),
                     ("RingOffset"  , ring_items[1]),
                     ("RingInterval", ring_items[2]),
                     ("RingWidth"   , ring_items[3])]
        self.set_GN_inputs(mod_ring, paramList)

        return obj, partsOB
        

    def make_telegraph_trans(self, context, pole, pos_z, rot_z, flip):
    
        m_name = self.TransDict[self.telegraph_trans_type]
        if ("LOWER" in self.telegraph_trans_pos):
            pos_z -= 1

        objTrans, partsOB = self.make_telegraph_instance_appendix(context, pole, m_name, "Trans", pos_z, rot_z, flip, as_instance = False)

        tgt_obj = objTrans
        #if (context.preferences.addons[addon_name].preferences.modify_instancee == True):
        #    tgt_obj = partsOB

        if (self.telegraph_trans_pos == "LOWER2"):
            mod = self.add_GN_modifier(tgt_obj, ".GN.Pole.PartsOffset")
            offset_vector = Vector((0, 0, 1))
            paramList = [("Realize Instance", True),
                         ("Name", "EXTEND"),
                         ("Offset", offset_vector)]
            self.set_GN_inputs(mod, paramList)
            
            #self.add_displace_modifier(tgt_obj, "EXTEND", "Z", 1)
            
    def make_lamp(self, context, pole, rot_z, flip):

        m_name      = self.LampDict[self.telegraph_lamp_type]
        pos_z = 3.5 + self.telegraph_lamp_offset

        objLamp, partsOB = self.make_telegraph_instance_appendix(context, pole, m_name, "Lamp", pos_z, rot_z, flip)

    def set_GN_for_sign(self, obj, yscale = 0, tex_index = 1, invert = False,
                        basecolor  = Vector((1,1,1,1)),
                        texcolor   = Vector((0.082, 0.153, 1, 1)),
                        extracolor = Vector((1,1,1,1)),
                        extra = 0,
                        ):
        
        mod = self.add_GN_modifier(obj, ".GN.Pole.BaseColor")
        paramList = paramList = [("BaseColor" , basecolor),
                                 ("TexColor"  , texcolor),
                                 ("ExtraColor", extracolor),
                                 ("Extra"     , extra)]
        self.set_GN_inputs(mod, paramList)

        if (yscale != 0):
            mod = self.add_GN_modifier(obj, ".GN.Pole.BorderYScale")
            paramList = [("YScale", yscale)]
            self.set_GN_inputs(mod, paramList)

        mod = self.add_GN_modifier(obj, ".GN.Pole.SignTexID")
        use_texture = True
        if tex_index == 0: 
            use_texture = False
        paramList = [("UseTexture", use_texture),
                     ("UDIM_ID", tex_index),
                     ("scale", 1.0),
                     ("Invert", invert)]
        self.set_GN_inputs(mod, paramList)

        mod = self.add_GN_modifier(obj, '.GN.Pole.ModifyHSV')
        paramList = [("Hue", 0.5),
                     ("Saturate", 1.0),
                     ("Value", 1.0)]
        self.set_GN_inputs(mod, paramList)
        

    def set_guidesign_large_modifier(self, objBoard):

        yscale = {"2.4x2.0m":0.915,
                  "2.8x2.4m":0.930,
                  "2.8x2.2m":0.893}[self.guidesign_large_type]

        self.set_GN_for_sign(objBoard, yscale = yscale, tex_index = self.guidesign_tex_index)

    def get_obj_arrow_signal(self, context, objBase, type, subtype):

        signal_type = self.traffic_light_arrow_signal_type

        m_name = self.TrafficLightArrowDict[signal_type]
        location = Vector(self.TrafficLightDict_ItemPos[subtype][0])

        if (type == "FOR_CARS"):
            if (self.traffic_light_mirror == True):
                m_name += ".mir"
                if ('SINGLE' in signal_type):
                    location += Vector((0, 0.9534, 0))
                    
            objArrowA = self.make_obj(context, m_name)
            objArrowA.location = location
            objArrowA.parent = objBase

            objArrowB = None
            
        if (type == "FOR_CARS_DOUBLE"):

            objArrowA = self.make_obj(context, m_name)
            objArrowA.location = location
            objArrowA.parent = objBase

            objArrowB = self.make_obj(context, m_name + ".mir")
            if ('SINGLE' in signal_type):
                location += Vector((0, 0.9534, 0))
                location *= Vector((-1, 1, 1))
                
            objArrowB.location = location
            objArrowB.parent = objBase

        return objArrowA, objArrowB

    def add_head_board(self, context, objBase, subtype, extend):

        head_board_type = self.traffic_light_head_board_type

        m_name = self.TrafficLightHeadBoardDict[head_board_type]
        location = Vector(self.TrafficLightDict_ItemPos[subtype][1])
        location.y += extend#self.traffic_light_extend

        if (self.traffic_light_mirror == True):
            m_name += ".mir"

        objBoard = self.make_obj(context, m_name)
        objBoard.location = location
        objBoard.parent = objBase
        
        if (self.traffic_light_head_board_offset != 0):
            self.add_displace_modifier(objBoard, "EXTEND", "Z", self.traffic_light_head_board_offset)

        yscale = {"TYPEA":0.710,
                  "TYPEB":0.853}[head_board_type]

        self.set_GN_for_sign(objBoard, yscale = yscale, tex_index = self.traffic_light_head_board_tex_index, invert = True)

    def add_side_board(self, context, objBase, subtype, extend):

        side_board_type = self.traffic_light_side_board_type

        if (self.traffic_light_side_board_double == True):
            side_board_type += "_DOUBLE"

        m_name = self.TrafficLightSideBoardDict[side_board_type]
        location = Vector(self.TrafficLightDict_ItemPos[subtype][2])
        location.y += extend

        if (self.traffic_light_side_board_double == False):
            if (self.traffic_light_mirror == True):
                m_name += ".mir"

        if ("TYPEC" in side_board_type):
            location.y -= 0.2
       
        objBoard = self.make_obj(context, m_name)
        objBoard.location = location
        objBoard.parent = objBase

        yscale = {"TYPEA":0.710, "TYPEA_DOUBLE":0.710,
                  "TYPEB":0.815, "TYPEB_DOUBLE":0.815,
                  "TYPEC":0.990, "TYPEC_DOUBLE":0.990}[side_board_type]
        invert = {"TYPEA":True , "TYPEA_DOUBLE":True,
                  "TYPEB":True , "TYPEB_DOUBLE":True,
                  "TYPEC":False, "TYPEC_DOUBLE":False }[side_board_type]

        self.set_GN_for_sign(objBoard, yscale = yscale, tex_index = self.traffic_light_side_board_tex_index, invert = invert)

    def add_side_board_walker(self, context, objBase, subtype, style):

        
        m_name = {"TYPEA":self.TrafficLightSideBoardWalkerDictA[subtype],
                  "TYPEB":self.TrafficLightSideBoardWalkerDictB[subtype],
                  "TYPEC":self.TrafficLightSideBoardWalkerDictB[subtype]}[style]
                  
        objBoard = self.make_obj(context, m_name)
        #objBoard.rotation_euler.z = -3.14159/2
        if (self.traffic_light_mirror == True):
            objBoard.rotation_euler.z = +3.14159

        objBoard.location.x = self.traffic_light_extend
        objBoard.parent = objBase

        self.set_GN_for_sign(objBoard, yscale = 1, tex_index = self.traffic_light_side_board_tex_index, invert = True)
        
        return objBoard

    def draw_traffic_light_car(self, layout):
        col, row = self.newbox(layout)
        row.prop(self, "traffic_light_arrow_signal")
        if (self.traffic_light_arrow_signal == True):
            row.prop(self, "traffic_light_arrow_signal_type", text="")
            
        col, row = self.newbox(layout)
        row.prop(self, "traffic_light_head_board")
        if (self.traffic_light_head_board == True):
            row.prop(self, "traffic_light_head_board_type", text="")
            col.prop(self, "traffic_light_head_board_offset")
            col.prop(self, "traffic_light_head_board_tex_index")

        row = col.row()
        row.prop(self, "traffic_light_side_board")
        if (self.traffic_light_side_board == True):
            row.prop(self, "traffic_light_side_board_type", text="")
            col.prop(self, "traffic_light_side_board_double")
            col.prop(self, "traffic_light_side_board_tex_index")

    def make_traffic_light_full(self, context, pole, type, offset, rot_z, flip):

        style = self.traffic_light_style
        
            

        if (type == "FOR_CARS"):
            subtype = self.traffic_light_subtype
            m_name = self.TrafficLightDict[subtype]
            if (self.traffic_light_mirror == True):
                m_name += ".mir"
            base_z = 5.5
                        
        if (type == "FOR_CARS_DOUBLE"):
            subtype = self.traffic_light_subtype
            m_name = self.TrafficLightDoubleDict[subtype]
            if (self.traffic_light_mirror == True):
                m_name += ".mir"
            base_z = 5.5

        
        if (type == "FOR_WALKERS"):
            subtype = self.traffic_light_walker_subtype
            m_name = {"TYPEA":self.TrafficLightWalkerDictA[subtype],
                      "TYPEB":self.TrafficLightWalkerDictB[subtype],
                      "TYPEC":self.TrafficLightWalkerDictB[subtype],}[style]
            base_z = 3.0
        
        pos_z = base_z + offset

        if type in ["FOR_CARS", "FOR_CARS_DOUBLE"]:
            items = {"TYPEA": self.TrafficLightRingDict_FullA[subtype],
                     "TYPEB": self.TrafficLightRingDict_FullB[subtype],
                     "TYPEC": self.TrafficLightRingDict_FullC[subtype]}[style]
            rot_z_body = 0
            
        if type in ["FOR_WALKERS"]:
            items = {"TYPEA":self.TrafficLightWalkerRingDict_FullA[subtype],
                     "TYPEB":self.TrafficLightWalkerRingDict_FullB[subtype],
                     "TYPEC":self.TrafficLightWalkerRingDict_FullC[subtype]}[style]

            if (self.traffic_light_support == True):
                support_part = {"TYPEA":"Full.A.D",
                                "TYPEB":"Full.B.F",
                                "TYPEC":"Full.C.F"}[style]
                items = (items[0] + (self.traffic_light_extend*0.6 + 0.2,),
                         items[1] + (support_part,))
                
            rot_z_body = -3.1415/2

        objLight = self.make_obj(context, m_name)
        objLight.parent = pole
        objLight.location.z = pos_z
        objLight.rotation_euler.z = rot_z

        if type in ["FOR_CARS", "FOR_CARS_DOUBLE"]:
            mod_traffic = self.add_GN_modifier(objLight, ".GN.TrafficLight")
            objLight.poleprops.pole_type = "TRAFFICLIGHT"
            objLight.poleprops.pole_subtype = type
            objLight.poleprops.pole_subsubtype = subtype

        if type in ["FOR_WALKERS"]:
            mod_traffic = self.add_GN_modifier(objLight, ".GN.Pole.CrossingLight")
            objLight.poleprops.poleprops = "CROSSINGLIGHT"

        extend = self.traffic_light_extend
        #self.add_displace_modifier(objLight, "EXTEND", ext_axis, extend)
        mod = self.add_GN_modifier(objLight, ".GN.Pole.AdjustParts")
        paramList = [("Offset", extend),]
        self.set_GN_inputs(mod, paramList)

        if type in ["FOR_WALKERS"]:
            #self.add_displace_modifier(objLight, "EXTEND2", "Z", extend*0.6)
            if (self.traffic_light_support == False):
                self.add_mask_modifier(objLight, "SUPPORT")
            if (self.traffic_light_speaker == False):
                self.add_mask_modifier(objLight, "SPEAKER")
        
        col = self.get_asset_collection(context, "PoleParts", context.scene.collection)
        
        for offset, connectorType in zip(items[0], items[1]):

            if connectorType != None:
                connector_name = "Connector" + connectorType
                connector   = self.load_object(context, connector_name, col)
            else:
                connector = None

            mod_connect = self.add_GN_modifier(objLight, ".GN.Pole.AddConnector.Full")
            paramList = [("ParentPole"  , pole),
                         ("Connector"   , connector),
                         ("BaseValue"   , 0.110),
                         ("RingOffset"  , offset),
                         ("ConnectorRot", rot_z_body),]
            self.set_GN_inputs(mod_connect, paramList)

        if (self.traffic_light_mirror == True and type in ["FOR_WALKERS"]):
            mod_mir = self.add_GN_modifier(objLight, ".GN.Pole.Mir.CrossingLigt")
            
        if (flip == True):
            objLight.rotation_euler.z += 3.14159

        if (self.traffic_light_arrow_signal == True):
            if type in ["FOR_CARS", "FOR_CARS_DOUBLE"]:
                self.add_arrow_signal(context, objLight, type, subtype, ext_axis, extend)
                
        if (self.traffic_light_head_board == True):
            if type in ["FOR_CARS", "FOR_CARS_DOUBLE"]:
                self.add_head_board(context, objLight, subtype, extend)

        if (self.traffic_light_side_board == True):
            if type in ["FOR_CARS", "FOR_CARS_DOUBLE"]:
                self.add_side_board(context, objLight, subtype, extend)
            if type in ["FOR_WALKERS"]:
                self.add_side_board_walker(context, objLight, subtype, style) 
                
        return m_name

    def add_arrow_signal(self, context, obj, type, subtype, ext_axis, extend):
        arrowLightA, arrowLightB = self.get_obj_arrow_signal(context, obj, type, subtype)
        
        if (arrowLightA != None):
            arrowLightA.location.y += extend
            self.add_GN_modifier(arrowLightA, ".GN.TrafficArrow")

        if (arrowLightB != None):
            arrowLightB.location.y += extend
            self.add_GN_modifier(arrowLightB, ".GN.TrafficArrow")
        

class MESH_OT_PoleMaker(Operator, PoleMakerCommons, AppendixCommon, curve_module.CurveModule, type_module.TypeModule):
    
    bl_idname = "mesh.make_pole"
    bl_label = "Pole Maker"
    bl_description = "Description"
    bl_options = {'REGISTER', 'UNDO'}

    CommonTypeSelection = type_module.TypeModule.CommonTypeSelection
    CommonTypeSelectionWithNone = type_module.TypeModule.CommonTypeSelectionWithNone
    ActionType = [("TELEGRAPH"  , "Telegraph"     , "", "CUSTOM_TELEGRAPH", 1),
                  ("SIGNALPOLE" , "Signal Pole"   , "", "CUSTOM_SIGNALPOLE", 2),
                  ("STREETLIGHT", "Street Light"  , "", "LIGHT", 3),
                  ("SIGNBOARDPOLE", "Sign Board Pole", "", "AUTO", 4),
                  ("CURVEMIRROR", "Curve Mirror"  , "", "PIVOT_ACTIVE", 5),
                  ("CURVEPOLE"  , "Curve Pole"    , "", "PLAY_REVERSE", 6),
                  ("GUIDESIGN"  , "Guide Sign"    , "", "CUSTOM_GUIDESIGN", 7),
                  ("POLESIGN"   , "Polesign"      , "", "CUSTOM_POLESIGN", 8),
                  ("SINGLE"     , "Single"        , "", "", 9)]

    def EnumCustomIcon_ActionType(self, context):
        return PoleMakerCommons.EnumCustomIconBase(MESH_OT_PoleMaker.ActionType)
    

    TelegraphType = [("8.3m", "8.3m", "10m (8.3m + 1.7m)", "", 1),
                     ("10.0m", "10.0m", "12m (10.0m + 2.0m)", "", 2),
                     ("11.6m", "11.6m", "14m (11.6m + 2.4m)", "", 3),
                     ("13.5m", "13.5m", "16m (13.5m + 2.5m)", "", 4),
                     ("13.5m(b)", "13.5m(b)", "16m (13.5m + 2.5m)", "", 5),
                     ("7.5m"    , "7.5m" , "9m (7.5m+1.5m)"   , "", 1),
                     ("7.5m(b)" , "7.5m" , "9m (7.5m+1.5m)"   , "", 3)]

    SignalpoleType = [("7.5m"    , "7.5m" , "9m (7.5m+1.5m)"   , "", 1),
                      ("8.3m"    , "8.3m" , "10m (8.33m+1.67m)", "", 2),
                      ("7.5m(b)" , "7.5m" , "9m (7.5m+1.5m)"   , "", 3),]
    

    # top, bottom, btm_radi, top_radi
    TelegraphDict = {'8.3m':(8.3, -1.7, 0.323/2, 0.19/2),
                     '10.0m':(10, -2, 0.35/2, 0.19/2),
                     '11.6m':(11.6, -2.4, 0.377/2, 0.19/2),
                     '13.5m':(13.5, -2.5, 0.403/2, 0.19/2),
                     '13.5m(b)':(13.5, -2.5, 0.453/2, 0.24/2),}

    SignalpoleDict = {'7.5m':(7.5, -1.5, 0.256/2, 0.200/2),
                      '8.3m':(8.3, -1.7, 0.263/2, 0.200/2),
                      '7.5m(b)':(7.5, -1.5, 0.165/2, 0.165/2),}

    GuidesignPoleDict = {"SMALL_TYPEA":(3.0, -0.3, 0.05, 0.05),
                         "SMALL_TYPEB":(3.0, -0.3, 0.05, 0.05),
                         "SMALL_TYPEC":(2.2, -0.3, 0.03, 0.03),
                         "LARGE_TYPEA":(8.0, -1.0, 0.15, 0.15),
                         "LARGE_TYPEB":(8.0, -1.0, 0.15, 0.15),}

    CurveMirrorConnectorDict = {'A':"MirrorConnector.0.6m",
                                'B':"MirrorConnector.0.8m",
                                'C':"MirrorConnector.1.0m",
                                'D':"MirrorConnector.SQ.0.6m",
                                'E':"MirrorConnector.SQ.0.8m",
                                'F':"MirrorConnector.SQ.H.0.6m",
                                'G':"MirrorConnector.SQ.H.0.8m",
                                'H':"MirrorPole.A",
                                'I':"MirrorPole.B"}

    CurvePoleDict = {'A':"CurvePole.L.A",
                     'B':"CurvePole.R.A",
                     'C':"CurvePole.L.B",
                     'D':"CurvePole.R.B",}
                                                                
    GuideSignLArmDict = {"TYPEA":"GuideSign.L.Arm.x2",
                         "TYPEB":"GuideSign.L.Arm.x2.Dual"}

    GuideSignLArmDict2 = {'A':"GuideSign.L.Arm.0.3m.A",
                          'B':"GuideSign.L.Arm.0.2m.A",
                          'C':"GuideSign.L.Arm.0.2m.B",}

    GuideSignLDict = {"TYPEA":"GuideSign.L.2.4x2.0m",
                      "TYPEB":"GuideSign.L.2.8x2.2m",
                      "TYPEC":"GuideSign.L.2.8x2.4m",}

    GuideSignSDict = {"TYPEA":"GuideSign.S.A",
                      "TYPEB":"GuideSign.S.B",
                      "TYPEC":"GuideSign.S.C",}

    AdPoleDict = {"CIRCLE":"Ad.Pole.Cylinder.10cm",
                  "SQUARE":"Ad.Pole.Box.10cm"}
                     
    CapDict = {'TYPEA': "Cap.A",
               'TYPEB': "Cap.B",}
    
    MirrorSizeType = [("0.6m", "0.6m", "0.6m", "", 1),
                      ("0.8m", "0.8m", "0.8m", "", 2),
                      ("1.0m", "1.0m", "1.0m", "", 3)]

    MirrorType = [("CIRCLE", "Circle", "Circle", "MESH_CIRCLE", 1),
                  ("SQUARE", "Square", "Square", "MESH_PLANE", 2)]

   
    StreetLightCategoryType = [("SIDEWALK", "Sidewalk", "Sidewalk", "", 1),
                               ("DRIVEWAY", "Driveway", "Driveway", "", 2),
                               ("PARK", "Park", "Park", "", 3),
                               ("FOOTLIGHT", "Foot Light", "Foot Light", "", 4),
                               ("GARDENLIGHT", "Garden Light", "Garden Light", "", 5),]

    StreetLightSidewalkDict = {'TYPEA':'StreetLight.A',
                               'TYPEB':'StreetLight.B',
                               'TYPEC':'StreetLight.C',
                               'TYPED':'StreetLight.D',}

    StreetLightDrivewayDict = {'TYPEA':'StreetLight.L.A',
                               'TYPEB':'StreetLight.L.B',
                               'TYPEC':'StreetLight.L.C',
                               'TYPED':'StreetLight.L.D',}

    StreetLightParkDict = {'TYPEA':'StreetLight.M.A',
                           'TYPEB':'StreetLight.M.B',
                           'TYPEC':'StreetLight.M.C',
                           'TYPED':'StreetLight.M.D',}
    
    StreetLightFootDict = {'TYPEA':'StreetLight.F.A',
                           'TYPEB':'StreetLight.F.B',
                           'TYPEC':'StreetLight.F.C',
                           'TYPED':'StreetLight.F.D',}

    StreetLightGuardenDict = {'TYPEA':'StreetLight.G.A',
                              'TYPEB':'StreetLight.G.B',
                              'TYPEC':'StreetLight.G.C',
                              'TYPED':'StreetLight.G.D',}

    GuideSignLargeType = [("2.8x2.4m", "2.8x2.4m", "2.8x2.4m", "", 1),
                         ("2.8x2.2m", "2.8x2.2m", "2.8x2.2m", "", 2),
                         ("2.4x2.0m", "2.4x2.0m", "2.4x2.0m", "", 3)]
    
                                                      
    PolesignDict = {'SQUARETYPEA2m':"Add.Box2m.A",
                    'SQUARETYPEB2m':"Add.Box2m.B",
                    'SQUARETYPEC2m':"Add.Box2m.C",
                    'SQUARETYPEA3m':"Add.Box3m.A",
                    'SQUARETYPEB3m':"Add.Box3m.B",
                    'SQUARETYPEC3m':"Add.Box3m.C",
                    'CIRCLETYPEA2m':"Add.Circle2m.A",
                    'CIRCLETYPEB2m':"Add.Circle2m.B",
                    'CIRCLETYPEC2m':"Add.Circle2m.C",
                    'CIRCLETYPEA3m':"Add.Circle3m.A",
                    'CIRCLETYPEB3m':"Add.Circle3m.B",
                    'CIRCLETYPEC3m':"Add.Circle3m.C"}

    def StreetLight_callback(self, context):
        type = {"SIDEWALK":"StreetLightSideWalkType",
                "DRIVEWAY":"StreetLightDriveWayType",
                "PARK":"StreetLightParkType",
                "FOOTLIGHT":"StreetLightFootLightType",
                "GARDENLIGHT":"StreetLightGardenType",}[self.streetlight_category]
        return common_type_callback(4, type)

    def CurveMirrorBoard_callback(self, context):
        return common_type_callback(3, "CurveMirrorBoardType")

    def CurvePole_callback(self, context):
        return common_type_callback(2, "CurvePoleType")

    def CurveMirrorPole_callback(self, context):
        return common_type_callback(4, "CurveMirrorPoleType")

    def PolesignAd_callback(self, context):
        type = {"CIRCLE":"AdCircleType",
                "SQUARE":"AdSquareType",}[self.polesign_ad_category]
        return common_type_callback(3, type)

    def PolesignLight_callback(self, context):
        return common_type_callback(2, "AdLightType")

    def GuideSignSmall_callback(self, context):
        return common_type_callback(3, "GuidesignSmallType")

    def GuideSignLarge_callback(self, context):
        return common_type_callback(2, "GuidesignLargeType")

    action : EnumProperty(
        name = "Action Type", default = 1,
        items = EnumCustomIcon_ActionType)

    single_mode : BoolProperty(
        name = "Single Mode", default = False)

    telegraph_type : EnumProperty(
        name = "Type", default = "10.0m",
        items = TelegraphType[:5])

    telegraph_arm_HV : BoolProperty(
        name = "Arm HV", default = True)

    telegraph_trans_interval : IntProperty(
        name = "Interval", default = 2,
        min = 1)

    telegraph_trans_offset : IntProperty(
        name = "Offset", default = 0,
        min = 0)

    telegraph_cap : BoolProperty(
        name = "Cap", default = False)

    telegraph_cap_type : EnumProperty(
        name = "Type", default = "TYPEA",
        items = CommonTypeSelection[0:2])

    telegraph_arm_HV_flip : BoolProperty(
        name = "Flip", default = False)

    use_collection : BoolProperty(
        name = "Use Collection", default = True)

    resolution : IntProperty(
        name = 'Resolution', default = 4,
        min = 2, max = 6)

    telegraph_arm_LV0 : BoolProperty(
        name = "Arm LV", default = True)

    telegraph_arm_LV1 : BoolProperty(
        name = "Arm LV", default = False)

    telegraph_arm_com : BoolProperty(
        name = "Arm Com", default = True)

    telegraph_lamp : BoolProperty(
        name = "Lamp", default = False)

    telegraph_lamp_flip : BoolProperty(
        name = "Flip", default = False)

    telegraph_arm_LV0_offset : FloatProperty(
        name = "Offset", default = 0,
        min = -2, max = 2, step = 0.3) #範囲が違うので別定義

    telegraph_arm_LV0_flip : BoolProperty(
        name = "Flip", default = False)

    telegraph_arm_LV1_flip : BoolProperty(
        name = "Flip", default = False)

    telegraph_arm_com_flip : BoolProperty(
        name = "Flip", default = False)

    telegraph_auto_connect : BoolProperty(
        name = "Auto Connect", default = False)

    telegraph_com_spiral : BoolProperty(
        name = "Use Spiral", default = False)

    telegraph_com_box : BoolProperty(
        name = "Set Box", default = False)

    telegraph_catenary_a_inv : FloatProperty(
        name = "a_inv", default = 1,
        min = 0.2, max = 2000)

    signalpole_type : EnumProperty(
        name = "Type", default = "7.5m",
        items = SignalpoleType)

    traffic_light_car_type : EnumProperty(
        name = "Type", default = "FOR_CARS",
        items = PoleMakerCommons.TrafficLightType[:2])

    traffic_light_subtype : EnumProperty(
        name = "Type", default = 1,
        items = PoleMakerCommons.TrafficLightSubtype_callback)

    traffic_light_offset : FloatProperty(
        name = "Offset", default = 0,
        min = -2, max = +2, step = 0.2)

    traffic_light_rotation : FloatProperty(
        name = "Rotation", default = 0, subtype = "ANGLE", 
        min = -3.14159/2, max = 3.14159/2, step = 10)

    traffic_light_flip : BoolProperty(
        name = "Flip", default = False)

    pole_flip : BoolProperty(
        name = "Flip", default = False)

    pole_random_seed : IntProperty(
        name = "Seed", default = 0)

    pole_randomize : FloatProperty(
        name = "Random", default = 0,
        min = 0, max = 1, step = 0.3)

    pole_single_rot : FloatProperty(
        name = "Rotation", subtype = "ANGLE", default = 0,
        min = -3.14159, max = 3.14159, step = 10)

    radius_factor : FloatProperty(
        name = "Radius Factor", default = 1,
        min = 0.5, max = 2)

    streetlight_category : EnumProperty(
        name = "Category", default = "SIDEWALK",
        items = StreetLightCategoryType)

    streetlight_type : EnumProperty(
        name = "Type", default = 1,
        items = StreetLight_callback)

    streetlight_extend : FloatProperty(
        name = "Extend", default = 0,
        min = -1, max = 2, step = 0.3)

    #mirror_type : EnumProperty(
    #    name = "Type", default = "CIRCLE",
    #    items = MirrorType)

    mirror_type : EnumProperty(
        name = "Type", default = 1,
        items = CurveMirrorPole_callback)

    mirror_size_type : EnumProperty(
        name = "Size", default = "0.8m",
        items = MirrorSizeType)

    mirror_decline : FloatProperty(
        name = "Decline", subtype = "ANGLE", default = 0,
        min = 0, max = 3.14159 / 9, step = 5)

    mirror_extend : FloatProperty(
        name = "Extend Z", default = 0,
        min = -0.5, max = 1, step = 0.3)

    mirror_arm_extend : FloatProperty(
        name = "Extend Arm", default = 0,
        min = -0.05, max = 0.2, step = 0.1)

    #mirror_dual : BoolProperty(
    #    name = "Dual Mirror", default = False)

    mirror_board_type : EnumProperty(
        name = "Board", default = 2,
        items = CurveMirrorBoard_callback)

    mirror_board_tex_id : IntProperty(
        name = "Tex ID", default = 1,
        min = 0, max = 99)
    
    #mirror_board : BoolProperty(
    #    name = "Board", default = True)

    curvepole_type : EnumProperty(
        name = "Type", default = 1,
        items = CurvePole_callback)

    curvepole_direction : EnumProperty(
        name = "Direction", default = "LEFT",
        items = [("LEFT", "Left", "Left", "", 1),
                 ("RIGHT", "Right", "Right", "", 2)])

    telegraph_footage_bolts : BoolProperty(
        name = "Footage Bolts", default = False)

    telegraph_footage_min_height : FloatProperty(
        name = "Min", default = 2,
        min = 1.5, max = 4.5, step = 0.3)

    signboardpole_size : EnumProperty(
        name = "Diameter", default = "0.04m",
        items = [("0.04m", "4cm", "4cm", "", 1),
                 ("0.06m", "6cm", "6cm", "", 2),
                 ("0.2m", "20cm", "20cm", "", 3),
                 ("0.3m", "30cm", "30cm", "", 4)])

    signboardpole_arm_type : EnumProperty(
        name = "Arm", default = "TYPEA",
        items = CommonTypeSelection[0:2])

    signboardpole_layout : EnumProperty(
        name = "Layout", default = "OFFSET",
        items = PoleMakerCommons.SignboardLayoutType)

    signboardpole_number : IntProperty(
        name = "Number", default = 1,
        min = 0, max = 3)

    signboardpole_square_type_0 : EnumProperty(
        name = "Square Type", default = "TYPEA",
        items = CommonTypeSelection[0:3])
    
    signboardpole_square_type_1 : EnumProperty(
        name = "Square Type", default = "TYPEA",
        items = CommonTypeSelection[0:3])

    signboardpole_square_type_2 : EnumProperty(
        name = "Square Type", default = "TYPEA",
        items = CommonTypeSelection[0:3])

    signboardpole_signtype_0 : EnumProperty(
        name = "Type", default = 2,
        items = PoleMakerCommons.SignboardPoleSignType_callback)

    signboardpole_signtype_1 : EnumProperty(
        name = "Type", default = 1,
        items = PoleMakerCommons.SignboardPoleSignType_callback)
    
    signboardpole_signtype_2 : EnumProperty(
        name = "Type", default = 1,
        items = PoleMakerCommons.SignboardPoleSignType_callback)

    signboardpole_signsize_0 : EnumProperty(
        name = "Size", default = "0.4m",
        items = PoleMakerCommons.SignboardSizeType[0:2])

    signboardpole_signsize_1 : EnumProperty(
        name = "Size", default = "0.4m",
        items = PoleMakerCommons.SignboardSizeType[0:2])

    signboardpole_signsize_2 : EnumProperty(
        name = "Size", default = "0.4m",
        items = PoleMakerCommons.SignboardSizeType[0:2])

    signboardpole_signsizeH_0 : EnumProperty(
        name = "Size", default = "0.9m",
        items = PoleMakerCommons.SignboardSizeType[2:4])

    signboardpole_signsizeH_1 : EnumProperty(
        name = "Size", default = "0.9m",
        items = PoleMakerCommons.SignboardSizeType[2:4])

    signboardpole_signsizeH_2 : EnumProperty(
        name = "Size", default = "0.9m",
        items = PoleMakerCommons.SignboardSizeType[2:4])

    signboardpole_extend : FloatProperty(
        name = "Extend", default = 0,
        min = -1, max = 2, step = 0.3)

    signboardpole_mirror : BoolProperty(
        name = "Mirror", default = False)

    guidesign_pole_size : EnumProperty(
        name = "Size", default = "LARGE",
        items = [("SMALL", "Small", "Small", "", 1),
                 ("LARGE", "Large", "Large", "", 2)])

    guidesign_pole_small_type : EnumProperty(
        name = "Type", default = 1,
        items = GuideSignSmall_callback)

    guidesign_pole_large_type : EnumProperty(
        name = "Type", default = 1,
        items = GuideSignLarge_callback)

    guidesign_mirror : BoolProperty(
        name = "Mirror", default = False)

    guidesign_extend_z : FloatProperty(
        name = "Extend Z", default = 0,
        min = -1, max = 2, step = 0.3)

    guidesign_extend_arm : FloatProperty(
        name = "Extend", default = 0,
        min = -0.5, max = 3, step = 0.3)

    guidesign_number : IntProperty(
        name = "Number", default = 1,
        min = 1, max = 3)

    guidesign_small_rot_0 : FloatProperty(
        name = "Rotation", subtype = "ANGLE", default = 0,
        min = -3.14159, max = 3.14159, step = 10)

    guidesign_small_rot_1 : FloatProperty(
        name = "Rotation", subtype = "ANGLE", default = 3.14159/2,
        min = -3.14159, max = 3.14159, step = 10)

    guidesign_small_rot_2 : FloatProperty(
        name = "Rotation", subtype = "ANGLE", default = 3.14159/4,
        min = -3.14159, max = 3.14159, step = 10)

    polesign_pole_size : FloatProperty(
        name = "Size", default = 4,
        min = 4, max = 10)

    polesign_pole_type : EnumProperty(
        name = "PoleType", default = "CIRCLE",
        items = [("CIRCLE", "Circle", "Circle", "MESH_CIRCLE", 1),
                 ("SQUARE", "Square", "Square", "MESH_PLANE", 2)])

    polesign_ad_category : EnumProperty(
        name = "Category", default = "SQUARE",
        items = [("CIRCLE", "Circle", "Circle", "MESH_CIRCLE", 1),
                 ("SQUARE", "Square", "Square", "MESH_PLANE", 2)])

    polesign_ad_type : EnumProperty(
        name = "Type", default = 1,
        items = PolesignAd_callback)

    polesign_ad_size : EnumProperty(
        name = "Size", default = "2m",
        items = [("2m", "2m", "2m", "", 1),
                 ("3m", "3m", "3m", "", 2)])

    polesign_ad_tex_index : IntProperty(
        name = "Tex Index", default = 1,
        min = 0, max = 99)

    polesign_ad_tex_scale : FloatProperty(
        name = "Tex Scale", default = 1,
        min = 0.5, max = 1.2, step = 0.3)

    polesign_ad_tex_offsetY : FloatProperty(
        name = "Tex Offset Y", default = 0,
        min = -0.5, max = 0.5, step = 0.3)

    polesign_ad_tex_emission : FloatProperty(
        name = "Tex Emission", default = 0,
        min = 0, max = 1, step = 0.3)
    

    polesign_ad_light : BoolProperty(
        name = "Light", default = False)

    polesign_light_num : IntProperty(
        name = "Number", default = 3,
        min = 1, max = 6)

    polesign_light_type : EnumProperty(
        name = "Type", default = 1,
        items = PolesignLight_callback)

    polesign_light_extend : FloatProperty(
        name = "Extend", default = 0,
        min = 0, max = 1)

    @classmethod
    def poll(cls, context):

        obj = context.object

        if (context.mode == "OBJECT"):
            if (obj == None):
                return True
                if (obj.select_get() == True):
                    return False

        return True

    def draw(self, context):
        layout = self.layout

        layout.prop(self, "action")
        layout.separator()

        root_col = layout.column(align = True)

        box = root_col.box()
        colA = box.column(align = True)
        if (self.action == "TELEGRAPH"):

            row = colA.row()
            row.prop(self, "telegraph_type")
            row.prop(self, "telegraph_make_rings", icon='MESH_CIRCLE')
            
            col, row = self.newbox(colA)
            col.prop(self, "telegraph_arm_HV")
            if (self.telegraph_arm_HV == True):
                self.PS2(col, ["telegraph_arm_HV_type", "telegraph_arm_HV_offset", "telegraph_arm_HV_flip"])
                self.PS2(col, ["telegraph_trans_type", "#telegraph_trans_pos", "telegraph_trans_interval", "telegraph_trans_offset"])
                                
            row = col.row()
            row.prop(self, "telegraph_cap")
            if (self.telegraph_cap == True):
                row.prop(self, "telegraph_cap_type", text="")

            col, row = self.newbox(colA)
            col.prop(self, "telegraph_arm_LV0")
            if (self.telegraph_arm_LV0 == True):
                self.PS2(col, ["telegraph_arm_LV0_type", "telegraph_arm_LV0_offset", "telegraph_arm_LV0_flip"])
                
            col.prop(self, "telegraph_arm_LV1")
            if (self.telegraph_arm_LV1 == True):
                self.PS2(col, ["telegraph_arm_LV1_type", "telegraph_arm_LV1_offset", "telegraph_arm_LV1_flip"])
                
            col.prop(self, "telegraph_arm_com")
            if (self.telegraph_arm_com == True):
                self.PS2(col, ["telegraph_arm_com_type", "telegraph_arm_com_offset", "telegraph_arm_com_flip"])
                if (self.telegraph_auto_connect == True):
                    row = col.row()
                    row.prop(self, "telegraph_com_spiral")
                    row.prop(self, "telegraph_com_box")
                    
            col, row = self.newbox(colA)
            row.prop(self, "telegraph_lamp")
            if (self.telegraph_lamp == True):
                self.PS2(col, ["telegraph_lamp_type", "telegraph_lamp_offset", "telegraph_lamp_flip"])
                
            col, row = self.newbox(colA)
            row.prop(self, "telegraph_footage_bolts")
            if (self.telegraph_footage_bolts == True):
                row.prop(self, "telegraph_footage_min_height")

            row = root_col.row()
            row.prop(self, "telegraph_auto_connect")
            if (self.telegraph_auto_connect == True):
                row.prop(self, "telegraph_catenary_a_inv")

        if (self.action == "SIGNALPOLE"):
            row = colA.row()
            row.prop(self, "signalpole_type")
            row.prop(self, "telegraph_make_rings", icon='MESH_CIRCLE')

            colA.prop(self, "traffic_light_car_type")
            colA.prop(self, "traffic_light_style")
            self.PS2(colA, ["traffic_light_subtype", "traffic_light_offset", "traffic_light_extend", "traffic_light_rotation"])
            row = colA.row()
            row.prop(self, "traffic_light_flip")
            row.prop(self, "traffic_light_mirror")
            
            self.draw_traffic_light_car(colA)
                            
        if (self.action == 'STREETLIGHT'):
            colA.prop(self, "streetlight_category")
            self.PS2(colA, ["streetlight_type", "streetlight_extend"])

        if (self.action == 'SIGNBOARDPOLE'):

            colA.prop(self, "signboardpole_size")
            if (self.signboardpole_size == "0.2m"):
                colA.prop(self, "signboardpole_arm_type")
                
            row = colA.row()
            row.prop(self, "signboardpole_layout", text="")
            row.prop(self, "signboardpole_number")

            for i in range(0,3):
                idx = str(i)
                if (self.signboardpole_number >= i+1):
                    proplist = ["signboardpole_signtype_"+idx]
                    
                    if ({0:self.signboardpole_signtype_0,
                         1:self.signboardpole_signtype_1,
                         2:self.signboardpole_signtype_2}[i]== "SQUARE"):
                        proplist.append("#signboardpole_square_type_"+idx)
                        
                    if self.signboardpole_size in ["0.2m", "0.3m"]:
                        proplist.append("signboardpole_signsizeH_"+idx)
                    else:
                        proplist.append("signboardpole_signsize_"+idx)

                    self.PS2(colA, proplist)
                        
            row = colA.row()
            row.prop(self, "signboardpole_extend")
            if self.signboardpole_size in ["0.2m", "0.3m"]:
                row.prop(self, "guidesign_extend_arm", text="Arm")

                row = colA.row()
                row.prop(self, "signboardpole_mirror")

        if (self.action == 'CURVEMIRROR'):
            self.PS2(colA, ["mirror_type", "mirror_decline", "mirror_extend", "mirror_arm_extend"])
            row = colA.row()
            row.prop(self, "mirror_size_type", text = "")
            self.PS2(colA, ["mirror_board_type", "mirror_board_tex_id"])
            #row.prop(self, "mirror_board")

        if (self.action == 'CURVEPOLE'):
            self.PS2(colA, ["curvepole_type", "#curvepole_direction"])

        if (self.action == 'GUIDESIGN'):
            row = colA.row()
            row.prop(self, "guidesign_pole_size")
            if (self.guidesign_pole_size == "SMALL"):
                #row.prop(self, "guidesign_pole_small_type", text="")
                colA.prop(self, "guidesign_extend_z")
            else:
                self.PS2(colA, ["guidesign_pole_large_type", "guidesign_extend_z", "guidesign_large_type", "guidesign_tex_index"])

            if (self.guidesign_pole_size == "SMALL"):

                self.PS2(colA, ["guidesign_pole_small_type", "guidesign_number", "guidesign_tex_index"])
                colA.prop(self, "guidesign_small_rot_0")
                if (self.guidesign_number >= 2):
                    colA.prop(self, "guidesign_small_rot_1")
                if (self.guidesign_number >= 3):
                    colA.prop(self, "guidesign_small_rot_2")
                    
            if (self.guidesign_pole_size == "LARGE"):
                colA.prop(self, "guidesign_extend_arm")
                row = colA.row()
                row.prop(self, "guidesign_mirror")

        if (self.action == "POLESIGN"):
            row = colA.row()
            row.prop(self, "polesign_pole_size")
            row.prop(self, "polesign_pole_type", text="")

            #box = colA.box()
            #col = box.column(align = True)
            self.PS2(colA, ["polesign_ad_type", "#polesign_ad_category", "#polesign_ad_size", "polesign_ad_tex_index"])
            colA.prop(self, "polesign_ad_tex_scale")
            colA.prop(self, "polesign_ad_tex_offsetY")
            colA.prop(self, "polesign_ad_tex_emission")
                        
            #box = colA.box()
            #col = box.column(align = True)
            if (self.polesign_ad_category == "SQUARE"):
                row = colA.row()
                row.prop(self, "polesign_ad_light")
                if (self.polesign_ad_light == True):
                    self.PS2(colA, ["polesign_light_type", "polesign_light_num", "polesign_light_extend"])
                                        
        row = root_col.row()
        row.prop(self, "pole_flip")
        row.prop(self, "use_collection")
        
        row = root_col.row()
        row.prop(self, "pole_randomize")
        row.prop(self, "pole_random_seed")

        row = root_col.row()
        if (self.single_mode == True):
            row.prop(self, "pole_single_rot")

        if (self.action == "TELEGRAPH"):
            row.prop(self, "resolution")
            
    def get_material(self, object):
        if (object == None):
            return None
        
        material = None
        if len(object.material_slots) != 0:
            material = object.material_slots[0].material
            
        return material

    def make_telegraph_cap(self, context, pole, type, pos_z):

        m_name = self.CapDict[type]
        #self.load_mesh(m_name)

        offset = 0
        if (type == 'TYPEB'):
            offset = 1

        objCap = self.make_on_pole(context, pole, m_name, pos_z, offset)
        objCap.poleprops.pole_type = "TELEGRAPH_ARM"
        objCap.poleprops.pole_subtype = "GW"
        return m_name

    def calc_rot_euler(self, v, v_list):
        e_num = len(v.link_edges)

        dpos = None
        if (e_num == 1):
            
            v_tgt = v.link_edges[0].verts[0]
            if (v == v_tgt):
                v_tgt = v.link_edges[0].verts[1]

            factor = 1
            if v_tgt in v_list:
                factor = -1
                
            dpos = (v_tgt.co - v.co) * factor

        if (e_num == 2):
            v_tgt0 = v.link_edges[0].verts[0]
            if (v == v_tgt0):
                v_tgt0 = v.link_edges[0].verts[1]
                
            v_tgt1 = v.link_edges[1].verts[0]
            if (v == v_tgt1):
                v_tgt1 = v.link_edges[1].verts[1]

            factor = 1
            if v_tgt1 in v_list:
                factor = -1
                
            dpos = (v_tgt1.co - v_tgt0.co) * factor

        rot_z = 0
        if (dpos != None):
            rot_z = atan2(dpos.y, dpos.x)

        if (self.pole_flip == True):
            rot_z += 3.14159

        if (self.single_mode == True):
            rot_z += self.pole_single_rot

        rot_x = (random.random()-0.5) * self.pole_randomize * 3.14159/8
        rot_y = (random.random()-0.5) * self.pole_randomize * 3.14159/8
        rot_euler = Vector((rot_x, rot_y, rot_z))

        return rot_euler

    def search_rim(self, bm, use_select):
        for v in bm.verts:
            if len(v.link_edges) == 1:
                if v.select == True or use_select == False:
                    return v
                
        return None

    def search_selected(self, bm):
        for v in bm.verts:
            if v.select == True:
                return v

        return None

    def get_pole_radius(self, dict, key):
        
        top, bottom, btm_radius, top_radius = dict[key]
        return btm_radius, top_radius

    def make_single_pole(self, curveData, dict, key, z_offset):

        top, bottom, btm_radius, top_radius = dict[key]
        
        top += z_offset
        co_0 = Vector((0,0,bottom))
        co_1 = Vector((0,0,top))
        polyline = self.set_poly_from3d(curveData, co_0, co_1)
        polyline.points[0].radius = btm_radius
        polyline.points[1].radius = top_radius 
        
        return polyline, top, bottom

    def make_curve_pole(self, context, name, dict, key, matrix_world, co, rot_euler, material, z_offset = 0):

        curveData = self.make_curve(name)

        polyline, top, btm = self.make_single_pole(curveData, dict, key, z_offset)
        
        curveOB = self.make_curve_object(curveData.name, curveData, material, context)

        curveOB.location = matrix_world @ co
        curveOB.rotation_euler = rot_euler
        
        return curveOB, top, btm

    def add_mod_curve_to_mesh(self, curveOB, material = None):
        
        mod_mesh = self.add_GN_modifier(curveOB, ".GN.Pole.CurveToMesh")
        paramList = [("Resolution", self.resolution_for_GN(curveOB.data.bevel_resolution)),
                     ("Material", material)]
        self.set_GN_inputs(mod_mesh, paramList)


    def exec_telegraph(self, context, matrix_world, v, rot_euler, material, count):
        
        curveOB, top, btm = self.make_curve_pole(context, "telegraph_pole", self.TelegraphDict, self.telegraph_type, matrix_world, v.co, rot_euler, material)
        
        curveOB.poleprops.pole_type = "TELEGRAPH"
        curveOB.poleprops.telegraph_type = self.telegraph_type

        if (self.telegraph_arm_HV == True):
            z = top - self.telegraph_arm_HV_offset
            _, trans_offset = self.make_telegraph_arm_HV(context, curveOB, self.telegraph_arm_HV_type, z, self.telegraph_arm_HV_flip)
            
            if (self.telegraph_trans_type != "NONE"):

                if ((count % self.telegraph_trans_interval) == (self.telegraph_trans_offset % self.telegraph_trans_interval)):
                    z = z - trans_offset

                    self.make_telegraph_trans(context, curveOB, z, 0, self.telegraph_arm_HV_flip)
                    
        if (self.telegraph_arm_LV0 == True):
            z = top-2 + self.telegraph_arm_LV0_offset
            self.make_telegraph_arm_LV(context, curveOB, self.telegraph_arm_LV0_type, z, self.telegraph_arm_LV0_flip)
                        
        if (self.telegraph_arm_LV1 == True):
            z = top-4 + self.telegraph_arm_LV1_offset
            self.make_telegraph_arm_LV(context, curveOB, self.telegraph_arm_LV1_type, z, self.telegraph_arm_LV1_flip)
                        
        if (self.telegraph_arm_com == True):
            z = 5 + self.telegraph_arm_com_offset
            self.make_telegraph_arm_com(context, curveOB, self.telegraph_arm_com_type, z, self.telegraph_arm_com_flip)
            
        if (self.telegraph_lamp == True):
            self.make_lamp(context, curveOB, 0, self.telegraph_lamp_flip)
            
        if (self.telegraph_cap == True):
            z = top
            self.make_telegraph_cap(context, curveOB, self.telegraph_cap_type, z)
            
        if (self.telegraph_footage_bolts == True):
            z = top - 1.5
            self.make_telegraph_footage(context, curveOB, self.telegraph_footage_min_height, z)
            
        ring_obj = None

        self.add_mod_curve_to_mesh(curveOB, material)
        
        return curveOB

    def exec_signalpole(self, context, matrix_world, v, rot_euler, material):
        curveOB, top, btm = self.make_curve_pole(context, "telegraph_pole", self.SignalpoleDict, self.signalpole_type, matrix_world, v.co, rot_euler, material)
        curveOB.poleprops.pole_type = "TELEGRAPH"
        curveOB.poleprops.telegraph_type = self.signalpole_type

        self.add_mod_curve_to_mesh(curveOB, material)

        self.make_traffic_light_full(context, curveOB, self.traffic_light_car_type, self.traffic_light_offset, self.traffic_light_rotation, self.traffic_light_flip)

        return curveOB
        

    def exec_streetlight(self, context, matrix_world, v, rot_euler, material):

        category = self.streetlight_category
        type = self.streetlight_type

        m_name={"SIDEWALK": self.StreetLightSidewalkDict,
                "DRIVEWAY": self.StreetLightDrivewayDict,
                "PARK":     self.StreetLightParkDict,
                "FOOTLIGHT":self.StreetLightFootDict,
                "GARDENLIGHT":self.StreetLightGuardenDict,}[category][type]

        objLight = self.make_obj(context, m_name)

        self.add_displace_modifier(objLight, "EXTEND", "Z", self.streetlight_extend)
        
        objLight.location = matrix_world @ v.co
        objLight.rotation_euler = rot_euler

        return objLight

    def exec_signboardpole_arm(self, context, curveOB, top, pole_size, material):

        if (pole_size == "0.3m"):
            m_name_arm = "GuideSign.L.Arm.0.3m.A"
            pos_z = top-0.5
        if (pole_size == "0.2m"):
            m_name_arm = "GuideSign.L.Arm.0.2m." + PoleMakerCommons.TypeDict[self.signboardpole_arm_type]
            pos_z = top+0.3
            
        objArm = self.make_on_pole(context, curveOB, m_name_arm, pos_z, 0)
        
        extend_arm = self.guidesign_extend_arm
        self.add_displace_modifier(objArm, "EXTEND", "Y", -extend_arm)

        objArm.poleprops.pole_type = "SIGNBOARD_ARM"
        
        sign_list = []
        size_list = []
        
        if (self.signboardpole_number >= 1):
            sign_list.append(self.signboardpole_signtype_0)
            size_list.append(self.signboardpole_signsizeH_0)

        if (self.signboardpole_number >= 2):
            sign_list.append(self.signboardpole_signtype_1)
            size_list.append(self.signboardpole_signsizeH_1)

        if (self.signboardpole_number >= 3):
            sign_list.append(self.signboardpole_signtype_2)
            size_list.append(self.signboardpole_signsizeH_2)

        pos_y = -2
        
        for i, (sign, str_size) in enumerate(zip(sign_list, size_list)):

            if sign == "CIRCLE":
                signboard_type = "CH" + str_size + "_Circle"

            if sign == "TRIANGLEA":
                signboard_type = "CH" + str_size + "_TriangleA"

            if sign == "TRIANGLEB":
                signboard_type = "CH" + str_size + "_TriangleB"

            if sign == "RHOMBUS":
                signboard_type = "CH" + str_size + "_Rhombus"

            if sign == "SQUARE":
                square_type = {0:self.signboardpole_square_type_0,
                               1:self.signboardpole_square_type_1,
                               2:self.signboardpole_square_type_2}[i]
                signboard_type = "CH" + str_size + "_Square"
                signboard_type += {"TYPEA":"A",
                                   "TYPEB":"B",
                                   "TYPEC":"C"}[square_type]

            m_name = signboard_type

            objSign = self.make_obj(context, m_name)
            objSign.parent = objArm
            objSign.location.y = pos_y

            if (str_size == "0.9m"):
                pos_y -= 1.2
                mod_offset = 0.15
            if (str_size == "1.2m"):
                pos_y -= 1.5
                mod_offset = 0.20

            if (self.signboardpole_mirror == True):
                objSign.rotation_euler.z = 3.14159

            objSign.poleprops.pole_type = "SIGNBOARD"
            objSign.poleprops.signboard_type = signboard_type

            mod = self.add_GN_modifier(objSign, '.GN.Pole.SignboardConnector.Hx2')
            paramList = [("Material", material),
                         ("Radius", 0.09),
                         ("Depth" , 0.045),
                         ("Offset", mod_offset)]
            self.set_GN_inputs(mod, paramList)

            # Tex 番号は固定： Traffic Mark で変更する
            mod = self.add_GN_modifier(objSign, '.GN.Pole.SignTexID')
            paramList = [("UseTexture", True),
                         ("UDIM_ID", 1),
                         ("scale", 1.0)]
            self.set_GN_inputs(mod, paramList)
            
        return objArm
    

    def exec_signboardpole(self, context, matrix_world, v, rot_euler, material):

        curveOB, top, btm = self.make_curve_pole(context, "signboard_pole", self.SignboardPoleDict, self.signboardpole_size, matrix_world, v.co, rot_euler, material, z_offset = self.signboardpole_extend)
        top_radius, btm_radius = self.get_pole_radius(self.SignboardPoleDict, self.signboardpole_size)

        self.add_mod_curve_to_mesh(curveOB)
        
        pole_size = self.signboardpole_size
        
        curveOB.poleprops.pole_type = "SIGNBOARD_POLE"
        curveOB.poleprops.pole_subtype = pole_size
        
        if pole_size in ["0.2m", "0.3m"]:
            return self.exec_signboardpole_arm(context, curveOB, top, pole_size, material)
        
        sign_list = []
        size_list = []

        if (self.signboardpole_number >= 1):
            sign_list.append(self.signboardpole_signtype_0)
            size_list.append(self.signboardpole_signsize_0)
            
        if (self.signboardpole_number >= 2):
            sign_list.append(self.signboardpole_signtype_1)
            size_list.append(self.signboardpole_signsize_1)

        if (self.signboardpole_number >= 3):
            sign_list.append(self.signboardpole_signtype_2)
            size_list.append(self.signboardpole_signsize_2)
            
        #print("sign_list=", sign_list)
        
        pos_z = top - 0.2 

        for i, (sign, str_size) in enumerate(zip(sign_list, size_list)):

            if (self.signboardpole_layout == "OFFSET"):
                str_layout = "OV"
            if (self.signboardpole_layout == "CENTERED"):
                str_layout = "CV"
            if (str_size == "0.6m"):
                str_layout = "CV"
                
            square_type = {0:self.signboardpole_square_type_0,
                           1:self.signboardpole_square_type_1,
                           2:self.signboardpole_square_type_2}[i]

            signboard_type = self.get_signboard_type(str_layout, str_size, sign, square_type)
            
            m_name = signboard_type
            
            prop = self.Signboard_PropDict[signboard_type]
            if (i > 0):
                pos_z -= prop[0]

            objSign = self.make_on_pole(context, curveOB, m_name, pos_z, offset = 0)
            objSign.poleprops.pole_type = "SIGNBOARD"
            objSign.poleprops.signboard_type = signboard_type

            pos_z -= prop[1]

            
            mod = self.add_GN_modifier(objSign, '.GN.Pole.SignboardConnector')
            paramList = [("Material", material),
                         ("Radius", top_radius + 0.005),
                         ("Depth", {"0.4m":0.03,
                                    "0.6m":0.045}[str_size])]
            self.set_GN_inputs(mod, paramList)

            # Tex 番号は固定： Traffic Mark で変更する
            mod = self.add_GN_modifier(objSign, '.GN.Pole.SignTexID')
            paramList = [("UseTexture", False),
                         ("UDIM_ID", 1),
                         ("scale", 1.0)]
            self.set_GN_inputs(mod, paramList)
            
        return curveOB

    def exec_mirror_sq(self, context, objPole, material):
        
        #m_name_connector = "MirrorConnector.B"
        m_name_mirror = "MirrorConnector.SQ.H." + self.mirror_size_type

        params = [(3.185, -1/4, 0), (3.127, -3/4, -1)]

        offset_x = {"0.6m":0.4,
                    "0.8m":0.5}[self.mirror_size_type]

        for param in params:

            pos_z = param[0]
            rot_z = param[1] * 3.14159
            mirror_rot_z = param[2] * 3.14159

            arm_ext = self.mirror_arm_extend
            
            objConnect = self.make_on_pole(context, objPole, m_name_mirror, pos_z + self.mirror_extend)
            objConnect.rotation_euler.z = rot_z
            
            mod = self.add_GN_modifier(objConnect, '.GN.Pole.MirrorConnector.H')
            paramList = [("OffsetX", offset_x),
                         ("Extend", arm_ext),
                         ("Decline", self.mirror_decline * 9 / 3.14159 ),
                         ("RotZ", mirror_rot_z)]
            self.set_GN_inputs(mod, paramList)
            
    def exec_mirror(self, context, matrix_world, v, rot_euler, material):

        print("exec_mirror")
        
        m_name_pole = {"TYPEA":"MirrorPole.A",
                       "TYPEB":"MirrorPole.A",
                       "TYPEC":"MirrorPole.B"}[self.mirror_board_type]
        
        objPole = self.make_obj(context, m_name_pole)
        objPole.location = matrix_world @ v.co
        objPole.rotation_euler = rot_euler

        if (self.mirror_board_type == "TYPEA"):
            self.add_mask_modifier(objPole, "BOARD")

        if (self.mirror_extend != 0):
            self.add_displace_modifier(objPole, "EXTEND", "Z", self.mirror_extend)
            
        extra = 0
        if (self.mirror_board_tex_id == 3):
            extra = 1

        self.set_GN_for_sign(objPole, tex_index = self.mirror_board_tex_id,
                             invert = True,
                             basecolor  = Vector((1,1,1,1)),
                             texcolor   = Vector((0,0,0,1)),
                             extracolor = Vector((1, 0.053536, 0.008541, 1)),
                             extra = extra)
        
        m_name_mirror = {
            "TYPEA":"MirrorConnector.",
            "TYPEB":"MirrorConnector.",
            "TYPEC":"MirrorConnector.SQ.",
            "TYPED":"MirrorConnector.SQ,",
        }[self.mirror_type]

        if "SQ" in m_name_mirror:
            if self.mirror_size_type == "1.0m":
                self.mirror_size_type = "0.8m"

        m_name_mirror += self.mirror_size_type

        if (self.mirror_type == "TYPED"):
            return self.exec_mirror_sq(context, objPole, material)

        is_dual = self.mirror_type == "TYPEB"

        params = {False:[(3.185, 0)],
                  True:[(3.185, 1/4), (3.127, -1/4)]}[is_dual]

       
        for param in params:
            pos_z = param[0]
            rot_z = param[1]
            
            objMirror = self.make_on_pole(context, objPole, m_name_mirror, pos_z + self.mirror_extend)

            ext = self.mirror_arm_extend
            
            if (is_dual == True):
                ext += {"0.6m":0.18,
                        "0.8m":0.28,
                        "1.0m":0.38}[self.mirror_size_type]

            mod = self.add_GN_modifier(objMirror, ".GN.Pole.MirrorConnector.Simple")
            paramList = [("Extend", ext),
                         ("Decline", self.mirror_decline * 9 / 3.14159 )]
            self.set_GN_inputs(mod, paramList)

            objMirror.rotation_euler.z = rot_z * 3.14159

        return

    def exec_curvepole(self, context, matrix_world, v, rot_euler, material):
        
        m_name_pole = "CurvePole." + {"LEFT":"L", "RIGHT":"R"}[self.curvepole_direction] + "." + {"TYPEA":"A", "TYPEB":"B"}[self.curvepole_type]
        objPole = self.make_obj(context, m_name_pole)
        objPole.location = matrix_world @ v.co
        objPole.rotation_euler = rot_euler

        return

    def exec_guidesign(self, context, matrix_world, v, rot_euler, material):

        pole_size = self.guidesign_pole_size
        pole_small_type = self.guidesign_pole_small_type
        pole_large_type = self.guidesign_pole_large_type
        pole_type = {"SMALL":pole_small_type,
                     "LARGE":pole_large_type}[pole_size]
        pole_key = pole_size + "_" + pole_type

        curveOB, top, btm = self.make_curve_pole(context, "guidesign_pole", self.GuidesignPoleDict, pole_key, matrix_world, v.co, rot_euler, material, z_offset = self.guidesign_extend_z)

        self.add_mod_curve_to_mesh(curveOB)
        
        curveOB.poleprops.pole_type = "GUIDESIGN_POLE"
        
        if (pole_size == "LARGE"):

            m_name_arm = self.GuideSignLArmDict[pole_large_type]
            if (pole_type == "TYPEA"):
                pos_y = -(3 + self.guidesign_extend_arm)
            if (pole_type == "TYPEB"):
                pos_y = -3

            pos_z = top - 1.5
            
            objArm = self.make_on_pole(context, curveOB, m_name_arm, pos_z, 0)

            extend_arm = self.guidesign_extend_arm + self.GuideSignLargeArmOffsetDict[self.guidesign_large_type]
            m_name_board = "GuideSign.L." + self.guidesign_large_type
            
            objBoard = self.make_obj(context, m_name_board)
            objBoard.parent = objArm
            
            objBoard.location.y = pos_y

            if (self.guidesign_mirror == True):
                objBoard.rotation_euler.z = 3.14159
                
            if (pole_type == "TYPEA"):
                self.add_displace_modifier(objArm, "EXTEND", "Y", -extend_arm)
            if (pole_type == "TYPEB"):
                mod = self.add_GN_modifier(objArm, ".GN.GuideSignLargeB")
                paramList = {("Pole", curveOB),
                             ("Extend", extend_arm)}
                self.set_GN_inputs(mod, paramList)

            objArm.poleprops.pole_type = "GUIDESIGN_ARM"
            objBoard.poleprops.pole_type = "GUIDESIGN_BOARD"

            self.set_guidesign_large_modifier(objBoard)

            
        if (pole_size == "SMALL"):

            def set_modifier(objSign):

                self.set_GN_for_sign(objSign, tex_index = self.guidesign_tex_index, invert = True)

            num = self.guidesign_number
            m_name_sign = self.GuideSignSmallType_Dict[self.guidesign_pole_small_type]
            
            pos_z = top - 0.3

            z_offset = 0.4
            if self.guidesign_pole_small_type in ["TYPEA", "TYPEB"]:
                z_offset = 0.5

            for index in range(num):
                objSign = self.make_on_pole(context, curveOB, m_name_sign, pos_z, 0)
                objSign.rotation_euler.z = getattr(self, "guidesign_small_rot_"+str(index))
                set_modifier(objSign)
                pos_z -= z_offset
                
    def exec_polesign(self, context, matrix_world, v, rot_euler, material):

        pole_size = self.polesign_pole_size
        pole_category = self.polesign_ad_category
        pole_type = self.polesign_pole_type
        pole_ad_type = self.polesign_ad_type
        pole_ad_size = self.polesign_ad_size
        pole_ad_light = self.polesign_ad_light

        m_name_pole = self.AdPoleDict[pole_type]

        objPole = self.make_obj(context, m_name_pole)
        objPole.location = matrix_world @ v.co
        objPole.rotation_euler = rot_euler

        extend = pole_size - 4
        self.add_displace_modifier(objPole, "EXTEND", "Z", extend)

        tag = pole_category + pole_ad_type + pole_ad_size

        m_name_add = self.PolesignDict[tag]

        objAd = self.make_obj(context, m_name_add)
        objAd.parent = objPole

        objAd.location = (0, 0, 4+extend)
        
        if (pole_category == "SQUARE" and pole_ad_light == True):
            light_type = self.polesign_light_type
            light_extend = self.polesign_light_extend
            m_name_light = {"TYPEA":"Ad.Light.A",
                            "TYPEB":"Ad.Light.B"}[light_type]
            col = self.get_asset_collection(context, "PoleParts", context.scene.collection)
            objLight = self.load_object(context, m_name_light, col)

            mod_light = self.add_GN_modifier(objAd, ".GN.Pole.AddAdLight")
            
            light_num = self.polesign_light_num
            if (pole_ad_size == "2m"):
                y_ini = {1:-0.5, 2:-0.5, 3:-0.8,
                         4:-0.75, 5:-0.8, 6:-0.75}[light_num]
            if (pole_ad_size == "3m"):
                y_ini = {1:-1, 2:-1, 3:-1.3,
                         4:-1.2, 5:-1.2, 6:-1.25}[light_num]

            paramList = [("Object"   , objLight),
                         ("Start"    , Vector((0,  y_ini, 2.5)) ),
                         ("End"      , Vector((0, -y_ini, 2.5)) ),
                         ("Count"    , light_num),
                         ("ExtendSel", "EXTEND"),
                         ("OffsetX" , -light_extend)]
            self.set_GN_inputs(mod_light, paramList)

        mod = self.add_GN_modifier(objAd, '.GN.Pole.SignTexID')
        
        use_texture = (self.polesign_ad_tex_index > 0)
            
        paramList = [("UseTexture", use_texture),
                     ("UDIM_ID", self.polesign_ad_tex_index),
                     ("scale", self.polesign_ad_tex_scale),
                     ("OffsetY", self.polesign_ad_tex_offsetY),
                     ("Emission", self.polesign_ad_tex_emission),]
        self.set_GN_inputs(mod, paramList)

        mod = self.add_GN_modifier(objAd, '.GN.Pole.ColorReplacer')
        paramList = [("Epsilon", 0.5),
                     ("Hue", 0.5),
                     ("Saturation", 0.5),
                     ("Value", 1.0)]
        self.set_GN_inputs(mod, paramList)
        
        mod = self.add_GN_modifier(objAd, '.GN.Pole.ModifyHSV')
        paramList = [("Hue", 0.5),
                     ("Saturate", 1.0),
                     ("Value", 1.0)]
        self.set_GN_inputs(mod, paramList)
        

    def auto_connect(self, obj_list):
        if (self.single_mode == True):
            return

        #bpy.ops.object.mode_set(mode = 'OBJECT')
        object = bpy.context.view_layer.objects.active

        for previous, current in zip(obj_list, obj_list[1:]):
            bpy.ops.object.select_all(action='DESELECT')
            previous.select_set(True)
            current.select_set(True)
            bpy.context.view_layer.objects.active = current
            bpy.ops.polemaker.catenary_connect(action='CONNECT', catenary_a_inv = self.telegraph_catenary_a_inv, com_spiral = self.telegraph_com_spiral, com_box = self.telegraph_com_box, use_collection = self.use_collection)

        bpy.context.view_layer.objects.active = object

        bpy.ops.object.select_all(action='DESELECT')
        #bpy.ops.object.mode_set(mode = 'EDIT')
        

    def make_poles(self, context, object, data, material):

        if (data != None):
            bpy.ops.object.mode_set(mode='EDIT')
            bm = bmesh.from_edit_mesh(data)
        else:
            bm = bmesh.new()
            bm.verts.new(context.scene.cursor.location)
            bm.verts.ensure_lookup_table()
            bm.verts[0].select = True

        if (object != None):
            mat_wld = object.matrix_world
        else:
            mat_wld = mathutils.Matrix()

        v_list = []
        obj_list = []

        v0 = self.search_rim(bm, True)

        if (v0 == None):
            v0 = self.search_selected(bm)

        if (v0 == None):
            v0 = self.search_rim(bm, False)

        if (v0 == None):
            bpy.ops.object.mode_set(mode='OBJECT')
            return {'FINISHED'}
        
        def exec_unit(context, mat_wld, v, rot, material, count):

            if (self.action == "TELEGRAPH"):
                obj = self.exec_telegraph(context, mat_wld, v, rot, material, count)
                obj_list.append(obj)

            if (self.action == "SIGNALPOLE"):
                obj = self.exec_signalpole(context, mat_wld, v, rot, material)

            if (self.action == "STREETLIGHT"):
                obj = self.exec_streetlight(context, mat_wld, v, rot, material)

            if (self.action == "SIGNBOARDPOLE"):
                obj = self.exec_signboardpole(context, mat_wld, v, rot, material)

            if (self.action == "CURVEMIRROR"):
                obj = self.exec_mirror(context, mat_wld, v, rot, material)

            if (self.action == "CURVEPOLE"):
                obj = self.exec_curvepole(context, mat_wld, v, rot, material)

            if (self.action == "GUIDESIGN"):
                self.exec_guidesign(context, mat_wld, v, rot, material)

            if (self.action == "POLESIGN"):
                self.exec_polesign(context, mat_wld, v, rot, material)
                
        rot = self.calc_rot_euler(v0, v_list)

        exec_unit(context, mat_wld, v0, rot, material, 0)
        
        v_list.append(v0)

        def is_next(v, v_list):

            for e in v.link_edges:
                if (e.verts[0] == v):
                    v_tgt = e.verts[1]
                else:
                    v_tgt = e.verts[0]

                #print("is_next", v, v_tgt, v_list)
                if v_tgt in v_list:
                    return True

            return False
        
        count = 0

        max_count = context.preferences.addons[addon_name].preferences.max_count

        while(True):
            count += 1
            do = False
            for v in bm.verts:
                if (do == True):
                    continue
                if v in v_list:
                    continue
                if is_next(v, v_list) == False:
                    continue

                if v.select == False:
                    v_list.append(v)
                    continue

                do = True

                rot = self.calc_rot_euler(v, v_list)
                exec_unit(context, mat_wld, v, rot, material, count)
                
                v_list.append(v)

            if (do == False):
                break

            if (count > max_count):
                break

        if bpy.context.mode=="EDIT_MESH":
            bpy.ops.object.mode_set(mode='OBJECT')

        if (obj_list != [] and self.telegraph_auto_connect == True):
            self.auto_connect(obj_list)

        if (object != None):
            
            bpy.ops.object.select_all(action='DESELECT')
            bpy.context.view_layer.objects.active = object
            object.select_set(True)

        return {'FINISHED'}

    def invoke(self, context, event):

        return self.execute(context)

    def execute(self, context):

        object = context.object

        if (object == None) or (object.type != 'MESH' or object.select_get() == False):
            object = None
            data = None
        else:
            data = object.data

            selection = 0
            for v in data.vertices:
                if v.select == True:
                    selection = True
                    break

            if (selection == 0):

                self.report({'INFO'}, "No vertices are selected, automatically selected all vertices.")

                for v in data.vertices:
                    v.select = True
                
        material = self.get_material(object)
            
        if (material == None):
            if (context.preferences.addons[addon_name].preferences.use_material == True):

                if self.action == "SIGNBOARDPOLE":
                    material = self.load_material(".Pole.White")
                elif self.action == "TELEGRAPH":
                    material = self.load_material(".Pole.Concrete")
                else:
                    material = self.load_material(".Pole.Basic")

        random.seed(self.pole_random_seed)

        self.single_mode = (object == None)

        return self.make_poles(context, object, data, material)

        return {'FINISHED'}

class MESH_OT_AddPoleAppendix(Operator, PoleMakerCommons, AppendixCommon, type_module.TypeModule, modifier_module.ModifierModule):

    bl_idname = "polemaker.add_pole_appendix"
    bl_label = "Add pole appendix"
    bl_description = "Description"
    bl_options = {'REGISTER', 'UNDO'}

    TelegraphDict = MESH_OT_PoleMaker.TelegraphDict
    
    CommonTypeSelection = type_module.TypeModule.CommonTypeSelection
    CommonTypeSelectionWithNone = type_module.TypeModule.CommonTypeSelectionWithNone

    def AppendixType_callback(self, context):
        type = "AppendixType"
        get = get_pcoll_icon_id
        items = [
            ("TRAFFICLIGHT", "Traffic Light","", get(type, "TRAFFICLIGHT"), 1),
            ("CONTROLLER"  , "Controller"   ,"", get(type, "CONTROLLER"), 2),
            ("ARM"         , "Arm"          ,"", get(type, "ARM"), 3),
            ("SIGNBOARD"   , "Sign Board"   ,"", get(type, "SIGNBOARD"), 4),
            ("FOOTAGEBOLTS", "Footage Bolts","", get(type, "FOOTAGEBOLTS"), 5),
            ("CURVE_MIRROR", "Curve Mirror" ,"", get(type, "CURVE_MIRROR"), 6),
            ("LAMP"        , "Lamp"         ,"", get(type, "LAMP"), 7),
            ("TRANS"       , "Trans"        ,"", get(type, "TRANS"), 8),
        ]            
        return items

    def CurveMirror_callback(self, context):
        type = "AppendixMirrorType"
        get = get_pcoll_icon_id
        items = [("SINGLE", "Single", "", get(type, 1), 1),
                 ("DOUBLE", "Double", "", get(type, 2), 2)
                 ]
        return items

    def WireAppendix_callback(self, context):
        type = "WireAppendixType"
        get = get_pcoll_icon_id
        items = [("SLACK"    , "Slack"     , "", get(type, "SLACK")    , 1),
                 ("BIRDFENCE", "Bird Fence", "", get(type, "BIRDFENCE"), 2),
                 ("INSULATOR", "Insulator" , "", get(type, "INSULATOR"), 3),
                 ("CLOSURE"  , "Closure"   , "", get(type, "CLOSURE")  , 4),]
        return items

    def BirdFence_callback(self, context):
        return common_type_callback(2, "BirdFenceType")

    def Insulator_callback(self, context):
        return common_type_callback(2, "InsulatorType")

    def Closure_callback(self, context):
        return common_type_callback(2, "ClosureType")
    
        
    action : StringProperty()

    appendix_type : EnumProperty(
        name = "Type", default = 1,
        items = AppendixType_callback,
        #items = PoleMakerCommons.EnumCustomIcon_AppendixType
    )

    appendix_rot_z : FloatProperty(
        name = "Rotation", default = 0, subtype = "ANGLE", 
        min = -3.14159*2, max = 3.14159*2, step = 10)

    traffic_light_offset : FloatProperty(
        name = "Offset", default = 0,
        min = -2, max = +2, step = 0.2)

    controller_offset : FloatProperty(
        name = "Offset", default = 0,
        min = -0.5, max = 0.5, step = 0.2)

    arm_type : EnumProperty(
        name = "Type", default = "LV",
        items = [("GW", "Ground", "Ground", "", 1),
                 ("HV", "HV", "High Voltage", "", 2),
                 ("LV", "LV", "Low Voltage", "", 3),
                 ("Com", "Com", "Communication", "", 4),
                 ("WIRE", "Wire", "Wire", "", 5)])
    
    telegraph_arm_LV0_offset : FloatProperty(
        name = "Offset", default = 0,
        min = -6, max = 2, step = 0.3) #範囲が違うので別定義

    SignBoardBaseDict = {"TYPEA": "SignBoardBase.A",
                         "TYPEB": "SignBoardBase.B",
                         "TYPEC": "SignBoardBase.C",
                         "TYPED": "SignBoardBase.D",
                         }

    SignBoard_RingDict = {"TYPEA":((-0.46, -0.73), 0.08, None),
                          "TYPEB":((0,), 0.08, ("B")),
                          "TYPEC":((0, 1.18), 0.08, ("B", "B")),
                          "TYPED":((-0.09, 0.09), 0.08, None),
                          }
    
    sign_board_base_type : EnumProperty(
        name = "Type", default = 1,
        items = PoleMakerCommons.Signboard_callback)

    sign_board_base_style : EnumProperty(
        name = "Style", default = "TYPEA",
        items = CommonTypeSelection[0:3])

    sign_board_style : EnumProperty(
        name = "Style", default = "TYPEA",
        items = CommonTypeSelection[0:3])
    
    sign_board_offset : FloatProperty(
        name = "Offset", default = 0,
        min = -1.5, max = 1.5, step = 0.3)

    sign_board_extend : FloatProperty(
        name = "Extend", default = 0,
        min = -1, max = 2, step = 0.2)

    sign_board_num : IntProperty(
        name = "Number", default = 1,
        min = 1, max = 3)

    footage_min_height : FloatProperty(
        name = "Min", default = 2,
        min = 1.5, max = 4.5, step = 0.3)
    
    CurveMirrorBaseDict= {"SINGLE":"MirrorParts.Base.Single",
                          "DOUBLE":"MirrorParts.Base.Double"}

    CurveMirror_RingDict = {"SINGLE":((-0.09, 0.09), 0.06, None),
                            "DOUBLE":((-0.09, 0.09), 0.06, None) }
    
    curve_mirror_type : EnumProperty(
        name = "Type", default = 1,
        items = CurveMirror_callback)

    curve_mirror_offset : FloatProperty(
        name = "Offset", default = 0,
        min = -2, max = +2, step = 0.2)
    
    curve_mirror_size : EnumProperty(
        name = "Size", default = "1.0m",
        items = MESH_OT_PoleMaker.MirrorSizeType)

    curve_mirror_base_rot_z : FloatProperty(
        name = "Rot Base", subtype = "ANGLE", default = 0,
        min = -3.14159, max = 3.14159, step = 10)

    curve_mirror_mirrorA_rot_z : FloatProperty(
        name = "Rot Mirror", subtype = "ANGLE", default = 0,
        min = -3.14159/2, max = 3.14159/2, step = 10)

    curve_mirror_mirrorB_rot_z : FloatProperty(
        name = "Rot Mirror", subtype = "ANGLE", default = 0,
        min = -3.14159/2, max = 3.14159/2, step = 10)

    curve_mirror_mirrorA_decline : FloatProperty(
        name = "Decline", subtype = "ANGLE", default = 0,
        min = 0, max = 3.14159 / 9, step = 5)

    curve_mirror_mirrorB_decline : FloatProperty(
        name = "Decline", subtype = "ANGLE", default = 0,
        min = 0, max = 3.14159 / 9, step = 5)

    flip : BoolProperty(
        name = "Flip", default = False)

    signboard_pole_appendix_type : EnumProperty(
        name = "Type", default = "SIGNBOARD",
        items =  [("SIGNBOARD", "Signboard", "Signboard", "", 1),
                  ("ADDITIONAL", "Additional Pole", "Additional Pole", "", 2),
                  ("CUSTOM", "Custom", "Custom", "", 3)])

    guidesign_large_offset : FloatProperty(
        name = "Offset", default = 0,
        min = -0.3, max = 3, step = 0.3)

    signboard_signsize_H : EnumProperty(
        name = "Size", default = "0.9m",
        items = PoleMakerCommons.SignboardSizeType[2:4])

    signboard_signsize_V : EnumProperty(
        name = "Size", default = "0.4m",
        items = PoleMakerCommons.SignboardSizeType[0:2])

    signboard_signtype : EnumProperty(
        name = "Type", default = 1,
        items = PoleMakerCommons.SignboardPoleSignType_callback)

    signboard_square_type : EnumProperty(
        name = "Type", default = "TYPEA",
        items = CommonTypeSelection[0:3])

    signboard_offset_H : FloatProperty(
        name = "Offset", default = 0,
        min = -0.3, max = 1.3, step = 0.2)

    signboard_offset_V : FloatProperty(
        name = "Offset", default = 0,
        min = -0.3, max = 0.3, step = 0.2)

    signboard_layout_type : EnumProperty(
        name = "Layout", default = "OFFSET",
        items = PoleMakerCommons.SignboardLayoutType)

    signboard_rotation : FloatProperty(
        name = "Rotation", subtype = "ANGLE", default = 0,
        min = -3.14159, max = 3.14159, step = 10)

    signboard_pole_extend : FloatProperty(
        name = "Extend", default = 0,
        min = -0.2, max = 1, step = 0.1)

    signboard_custom : StringProperty(
        name = "Custom")

    trafficlight_addtional_type : EnumProperty(
        name = "Type", default = "ARROW",
        items = [("ARROW", "Arrow Signal", "Arrow Signal", "", 1),
                 ("HEADBOARD", "Head Board", "Head Board", "", 2),
                 ("SIDEBOARD", "Side Board", "Side Border", "", 3)])

    trafficlight_arrow_signal_type : EnumProperty(
        name = "Type", default = "TYPEA_SINGLE",
        items = PoleMakerCommons.ArrowLightType)

    appendix_trans_offset : FloatProperty(
        name = "Offset", default = 0,
        min = -3, max = 3, step = 0.3)

    wire_appendix_type : EnumProperty(
        name = "Type", default = 2,
        items = WireAppendix_callback
    )

    BirdFenceDict = {"TYPEA":"BirdFence.A",
                     "TYPEB":"BirdFence.B"}

    InsulatorDict = {"TYPEA":"Insulator.A",
                     "TYPEB":"Insulator.B"}

    ClosureDict = catenary_module.POLEMAKEROBJ_OT_CatenaryConnect.ClosureDict

    bird_fence_type : EnumProperty(
        name = "Type", default = 1,
        items = BirdFence_callback)

    insulator_type : EnumProperty(
        name = "Type", default = 1,
        items = Insulator_callback)

    closure_type : EnumProperty(
        name = "Type", default = 1,
        items = Closure_callback)

    wire_spline_index : IntProperty(
        name = "Spline Index", default = 0,
        description = "-1 for all splines",
        min = -1)

    wire_appendix_count : IntProperty(
        name = "Count", default = 1,
        min = 1, max = 30)

    wire_start : FloatProperty(
        name = "Start", default = 0.05,
        min = 0, max = 1, precision = 3, )

    wire_end : FloatProperty(
        name = "End", default = 0.25,
        min = 0, max = 1, precision = 3)

    wire_randomize : FloatProperty(
        name = "Randomize", default = 0,
        min = 0, max = 1)

    slack_start : FloatProperty(
        name = "Start", default = 0.05,
        min = 0, precision = 3)

    slack_width : FloatProperty(
        name = "Width", default = 1,
        min = 0, step = 0.3, precision = 4)

    slack_slack : FloatProperty(
        name = "Slack", default = 1,
        min = 0, step = 0.3, precision = 3)

    slack_roundness : FloatProperty(
        name = "Roundness", default = 0,
        min = 0, max = 1, precision = 3)

    slack_radius : FloatProperty(
        name = "Radius Factor", default = 1,
        min = 0, max = 2, precision = 3)

    

    #test_cancel : BoolProperty(
    #    name = "cancel",
    #    default = False)

    pole_type : StringProperty() #一時利用
    pole_subtype : StringProperty() #一時利用
    telegraph_type : StringProperty() #一時利用

    time_prev = 0

    def draw_catenary_appendix(self, context, layout):

        col = layout.column(align = True)

        current_name = self.get_enum_current_name("wire_appendix_type", self.WireAppendix_callback(context))
        
        if (self.wire_appendix_type == "SLACK"):
            self.PS2(col, ["wire_appendix_type", current_name+"@", "wire_spline_index"])
            
            col.prop(self, "slack_start")
            row = col.row()
            row.prop(self, "slack_width")
            row.prop(self, "slack_slack")
            col.prop(self, "slack_roundness")
            col.prop(self, "slack_radius")

        def wire_item(col):
            col.prop(self, "wire_appendix_count")
            col.prop(self, "wire_start", slider=True)
            col.prop(self, "wire_end", slider=True)
            col.prop(self, "wire_randomize", slider=True)

        if (self.wire_appendix_type == "BIRDFENCE"):
            self.PS3(col, ["wire_appendix_type", "bird_fence_type", current_name+"@", "wire_spline_index"])
            wire_item(col)

        if (self.wire_appendix_type == "INSULATOR"):
            self.PS3(col, ["wire_appendix_type", "insulator_type", current_name+"@", "wire_spline_index"])
            wire_item(col)

        if (self.wire_appendix_type == "CLOSURE"):
            self.PS3(col, ["wire_appendix_type", "closure_type", current_name+"@", "wire_spline_index"])
            wire_item(col)
            
            

    def draw_guidesign_arm(self, layout):

        col = layout.column(align = True)
        col.prop(self, "guidesign_large_type")
        col.prop(self, "guidesign_tex_index")
        col.prop(self, "guidesign_large_offset")
        col.prop(self, "flip")

    def draw_signboard_pole(self, layout, strDir):

        col = layout.column(align = True)
        
        col.prop(self, "signboard_signsize_" + strDir)
        
        if (strDir == "V"):
            col.prop(self, "signboard_layout_type")
            
        proplist = ["signboard_signtype", "signboard_offset_" + strDir]
        if (self.signboard_signtype == "SQUARE"):
            proplist.append("#signboard_square_type")
            self.PS2(col, proplist)
            
        if (strDir == "V"):
            layout.prop(self, "signboard_offset_V")
            layout.prop(self, "signboard_rotation")
        if (strDir == "H"):
            layout.prop(self, "flip")

    def draw_signboard_additional(self, layout):

        layout.prop(self, "signboard_offset_V")
        layout.prop(self, "signboard_pole_extend")
        layout.prop(self, "signboard_rotation")

           
    def draw_signboard_custom(self, layout):
        
        layout.prop(self, "signboard_offset_V")
        layout.prop_search(self, "signboard_custom", bpy.data, "meshes")
        layout.prop(self, "signboard_rotation")

    def draw_trafficlight_additional(self, layout):

        layout.prop(self, "trafficlight_addtional_type")
        if self.trafficlight_addtional_type == "ARROW":
            layout.prop(self, "traffic_light_arrow_signal_type")
        if self.trafficlight_addtional_type == "HEADBOARD":
            layout.prop(self, "traffic_light_head_board_type")
            layout.prop(self, "traffic_light_head_board_offset")
            layout.prop(self, "traffic_light_head_board_tex_index")
        if self.trafficlight_addtional_type == "SIDEBOARD":
            layout.prop(self, "traffic_light_side_board_type")
            layout.prop(self, "traffic_light_side_board_tex_index")
            
    def draw(self, context):

        layout = self.layout
        pole_type = self.pole_type

        if (pole_type == "CATENARY"):
            return self.draw_catenary_appendix(context, layout)

        if (pole_type == "GUIDESIGN_ARM"):
            return self.draw_guidesign_arm(layout)

        if (pole_type == "SIGNBOARD_ARM"):
            return self.draw_signboard_pole(layout, "H")

        if (pole_type in {"SIGNBOARD_POLE", "SIGNBOARD_ADDTIONAL"}):
            layout.prop(self, "signboard_pole_appendix_type")
            if (self.signboard_pole_appendix_type == "SIGNBOARD"):
                return self.draw_signboard_pole(layout, "V")
            if (self.signboard_pole_appendix_type == "ADDITIONAL"):
                return self.draw_signboard_additional(layout)
            if (self.signboard_pole_appendix_type == "CUSTOM"):
                return self.draw_signboard_custom(layout)

        if (pole_type in ["TRAFFICLIGHT"]):
            return self.draw_trafficlight_additional(layout)

        prop_list = ["appendix_type", "appendix_type"]
        if (self.appendix_type == "TRAFFICLIGHT"):
            prop_list.append("traffic_light_style")
        if (self.appendix_type == "SIGNBOARD"):
            prop_list.append("sign_board_base_style")
            
        self.PS2(layout, prop_list, flip=True)
        
        if (self.appendix_type == "TRAFFICLIGHT"):

            subtype = self.traffic_light_type

            layout.prop(self, "traffic_light_type", text="")
            #layout.prop(self, "traffic_light_style")
            if subtype in ["FOR_CARS", "FOR_CARS_DOUBLE"]:
                self.PS2(layout, ["traffic_light_subtype", "traffic_light_offset", "traffic_light_extend", "appendix_rot_z"])
                
            if subtype in ["FOR_WALKERS"]:
                self.PS2(layout, ["traffic_light_walker_subtype", "traffic_light_offset", "traffic_light_extend", "appendix_rot_z"])
                
            if subtype in ["FOR_CARS", "FOR_CARS_DOUBLE"]:
                self.draw_traffic_light_car(layout)
                
            if subtype in ["FOR_WALKERS"]:
                col, row = self.newbox(layout)
                row.prop(self, "traffic_light_side_board")
                row.prop(self, "traffic_light_support")
                if (self.traffic_light_side_board == True):
                    col.prop(self, "traffic_light_side_board_tex_index")
                    row = col.row()
                    row.prop(self, "traffic_light_speaker")

            row = layout.row()
            row.prop(self, "flip")
            row.prop(self, "traffic_light_mirror")

        if (self.appendix_type == "CONTROLLER"):
            col, row = self.newbox(layout)
            col.prop(self, "controller_type")
            if (self.controller_type == "BOX"):
                self.PS2(col, ["controller_box_type", "controller_offset", "controller_box_ext", "appendix_rot_z"])
                
                row = col.row()
                row.prop(self, "controller_box_mark")
                row.prop(self, "controller_box_opt1")
                

                row = layout.row()
                row.prop(self, "flip")
                row.prop(self, "controller_box_mirror")

            if (self.controller_type == "BUTTON"):
                self.PS2(col, ["controller_button_type", "controller_offset", "controller_button_ext", "appendix_rot_z"])
                

                row = col.row()
                row.prop(self, "flip")

        if (self.appendix_type == "ARM"):
            col, row = self.newbox(layout)
            col.prop(self, "arm_type")
            if (self.arm_type == "HV"):
                self.PS2(col, ["telegraph_arm_HV_type", "telegraph_arm_HV_offset", "appendix_rot_z", "flip"])

            if (self.arm_type == "LV"):
                self.PS2(col, ["telegraph_arm_LV0_type", "telegraph_arm_LV0_offset", "appendix_rot_z", "flip"])
                

            if (self.arm_type == "Com"):
                self.PS2(col, ["telegraph_arm_com_type", "telegraph_arm_com_offset", "appendix_rot_z", "flip"])
                
            if (self.arm_type == "WIRE"):
                self.PS2(col, ["telegraph_arm_wire_type", "telegraph_arm_wire_offset", "appendix_rot_z", "flip"])
                

        if (self.appendix_type == "SIGNBOARD"):
            col, row = self.newbox(layout)
            row.prop(self, "sign_board_num")
            row.prop(self, "sign_board_style")
            self.PS2(col, ["sign_board_base_type", "sign_board_offset", "sign_board_extend", "appendix_rot_z"])
            
            row = col.row()
            row.prop(self, "appendix_rot_z")
            
            row = col.row()
            row.prop(self, "flip")
            row.prop(self, "traffic_light_mirror")

        if (self.appendix_type == "FOOTAGEBOLTS"):
            layout.prop(self, "footage_min_height")
            
        if (self.appendix_type == "CURVE_MIRROR"):
            col, row = self.newbox(layout)

            self.PS2(col, ["curve_mirror_type", "curve_mirror_offset", "curve_mirror_base_rot_z", "curve_mirror_size"])
            
            row = col.row()
            row.prop(self, "curve_mirror_mirrorA_rot_z")
            if (self.curve_mirror_type == "DOUBLE"):
                row.prop(self, "curve_mirror_mirrorB_rot_z")

            row = col.row()
            row.prop(self, "curve_mirror_mirrorA_decline")
            if (self.curve_mirror_type == "DOUBLE"):
                row.prop(self, "curve_mirror_mirrorB_decline")

        if (self.appendix_type == "LAMP"):
            col, row = self.newbox(layout)
            self.PS2(col, ["telegraph_lamp_type", "telegraph_lamp_offset", "appendix_rot_z", "flip"])
            
        if (self.appendix_type == "TRANS"):
            col, row = self.newbox(layout)
            self.PS2(col, ["telegraph_trans_type", "telegraph_trans_pos", "appendix_trans_offset", "flip"])
            
        layout.prop(self, "telegraph_make_rings")
        
    def make_arm(self, context, pole):

        top, bottom, btm_radius, top_radius = self.TelegraphDict[self.telegraph_type]
        if (self.arm_type == "HV"):
            z = top - self.telegraph_arm_HV_offset
            obj, _ = self.make_telegraph_arm_HV(context, pole, self.telegraph_arm_HV_type, z, self.flip)

        if (self.arm_type == "LV"):
            z = top-2 + self.telegraph_arm_LV0_offset
            obj = self.make_telegraph_arm_LV(context, pole, self.telegraph_arm_LV0_type, z, self.flip)

        if (self.arm_type == "Com"):
            z = 5 + self.telegraph_arm_com_offset
            obj = self.make_telegraph_arm_com(context, pole, self.telegraph_arm_com_type, z, self.flip)

        if (self.arm_type == "WIRE"):
            z = top-2 + self.telegraph_arm_wire_offset
            obj = self.make_telegraph_arm_wire(context, pole, self.telegraph_arm_wire_type, z, self.flip)

        obj.rotation_euler.z = self.appendix_rot_z
        
    def make_controller(self, context, pole):

        top, bottom, btm_radius, top_radius = self.TelegraphDict[self.telegraph_type]
        
        controller_type = self.controller_type

        if (controller_type == "BOX"):
            box_type = self.controller_box_type
            m_name = self.ControlBoxDict[box_type]
            if (self.controller_box_mirror == True):
                m_name += ".mir"

        if (controller_type == "BUTTON"):
            button_type = self.controller_button_type
            m_name = self.ControlButtonDict[button_type]

        pos_z = 0.9 + self.controller_offset
        rot_z = self.appendix_rot_z

        offset_vector = {"BUTTON":Vector((1,0,0)),
                         "BOX"   :Vector((-1,0,0))}[controller_type]
        
        objCtrl, partsOB = self.make_telegraph_instance_appendix(context, pole, m_name, "Controller", pos_z, rot_z, self.flip, as_instance = False, offset_vector = offset_vector)

        tgt_obj = objCtrl
        #if (context.preferences.addons[addon_name].preferences.modify_instancee == True):
        #    tgt_obj = partsOB

        if (controller_type == "BUTTON"):
            mod = self.add_GN_modifier(tgt_obj, ".GN.Pole.PartsOffset")
            offset_vector = Vector((0, 0, self.controller_button_ext))
            paramList = [("Realize Instance", True),
                         ("Name", "EXTEND"),
                         ("Offset", offset_vector)]
            self.set_GN_inputs(mod, paramList)
            
            
        if (controller_type == "BOX"):
            mod = self.add_GN_modifier(tgt_obj, ".GN.Pole.PartsOffset")
            offset_vector = Vector((0, 0, self.controller_box_ext))
            paramList = [("Realize Instance", True),
                         ("Name", "EXTEND"),
                         ("Offset", offset_vector)]
            self.set_GN_inputs(mod, paramList)

            mod = self.add_GN_modifier(tgt_obj, ".GN.Pole.Mask")
            paramList = [("Name1", "MARK"),
                         ("Hide1", self.controller_box_mark),
                         ("Name2", "OPT1"),
                         ("Hide2", self.controller_box_opt1),
                         ]
            self.set_GN_inputs(mod, paramList)
            
            #mod_mask = self.add_mask_modifier(tgt_obj, "MARK", "MaskMark")
            #mod_mask.show_viewport = not self.controller_box_mark
            #mod_mask.show_render = not self.controller_box_mark
            #mod_mask = self.add_mask_modifier(tgt_obj, "OPT1", "MaskOpt1")
            #mod_mask.show_viewport = not self.controller_box_opt1
            #mod_mask.show_render = not self.controller_box_opt1
            
            
            
    def make_sign_board(self, context, pole):

        col = self.get_asset_collection(context, "PoleParts", context.scene.collection)

        base_type = self.sign_board_base_type
        base_style = self.sign_board_base_style

        m_name_pole = self.SignBoardBaseDict[base_type]

        base_z = 5.0
        pos_z = base_z + self.sign_board_offset

        radius0 = 0.128333
        print("radius0=", radius0)
        radius  = self.get_radius_at(pos_z, self.telegraph_type)

        objBoardPole = self.make_obj(context, m_name_pole)
        objBoardPole.parent = pole
        objBoardPole.location.z = pos_z
        objBoardPole.rotation_euler.z = self.appendix_rot_z

        ringInfo = self.RingInfoDict[m_name_pole].copy()
        ringNum = ringInfo[0] * self.telegraph_make_rings

        mod = self.add_GN_modifier(objBoardPole, ".GN.Pole.AdjustParts")
        paramList = [("Offset", self.sign_board_extend),]
        self.set_GN_inputs(mod, paramList)

        if (base_style == "TYPEA"):
            material = self.load_material(".Pole.Basic")
            mod = self.add_GN_modifier(objBoardPole, ".GN.Pole.AddRingConnector")
            paramList = [("ParentPole"  , pole),
                         ("Material"    , material),
                         ("RingNum"     , ringNum),
                         ("RingOffset"  , ringInfo[1]),
                         ("RingInterval", ringInfo[2]),
                         ("RingWidth"   , ringInfo[3]),
                         ("PosOffset"   , Vector((0,0,0)) )]
            self.set_GN_inputs(mod, paramList)

        if (base_style in ["TYPEB", "TYPEC"]):
            for idx in range(ringNum):
                connector_name = {"TYPEB":"ConnectorFull.B.B",
                                  "TYPEC":"ConnectorFull.C.B"}[base_style]
                connector = self.load_object(context, connector_name, col)
                mod = self.add_GN_modifier(objBoardPole, ".GN.Pole.AddConnector.Full")
                paramList = [("ParentPole"  , pole),
                             ("Connector"   , connector),
                             ("BaseValue"   , 0.110),
                             ("RingOffset"  , ringInfo[1]+idx*ringInfo[2]),
                             ("Sel_ScaleXY" , "ScaleXY"),
                             ("ScaleXY"     , 1.2)]
                self.set_GN_inputs(mod, paramList)

            if (base_type in ["TYPEA"]):
                mod = self.add_GN_modifier(objBoardPole, ".GN.Pole.Displace")
                paramList = [("Selection"   , "Base"),
                             ("Strength"    , 0.06),
                             ("Direction"   , Vector((0,1,0))),
                             ("AsInstance"  , 0)]
                self.set_GN_inputs(mod, paramList)
                
                

        if (base_type == "TYPED"):
            objBoardPole.location.x = -(radius - radius0)+0.02
        else:
            objBoardPole.location.y = (radius - radius0)
            
        if (self.flip == True):
            objBoardPole.rotation_euler.z += 3.14159

        if (base_type == "TYPEC") or (base_type == "TYPED"):
            m_name_board = "CV0.6m_Circle"
        else:
            m_name_board = "CH0.6m_Circle"

        board_style = self.sign_board_style
        if (board_style in ["TYPEB", "TYPEC"]):
            m_name_board += ".B"

        material = self.load_material(".Pole.White")
        num = self.sign_board_num

        for i in range(num):
            objBoard = self.make_obj(context, m_name_board)
            objBoard.parent = objBoardPole

            if (base_type == "TYPEC"):
                objBoard.location.y = 1.123 + self.sign_board_extend
                objBoard.location.z = 0.9 - i*0.7
            elif (base_type == "TYPED"):
                objBoard.location.x = -0.40
                objBoard.location.z = 0.6 + self.sign_board_extend - i*0.7
            else:
                objBoard.location.y = 1.5 + self.sign_board_extend - i*0.7
                
            pole_type = "SIGNBOARD"
            
            if (self.traffic_light_mirror == True):
                objBoard.rotation_euler.z = 3.14159

            objBoard.poleprops.pole_type = pole_type

            paramListA = [("Material", material),
                          ("Radius"  , 0.055),
                          ("Depth"   , 0.045)] # 共通
            
            if 'CV' in m_name_board:
                objBoard.poleprops.signboard_type = "CV0.6m_Circle"
                if (board_style in ["TYPEA"]):
                    mod = self.add_GN_modifier(objBoard, '.GN.Pole.SignboardConnector')
                    self.set_GN_inputs(mod, paramListA)
                else:
                    connector_name = {"TYPEB":"ConnectorBoard.A.B",
                                      "TYPEC":"ConnectorBoard.A.C"}[board_style]
                    connector = self.load_object(context, connector_name, col)
                    mod = self.add_GN_modifier(objBoard, '.GN.Pole.AddConnector.Board.V')
                    paramList = [("Connector", connector),]
                    self.set_GN_inputs(mod, paramList)
                    
            if 'CH' in m_name_board:
                objBoard.poleprops.signboard_type = "CH0.6m_Circle"
                if (board_style in ["TYPEA"]):
                    mod = self.add_GN_modifier(objBoard, '.GN.Pole.SignboardConnector.H')
                    self.set_GN_inputs(mod, paramListA)
                else:
                    connector_name = {"TYPEB":"ConnectorBoard.A.B",
                                      "TYPEC":"ConnectorBoard.A.C"}[board_style]
                    connector = self.load_object(context, connector_name, col)
                    mod = self.add_GN_modifier(objBoard, '.GN.Pole.AddConnector.Board.H')
                    paramList = [("Connector", connector),]
                    self.set_GN_inputs(mod, paramList)

            mod = self.add_GN_modifier(objBoard, '.GN.Pole.SignTexID')
            paramList = [("UseTexture", False),
                         ("UDIM_ID", 1),
                         ("scale", 1.0)]
            self.set_GN_inputs(mod, paramList)
            
    def make_footage_bolts(self, context, pole):

        top, bottom, btm_radius, top_radius = self.TelegraphDict[self.telegraph_type]
        pos_z = top - 1.5

        min_height = self.footage_min_height

        self.make_telegraph_footage(context, pole, min_height, pos_z)

    def make_curve_mirror(self, context, pole):

        top, bottom, btm_radius, top_radius = self.TelegraphDict[self.telegraph_type]

        m_name_base = self.CurveMirrorBaseDict[self.curve_mirror_type]
        m_name_mirror = "MirrorConnector." + self.curve_mirror_size
        
        pos_z = 3 + self.curve_mirror_offset

        objBase = self.make_obj(context, m_name_base)
        objBase.parent = pole
        objBase.location.z = pos_z
        objBase.rotation_euler.z = self.curve_mirror_base_rot_z

        material = self.load_material(".Pole.Basic")
        mod_ring = self.add_GN_modifier(objBase, ".GN.Pole.AddRingConnector")
        paramList = [("ParentPole"  , pole),
                     ("Material"    , material),
                     ("RingNum"     , 2 * self.telegraph_make_rings),
                     ("RingOffset"  , -0.09),
                     ("RingInterval", 0.18)]
        self.set_GN_inputs(mod_ring, paramList)

        radius = self.get_radius_at(pos_z, self.telegraph_type)
        self.add_displace_modifier(objBase, "BOX","Y", radius)

        col = self.get_asset_collection(context, "PoleParts", context.scene.collection)
        mod_name = ".GN.Pole.CurveMirrorConnector"
        
        if (self.curve_mirror_type == "SINGLE"):

            objMirror = self.make_obj(context, m_name_mirror)
            objMirror.rotation_euler.z = self.curve_mirror_mirrorA_rot_z
            objMirror.parent = objBase
            objMirror.location = (0, 0.4396, 0.1)
            
            mod = self.add_GN_modifier(objMirror, ".GN.Pole.MirrorConnector.Simple")
            paramList = [("Extend", 0),
                         ("Decline", self.curve_mirror_mirrorA_decline * 9 / 3.14159 )]
            self.set_GN_inputs(mod, paramList)

            
            
        if (self.curve_mirror_type == "DOUBLE"):

            objMirrorA = self.make_obj(context, m_name_mirror)
            objMirrorA.rotation_euler.z = self.curve_mirror_mirrorA_rot_z
            objMirrorA.parent = objBase
            objMirrorA.location = (0.4257, 0.427, 0.1)
            
            mod = self.add_GN_modifier(objMirrorA, ".GN.Pole.MirrorConnector.Simple")
            paramList = [("Extend", 0),
                         ("Decline", self.curve_mirror_mirrorA_decline * 9 / 3.14159 )]
            self.set_GN_inputs(mod, paramList)

            objMirrorB = self.make_obj(context, m_name_mirror)
            objMirrorB.rotation_euler.z = self.curve_mirror_mirrorB_rot_z
            objMirrorB.parent = objBase
            objMirrorB.location = (-0.4257, 0.427, 0.1)
            
            mod = self.add_GN_modifier(objMirrorB, ".GN.Pole.MirrorConnector.Simple")
            paramList = [("Extend", 0),
                         ("Decline", self.curve_mirror_mirrorB_decline * 9 / 3.14159 )]
            self.set_GN_inputs(mod, paramList)
            
    @classmethod
    def poll(cls, context):

        obj = context.object

        if (obj == None):
            return False
        if (obj.select_get() == False):
            return False

        pole_type = obj.poleprops.pole_type
        pole_subtype = obj.poleprops.pole_subtype

        #print("poll", pole_type)
        
        if (pole_type in ["TELEGRAPH", "GUIDESIGN_ARM", "SIGNBOARD_ARM"]):
            return True

        if (pole_type in ["SIGNBOARD_POLE", "SIGNBOARD_ADDTIONAL"]):
            if (pole_subtype in ["0.04m", "0.06m"]):
                return True

        if (pole_type in ["CATENARY"]):
            return True

        if (pole_type in ["TRAFFICLIGHT"]):
            return True
        
        return False

    def invoke(self, context, event):

        annotations = self.__annotations__
        annotations_common = AppendixCommon.__annotations__

        default = annotations["traffic_light_offset"].keywords['default']
        setattr(self, "traffic_light_offset", default + 0.3 * (random.random() - 0.5) )

        default = annotations["curve_mirror_offset"].keywords['default']
        setattr(self, "curve_mirror_offset", default + 0.3 * (random.random() - 0.5) )

        default = annotations["sign_board_offset"].keywords['default']
        setattr(self, "sign_board_offset", default + 0.3 * (random.random() - 0.5) )

        default = annotations["controller_offset"].keywords['default']
        setattr(self, "controller_offset", default + 0.1 * (random.random() - 0.5) )

        default = annotations_common["telegraph_lamp_offset"].keywords['default']
        setattr(self, "telegraph_lamp_offset", default + 0.1 * (random.random() - 0.5) )
        
        return self.execute(context)

    def exec_wire_appendix(self, context, obj):

        if (self.wire_appendix_type == "SLACK"):
            
            mod_slack = self.add_GN_modifier(obj, ".GN.Pole.AddSlack")
            bpy.ops.object.modifier_move_up('INVOKE_DEFAULT', modifier=mod_slack.name)
            material = self.load_material(".Pole.Wire")
            modDict = [("Index", self.wire_spline_index),
                       ("Start", self.slack_start),
                       ("Width", self.slack_width),
                       ("Slack", self.slack_slack),
                       ("Roundness"   , self.slack_roundness),
                       ("RadiusFactor", self.slack_radius),
                       ("Material", material),
                       ]

            self.set_GN_inputs(mod_slack, modDict)
            
            #force update
            obj.data.splines[0].points[0].co[2] = obj.data.splines[0].points[0].co[2]


        if (self.wire_appendix_type in ["BIRDFENCE", "INSULATOR", "CLOSURE"]):

            mod_parts = self.add_GN_modifier(obj, ".GN.MultiPartsAlongCurve")

            end = self.wire_end
            if (end < self.wire_start+0.001):
                end = self.wire_start+0.001

            paramList = [("Index", self.wire_spline_index),
                         ("Count", self.wire_appendix_count),
                         ("Start", self.wire_start),
                         ("Randomize", self.wire_randomize),
                         ("End"  , end)]

            if (self.wire_appendix_type == "BIRDFENCE"):
                objname = self.BirdFenceDict[self.bird_fence_type]

            if (self.wire_appendix_type == "INSULATOR"):
                objname = self.InsulatorDict[self.insulator_type]

            if (self.wire_appendix_type == "CLOSURE"):
                objname = self.ClosureDict[self.closure_type]

            col = self.get_asset_collection(context, "PoleParts", context.scene.collection)
            objParts = self.load_object(context, objname, col)

            paramList.append(("Object", objParts))
            
            self.set_GN_inputs(mod_parts, paramList)
            
        

    def search_guidesign_pos(self, obj):

        childen = [o for o in bpy.data.objects if o.parent == obj]
        y_min = -3

        for o in childen:
            if o.poleprops.pole_type == "GUIDESIGN_BOARD":
                y_min = min(y_min, o.location.y)

        return y_min

    def search_board_pos_H(self, obj):

        childen = [o for o in bpy.data.objects if o.parent == obj]
        y_min = -1
        size = 0

        for o in childen:
            if o.poleprops.pole_type == "SIGNBOARD":
                y_min = min(y_min, o.location.y)
                if (y_min == o.location.y):
                    size = float(o.poleprops.signboard_type[2:5])
                    
        return y_min, size

    def search_board_pos_V_upward(self, obj):

        childen = [o for o in bpy.data.objects if o.parent == obj]
        z_max = 0
        size = 0
        type_full_tag = ""

        for o in childen:
            if o.poleprops.pole_type == "SIGNBOARD":
                z_max = max(z_max, o.location.z)
                if (z_max == o.location.z):
                    type_full_tag = o.poleprops.signboard_type
                    
            if o.poleprops.pole_type == "SIGNBOARD_ADDTIONAL":
                z_max = max(z_max, o.location.z)
                if (z_max == o.location.z):
                    type_full_tag = "ADDITIONAL"

            if o.poleprops.pole_type == "SIGNBOARD_CUSTOM":
                z_max = max(z_max, o.location.z)
                if (z_max == o.location.z):
                    type_full_tag = "CUSTOM"

        return z_max, type_full_tag

    def search_board_pos_V_downward(self, obj):

        childen = [o for o in bpy.data.objects if o.parent == obj]
        z_min = 10
        size = 0
        type_full_tag = ""

        for o in childen:
            if o.poleprops.pole_type == "SIGNBOARD":
                z_min = min(z_min, o.location.z)
                if (z_min == o.location.z):
                    type_full_tag = o.poleprops.signboard_type

            if o.poleprops.pole_type == "SIGNBOARD_ADDTIONAL":
                z_min = min(z_min, o.location.z)
                if (z_min == o.location.z):
                    type_full_tag = "ADDITIONAL"

            if o.poleprops.pole_type == "SIGNBOARD_CUSTOM":
                z_min = min(z_min, o.location.z)
                if (z_min == o.location.z):
                    type_full_tag = "CUSTOM"

        if (z_min == 10):
            z_min = 2.5

        return z_min, type_full_tag
    
    def exec_guidesign_arm(self, context, obj):

        m_name_board = "GuideSign.L." + self.guidesign_large_type

        objBoard = self.make_obj(context, m_name_board)
        objBoard.parent = obj
        objBoard.poleprops.pole_type = "GUIDESIGN_BOARD"

        objBoard.location.y = self.search_guidesign_pos(obj) - 3 - self.guidesign_large_offset

        self.set_guidesign_large_modifier(objBoard)

        if (self.flip == True):
            objBoard.rotation_euler.z = 3.14159

    def exec_signboard_arm(self, context, obj):

        str_layout = "CH"
        str_size = self.signboard_signsize_H
        sign = self.signboard_signtype
        square_type = self.signboard_square_type

        signboard_type = self.get_signboard_type(str_layout, str_size, sign, square_type)
        m_name_board = signboard_type

        objSign = self.make_obj(context, m_name_board)
        objSign.parent = obj
        objSign.poleprops.pole_type = "SIGNBOARD"
        objSign.poleprops.signboard_type = signboard_type

        pos_y, size = self.search_board_pos_H(obj)
        
        #print("size = ", size)
        pos_y -= size / 2 + self.signboard_offset_H
        
        if (str_size == "0.9m"):
            pos_y -= 0.5
            mod_offset = 0.15
        if (str_size == "1.2m"):
            pos_y -= 0.7
            mod_offset = 0.20

        objSign.location.y = pos_y

        material = self.load_material(".Pole.White")
        mod = self.add_GN_modifier(objSign, '.GN.Pole.SignboardConnector.Hx2')
        paramList = [("Material", material),
                     ("Radius", 0.09),
                     ("Depth" , 0.045),
                     ("Offset", mod_offset)]
        self.set_GN_inputs(mod, paramList)

        mod = self.add_GN_modifier(objSign, '.GN.Pole.SignTexID')
        paramList = [("UseTexture", True),
                     ("UDIM_ID", 1),
                     ("scale", 1.0)]
        self.set_GN_inputs(mod, paramList)

        if (self.flip == True):
            objSign.rotation_euler.z = 3.14159

    def exec_addtional_pole(self, context, pole, radius, pos_z):

        m_name_add = "SignboardAdditionalPole"

        objAddtional = self.make_on_pole(context, pole, m_name_add, pos_z, 0, radius = radius)
        objAddtional.rotation_euler.z = self.signboard_rotation
        if (self.pole_type == "SIGNBOARD_ADDTIONAL"):
            objAddtional.location.x = -0.5
            objAddtional.location.z += 0.15
        if (self.pole_type == "SIGNBOARD_POLE"):
            objAddtional.location.z -= 0.15

        self.add_displace_modifier(objAddtional, "EXTEND", "Z", self.signboard_pole_extend)

        objAddtional.poleprops.pole_type = "SIGNBOARD_ADDTIONAL"
        objAddtional.poleprops.pole_subtype = "0.06m"

        material = self.load_material(".Pole.White")
        mod = self.add_GN_modifier(objAddtional, '.GN.Pole.SignboardConnector')
        paramList = [("Material", material),
                     ("Radius"  , radius + 0.005),
                     ("Depth"   , 0.07)]
        self.set_GN_inputs(mod, paramList)

    def exec_signboard_custom(self, context, pole, radius, pos_z):

        m_name_custom = self.signboard_custom

        if not m_name_custom in bpy.data.meshes:
            return

        objCustom = self.make_on_pole(context, pole, m_name_custom, pos_z, 0, radius = radius)
        objCustom.poleprops.pole_type = "SIGNBOARD_CUSTOM"
        objCustom.rotation_euler.z = self.signboard_rotation
        if (self.pole_type == "SIGNBOARD_ADDTIONAL"):
            objCustom.location.x = -0.5
            objCustom.location.z += 0.2
        if (self.pole_type == "SIGNBOARD_POLE"):
            objCustom.location.z -= 0.2

    def get_signboard_layout_str(self):
        str_layout = {"OFFSET":"OV",
                      "CENTERED":"CV"}[self.signboard_layout_type]
        str_size = self.signboard_signsize_V
        if (str_size == "0.6m"):
            str_layout = "CV"

        return str_layout, str_size

    def set_signboard_mod(self, objSign, signboard_type, str_size, radius):

        objSign.rotation_euler.z = self.signboard_rotation
        objSign.poleprops.pole_type = "SIGNBOARD"
        objSign.poleprops.signboard_type = signboard_type
        mod = self.add_GN_modifier(objSign, '.GN.Pole.SignboardConnector')
        paramList = [("Radius", radius + 0.005),
                     ("Depth", {"0.4m":0.03,
                                "0.6m":0.045}[str_size])]
        self.set_GN_inputs(mod, paramList)

        mod = self.add_GN_modifier(objSign, '.GN.Pole.SignTexID')
        paramList = [("UseTexture", True),
                     ("UDIM_ID", 1),
                     ("scale", 1.0)]
        self.set_GN_inputs(mod, paramList)

    def exec_signboard_addtional(self, context, pole):
        pos_z, type_full_tag = self.search_board_pos_V_upward(pole)
        pos_z += self.signboard_offset_V
        if (type_full_tag != ""):
            prop2 = self.Signboard_PropDict[type_full_tag]
            pos_z += prop2[0] 

        str_pole_size = self.pole_subtype
        top_radius = PoleMakerCommons.SignboardPoleDict[str_pole_size][2]

        if (self.signboard_pole_appendix_type == "ADDITIONAL"):
            return self.exec_addtional_pole(context, pole, top_radius, pos_z)

        if (self.signboard_pole_appendix_type == "CUSTOM"):
            return self.exec_signboard_custom(context, pole, top_radius, pos_z)

        str_layout, str_size = self.get_signboard_layout_str()
        sign = self.signboard_signtype
        square_type = self.signboard_square_type

        signboard_type = self.get_signboard_type(str_layout, str_size, sign, square_type)
        m_name_board = signboard_type

        prop1 = self.Signboard_PropDict[signboard_type]
        pos_z += prop1[1] 
        offset_x = -0.5

        objSign = self.make_on_pole(context, pole, m_name_board, pos_z, 0, radius = top_radius)
        objSign.location.x = offset_x

        self.set_signboard_mod(objSign, signboard_type, str_size, top_radius)
        

    def exec_signboard_pole(self, context, pole):

        pos_z, type_full_tag = self.search_board_pos_V_downward(pole)
        if (type_full_tag != ""):
            prop2 = self.Signboard_PropDict[type_full_tag]
            pos_z -= prop2[1] + self.signboard_offset_V

        str_pole_size = self.pole_subtype
        top_radius = PoleMakerCommons.SignboardPoleDict[str_pole_size][2]

        if (self.signboard_pole_appendix_type == "ADDITIONAL"):
            return self.exec_addtional_pole(context, pole, top_radius, pos_z)

        if (self.signboard_pole_appendix_type == "CUSTOM"):
            return self.exec_signboard_custom(context, pole, top_radius, pos_z)

        str_layout, str_size = self.get_signboard_layout_str()
        sign = self.signboard_signtype
        square_type = self.signboard_square_type

        signboard_type = self.get_signboard_type(str_layout, str_size, sign, square_type)
        m_name_board = signboard_type

        prop1 = self.Signboard_PropDict[signboard_type]
        pos_z -= prop1[0] 
        offset_x = 0
        
        objSign = self.make_on_pole(context, pole, m_name_board, pos_z, 0, radius = top_radius)
        objSign.location.x = offset_x

        self.set_signboard_mod(objSign, signboard_type, str_size, top_radius)


    def exec_trafficlight_additional(self, context, obj):

        type = self.trafficlight_addtional_type
        objtype = obj.poleprops.pole_subtype
        subtype = obj.poleprops.pole_subsubtype

        #if "Displace" in obj.modifiers:
        #    extend = obj.modifiers["Displace"].strength
        extend = 0
        if ".GN.Pole.AdjustParts" in obj.modifiers:
            extend = obj.modifiers[".GN.Pole.AdjustParts"]["Input_7"]
            
        if (type == "ARROW"):
            self.add_arrow_signal(context, obj, objtype, subtype, "Y", extend)

        if (type == "HEADBOARD"):
            self.add_head_board(context, obj, subtype, extend)

        if (type == "SIDEBOARD"):
            self.add_side_board(context, obj, subtype, extend)
            
    def execute(self, context):
        
        time0 = time.time()
        
        for obj in context.selected_objects:

            self.pole_type = obj.poleprops.pole_type
            self.pole_subtype = obj.poleprops.pole_subtype

            pole_type = self.pole_type

            if (pole_type == "CATENARY"):
                self.exec_wire_appendix(context, obj)

            if (pole_type == "GUIDESIGN_ARM"):
                self.exec_guidesign_arm(context, obj)

            if (pole_type == "SIGNBOARD_ARM"):
                self.exec_signboard_arm(context, obj)

            if (pole_type == "SIGNBOARD_POLE"):
                self.exec_signboard_pole(context, obj)

            if (pole_type == "SIGNBOARD_ADDTIONAL"):
                self.exec_signboard_addtional(context, obj)

            if (pole_type == "TRAFFICLIGHT"):
                self.exec_trafficlight_additional(context, obj)

            if (pole_type != "TELEGRAPH"):
                continue
            
            self.telegraph_type = obj.poleprops.telegraph_type

            if (self.appendix_type == "TRAFFICLIGHT"):
                self.make_traffic_light_full(context, obj, self.traffic_light_type, self.traffic_light_offset, self.appendix_rot_z, self.flip)

            if (self.appendix_type == "CONTROLLER"):
                self.make_controller(context, obj)

            if (self.appendix_type == "ARM"):
                self.make_arm(context, obj)

            if (self.appendix_type == "SIGNBOARD"):
                self.make_sign_board(context, obj)
                
            if (self.appendix_type == "FOOTAGEBOLTS"):
                self.make_footage_bolts(context, obj)

            if (self.appendix_type == "CURVE_MIRROR"):
                self.make_curve_mirror(context, obj)

            if (self.appendix_type == "LAMP"):
                self.make_lamp(context, obj, self.appendix_rot_z, self.flip)

            if (self.appendix_type == "TRANS"):
                self.make_telegraph_trans(context, obj, 8-self.appendix_trans_offset, self.appendix_rot_z, self.flip)

        time1 = time.time()
        #print("time", time1-time0, time1 - self.time_prev)
        self.time_prev = time1

        return {'FINISHED'}

class OBJECT_OT_AddMark(Operator, PoleMakerCommons, modifier_module.ModifierModule):
    bl_idname = "object.add_traffic_mark"
    bl_label = "Add Traffic Mark"
    bl_description = "Description"
    bl_options = {'REGISTER', 'UNDO'}

    action : StringProperty()


    def MarkCircular_callback(self, context):
        type = "SignCircularType"
        get = get_pcoll_icon_id
        items = [("NOPARKING"    , "No Parking"     , "", get(type, 19), 1),
                 ("NOCAR"        , "No Car"         , "", get(type, 15), 2),
                 ("NOCAR4"       , "No Car 4"       , "", get(type, 16), 3),
                 ("ROADCLOSED"   , "Closed"         , "", get(type, 22), 4),
                 ("NOENTRY"      , "No Entry"       , "", get(type, 17), 5),
                 ("NOTURN"       , "No Turn"        , "", get(type, 21), 6),
                 ("NOOVERTAKE"   , "No Overtake"    , "", get(type, 18), 7),
                 ("NORIGHT"      , "No Right"       , "", get(type, 20), 8),
                 ("LIMIT20"      , "Limit 20"       , "", get(type, 10), 9),
                 ("LIMIT30"      , "Limit 30"       , "", get(type, 11), 10),
                 ("LIMIT40"      , "Limit 40"       , "", get(type, 12), 11),
                 ("LIMIT50"      , "Limit 50"       , "", get(type, 13), 12),
                 ("LIMIT60"      , "Limit 60"       , "", get(type, 14), 13),
                 ("HEIGHT33"     , "Height 3.3m"    , "", get(type, 25), 14),
                 ("HEIGHT25"     , "Height 2.5m"    , "", get(type, 26), 15),
                 ("HEIGHT20"     , "Height 2.0m"    , "", get(type, 27), 16),
                 ("WIDTH22"      , "Width 2.2m"     , "", get(type, 28), 17),
                 ("STRAIGHT"     , "Straight"       , "", get(type, 23), 18),
                 ("LEFTONLY1"    , "Left Only 1"    , "", get(type, 7 ), 19),
                 ("LEFTONLY2"    , "Left Only 2"    , "", get(type, 8 ), 20),
                 ("LEFTRIGHT"    , "Left Right"     , "", get(type, 9 ), 21),
                 ("COMPLEX"      , "Complex"        , "", get(type, 4 ), 22),
                 ("BIKEANDWALKER", "Bike and Walker", "", get(type, 2) , 23),
                 ("BIKE"         , "Bike"           , "", get(type, 1) , 24),
                 ("WALKER"       , "Walker"         , "", get(type, 24), 25),
                 ("CAR"          , "Car"            , "", get(type, 3) , 26),
                 ("CISTERN"      , "Cistern"        , "", get(type, 5 ), 27),
                 ("HYDRANT"      , "Hydrant"        , "", get(type, 6 ), 28),
                 ]
        return items

    def MarkTriangle_callback(self, context):
        type = "SignTriangleType"
        get = get_pcoll_icon_id

        items = [("STOP1"        , "Stop 1"         , "", get(type, 3), 1),
                 ("STOP2"        , "Stop 2"         , "", get(type, 4), 2),
                 ("SLOW1"        , "Slowdown 1"     , "", get(type, 1), 3),
                 ("SLOW2"        , "Slowdown 2"     , "", get(type, 2), 4)
                 ]
        return items

    def MarkCrossing_callback(self, context):
        type = "SignCrossingType"
        get = get_pcoll_icon_id
        items = [("CROSSING_CHILD", "Crossing Child" , "", get(type, 3), 1),
                 ("CROSSING_ADULT", "Crossing Adult" , "", get(type, 1), 2),
                 ("CROSSING_BIKE" , "Crossing Bike"  , "", get(type, 2), 3),
                 ("CROSSING_BOTH" , "Crossing Both"  , "", get(type, 4), 4),
                 ]
        return items

    def MarkRhombus_callback(self, context):
        type = "SignRhombusType"
        get = get_pcoll_icon_id
        items = [("TRAIN"      , "Train"        , "", get(type, 14), 1),
                 ("SCHOOLCHILD", "School Child" , "", get(type, 10), 2),
                 ("SIGNAL"     , "Signal"       , "", get(type, 12), 3),
                 ("ROCKFALL"   , "Rockfall"     , "", get(type, 18), 4),
                 ("WORKING"    , "Working"      , "", get(type, 15), 5),
                 ("ANIMAL"     , "Animal"       , "", get(type, 1 ), 6),
                 ("EXCLAMATION", "Exclamation"  , "", get(type, 7 ), 7),
                 ("SLIPPERY"   , "Slippery"     , "", get(type, 13), 8),
                 ("INCLINE"    , "Incline"      , "", get(type, 17), 9),
                 ("DECLINE"    , "Decline"      , "", get(type, 18), 10),
                 ("SERPENTINE" , "Serpentine"   , "", get(type, 11), 11),
                 ("CRANK_L"    , "Crank L"      , "", get(type, 2 ), 12),
                 ("CRANK_R"    , "Crank R"      , "", get(type, 3 ), 13),
                 ("CURVE_L"    , "Cuver L"      , "", get(type, 5 ), 14),
                 ("CURVE_R"    , "Curve R"      , "", get(type, 6 ), 15),
                 ("CROSS"      , "Cross Road"   , "", get(type, 4 ), 16),
                 ("ROTARY"     , "Rotary"       , "", get(type, 9 ), 17),
                 ("NARROWING"  , "Narrowing"    , "", get(type, 16), 18),
                 ]
        return items

    def MarkSquare_callback(self, context):

        type = "SignSquareType"
        get = get_pcoll_icon_id

        items = [("LEFTSQA"    , "Left A"   , "", get(type, 1 ), 1),
                 ("LEFTSQB"    , "Left B"   , "", get(type, 2 ), 2),
                 ("RIGHTSQA"   , "Right A"  , "", get(type, 3 ), 3),
                 ("RIGHTSQB"   , "Right B"  , "", get(type, 4 ), 4),
                 ("50M"        , "50m"      , "", get(type, 5 ), 5),
                 ("100M"       , "100m"     , "", get(type, 6 ), 6),
                 ("ALLCITY"    , "All City" , "", get(type, 7 ), 7),
                 ("WEEKDAY"    , "Weekday"  , "", get(type, 8 ), 8),
                 ("730-830"    , "7:30-8:30", "", get(type, 9 ), 9),
                 ("FROMHERE"   , "From Here", "", get(type, 13), 10),
                 ("TOHERE"     , "To Here"  , "", get(type, 14), 11),
                 ("SCHOOL"     , "School"   , "", get(type, 15), 12),
                 ("ANIMAL"     , "Animal"   , "", get(type, 16), 13),
                 ("ATTENTION"  , "Attention", "", get(type, 17), 14),
                 ("TRUCK"      , "Truck"    , "", get(type, 10), 15),
                 ("TRUCK3T"    , "Truck 3t" , "", get(type, 12), 16),
                 ("BUS"        , "Bus"      , "", get(type, 11), 17),
                 
                 ]
        
        return items

    def GetNameFromID(self, context, callback, ID):
        items = callback(context)
        for item in items:
            if item[0] == ID:
                return item[1]
        return ""

    MarkTriangleDict =\
        {"STOP1":"Stop1"    , "STOP2":"Stop2",
         "SLOW1":"SlowDown1", "SLOW2":"SlowDown2",
         }

    MarkCrossingDict =\
        {"CROSSING_CHILD":"CrossingChild", "CROSSING_ADULT":"CrossingAdult",
         "CROSSING_BIKE":"CrossingBike"  , "CROSSING_BOTH":"CrossingBoth",
         }

    MarkCircularDict = \
        {"NOPARKING":"NoParking"     , "NOCAR":"NoCar",
         "NOCAR4":"NoCar4"           , "NOENTRY":"NoEntry",
         "ROADCLOSED":"RoadClosed"   , "NOTURN":"NoTurn",
         "NOOVERTAKE":"NoOvertake"   , "NORIGHT":"NoRight",
         "LIMIT20":"Limit20"         , "LIMIT30":"Limit30",
         "LIMIT40":"Limit40"         , "LIMIT50":"Limit50",
         "LIMIT60":"Limit60"         , "HEIGHT33":"LimitHeight33",
         "HEIGHT25":"LimitHeight25"  , "HEIGHT20":"LimitHeight20",
         "WIDTH22":"LimitWidth22"    , "STRAIGHT":"Straight",
         "LEFTONLY1":"LeftOnly1"     , "LEFTONLY2":"LeftOnly2",
         "LEFTRIGHT":"LeftRight"     , "COMPLEX":"Complex",
         "BIKEANDWALKER":"BikeAndWalker", "BIKE":"Bike",
         "WALKER":"Walker"           , "CAR":"Car",
         "CISTERN":"FireCistern"     , "HYDRANT":"FireHydrant"
         }

    MarkSquareDict = \
        {"LEFTSQA":"LeftSQ.A"          , "LEFTSQB":"LeftSQ.B",
         "RIGHTSQA":"RightSQ.A"        , "RIGHTSQB":"RightSQ.B",
         "WEEKDAY":"Supplement.Weekday", "730-830":"Supplement.730-830",
         "50M":"Supplement.50m"        , "100M":"Supplement.100m",
         "ALLCITY":"Supplement.AllCity", "FROMHERE":"Supplement.FromHere",
         "TOHERE":"Supplement.ToHere"  , "TRUCK":"Supplement.Truck",
         "TRUCK3T":"Supplement.Truck3t", "BUS":"Supplement.Bus",
         "SCHOOL":"Supplement.School"  , "ANIMAL":"Supplement.Animal" ,
         "ATENTION":"Supplement.Attention",
         }

    MarkRhombusDict = \
        {"TRAIN":"Rhombus.Train"       , "SCHOOLCHILD":"Rhombus.SchoolChild",
         "SIGNAL":"Rhombus.Signal"     , "ROCKFALL":"Rhombus.RockFall",
         "WORKING":"Rhombus.Working"   , "ANIMAL":"Rhombus.Animal",
         "EXCLAMATION":"Rhombus.Exclamation", "SLIPPERY":"Rhombus.Slippery",
         "INCLINE":"Rhombus.Incline"   ,  "DECLINE":"Rhombus.Decline",
         "SERPENTINE":"Rhombus.Serpentine", "CRANK_L":"Rhombus.Crank.L",
         "CRANK_R":"Rhombus.Crank.R"   , "CURVE_L":"Rhombus.Curve.L",
         "CURVE_R":"Rhombus.Curve.R"   , "CROSS":"Rhombus.Cross",
         "ROTARY":"Rhombus.Rotary"     , "NARROWING":"Rhombus.Narrowing"
         }

    TexIDDict = {"Rhombus.Animal":1,     "Rhombus.Crank.L":2,
                 "Rhombus.Crank.R":3,    "Rhombus.Cross":4,
                 "Rhombus.Curve.L":5,    "Rhombus.Curve.R":6,
                 "Rhombus.Exclamation":7, "Rhombus.RockFall":8,
                 "Rhombus.Rotary":9,      "Rhombus.SchoolChild":10,
                 "Rhombus.Serpentine":11, "Rhombus.Signal":12,
                 "Rhombus.Slippery":13,  "Rhombus.Train":14,
                 "Rhombus.Working":15,   "Rhombus.Narrowing":16,
                 "Rhombus.Incline":17,   "Rhombus.Decline":18,
                 "SlowDown1":1,          "SlowDown2":2,
                 "Stop1":3,              "Stop2":4,
                 "Bike":1,               "BikeAndWalker":2,
                 "Car":3,                "Complex":4,
                 "FireCistern":5,        "FireHydrant":6,
                 "LeftOnly1":7,          "LeftOnly2":8,
                 "LeftRight":9,          "Limit20":10,
                 "Limit30":11,           "Limit40":12,
                 "Limit50":13,           "Limit60":14,
                 "NoCar":15,             "NoCar4":16,
                 "NoEntry":17,           "NoOvertake":18,
                 "NoParking":19,         "NoRight":20,
                 "NoTurn":21,            "RoadClosed":22,
                 "Straight":23,          "Walker":24,
                 "LimitHeight33":25,     "LimitHeight25":26,
                 "LimitHeight20":27,     "LimitWidth22":28,
                 "LeftSQ.A":1,           "LeftSQ.B":2,
                 "RightSQ.A":3,          "RightSQ.B":4,
                 "Supplement.50m":5,     "Supplement.100m":6,
                 "Supplement.AllCity":7, "Supplement.Weekday":8,
                 "Supplement.730-830":9, "Supplement.Truck":10,
                 "Supplement.Bus":11,    "Supplement.Truck3t":12,
                 "Supplement.FromHere":13, "Supplement.ToHere":14,
                 "Supplement.School":15,   "Supplement.Animal":16,
                 "Supplement.Attention":17, "CrossingAdult":1,
                 "CrossingBike":2,       "CrossingChild":3,
                 "CrossingBoth":4, 
                 }
    
    CommonTypeSelection = type_module.TypeModule.CommonTypeSelection
    
    signboard_pole_sign_type : EnumProperty(
        name = "Sign Board", default = 1,
        #items = PoleMakerCommons.EnumCustomIcon_SignboardPoleSignType
        items = PoleMakerCommons.SignboardPoleSignType_callback)
    

    signboard_layout_type : EnumProperty(
        name = "Layout", default = "OFFSET",
        items = PoleMakerCommons.SignboardLayoutType)

    signboard_size_type : EnumProperty(
        name = "Size", default = "0.4m",
        items = PoleMakerCommons.SignboardSizeType)

    signboard_pole_direction : StringProperty(
        name = "Pole Direction")

    signboard_square_type : EnumProperty(
        name = "Square Type", default = "TYPEA",
        items = CommonTypeSelection[0:3])

    use_collection : BoolProperty(
        name = "Use Collection", default = True)

    mark_use_grease_pencil : BoolProperty(
        name = "Set Mark GP", default = False)
    
    mark_circular_type : EnumProperty(
        name = "Type", default = 1,
        items = MarkCircular_callback)

    mark_square_type : EnumProperty(
        name = "Type", default = 1,
        items = MarkSquare_callback)
    
    mark_triangle_type : EnumProperty(
        name = "Type", default = 1,
        items = MarkTriangle_callback)

    mark_crossing_type : EnumProperty(
        name = "Type", default = 1,
        items = MarkCrossing_callback)

    mark_rhombus_type : EnumProperty(
        name = "Type", default = 1,
        items = MarkRhombus_callback)

    mark_depth_offset : FloatProperty(
        name = "Depth Offset", default = 0,
        min = 0, max = 0.05, step = 0.03, precision = 3)
    
    @classmethod
    def poll(cls, context):

        obj = context.object
        if (obj == None):
            return False

        if (obj.select_get() == False):
            return False
        
        if (obj.poleprops.pole_type == "SIGNBOARD"):
            return True
        
        return False

    def invoke(self, context, event):

        obj = context.object

        full_tag = obj.poleprops.signboard_type
        self.signboard_pole_sign_type = self.SignboardMappingDict[full_tag]


        if ("Square A" in full_tag):
            self.signboard_square_type = "TYPEA"
            
        if ("Square B" in full_tag):
            self.signboard_square_type = "TYPEB"
            
        if ("Square C" in full_tag):
            self.signboard_square_type = "TYPEC"
            
        if ("CV" in full_tag):
            self.signboard_layout_type = "CENTERED"
            self.signboard_pole_direction = "V"
            
        if ("OV" in full_tag):
            self.signboard_layout_type = "OFFSET"
            self.signboard_pole_direction = "V"

        if ("CH" in full_tag):
            self.signboard_pole_direction = "H"
            
        if ("0.4m" in full_tag):
            self.signboard_size_type = "0.4m"

        if ("0.6m" in full_tag):
            self.signboard_size_type = "0.6m"

        if ("0.9m" in full_tag):
            self.signboard_size_type = "0.9m"

        if ("1.2m" in full_tag):
            self.signboard_size_type = "1.2m"
            

        return self.execute(context)

    def draw(self, context):
        obj = context.object
        layout = self.layout

        full_type = obj.poleprops.signboard_type
        signboard_type = self.SignboardMappingDict[obj.poleprops.signboard_type]

        col = layout.column(align = True)

        if ("CV" in full_type) or ("OV" in full_type):
            col.prop(self, "signboard_layout_type")

        col.prop(self, "signboard_size_type")
        
        col.separator()

        attrname = "signboard_pole_sign_type"
        if (signboard_type == "TRIANGLEA"):
            current_name = self.get_enum_current_name("mark_triangle_type", self.MarkTriangle_callback(context))
            self.PS3(col, [attrname, "mark_triangle_type", "@"+current_name])
            
        if (signboard_type == "TRIANGLEB"):
            current_name = self.get_enum_current_name("mark_crossing_type", self.MarkCrossing_callback(context))
            self.PS3(col, [attrname, "mark_crossing_type", "@"+current_name])
            
        if ("RHOMBUS" in signboard_type):
            current_name = self.get_enum_current_name("mark_rhombus_type", self.MarkRhombus_callback(context))
            self.PS3(col, [attrname, "mark_rhombus_type", "@"+current_name])
            
        if ("CIRCLE" in signboard_type):
            current_name = self.get_enum_current_name("mark_circular_type", self.MarkCircular_callback(context))
            self.PS3(col, [attrname, "mark_circular_type", "@"+current_name])
            
        if (signboard_type == "SQUARE"):
            current_name = self.get_enum_current_name("mark_square_type", self.MarkSquare_callback(context))
            self.PS3(col, [attrname, "mark_square_type", "#signboard_square_type", "@"+current_name])

        row = col.row()
        row.prop(self, "mark_use_grease_pencil")
        row.prop(self, "use_collection")
        col.prop(self, "mark_depth_offset")

    def get_type_full(self):

        #print("signboard_pole_sign_type=", self.signboard_pole_sign_type)

        inv_map = {v:k for k, v in self.SignboardMappingDict.items()}

        tag = inv_map[self.signboard_pole_sign_type]
        if "Square" in tag:
            tag = tag[:-1] + {"TYPEA":"A",
                              "TYPEB":"B",
                              "TYPEC":"C"}[self.signboard_square_type]

        tag = tag[:1] + self.signboard_pole_direction + self.signboard_size_type + tag[6:]

        #print("size_type=", self.signboard_size_type)
        #print("direction=", self.signboard_pole_direction)
        #print("tag=", tag)
        
        if (self.signboard_size_type == "0.4m"):
            if ("CV" in tag) or ("OV" in tag):
                if (self.signboard_layout_type == "OFFSET"):
                    layout_tag = "OV"
                if (self.signboard_layout_type == "CENTERED"):
                    layout_tag = "CV"

                tag = layout_tag + tag[2:]

        if ("CH" in tag):
            if ("0.4m" in tag):
                tag = tag.replace("0.4m", "0.6m")

        if ("CV" in tag) or ("OV" in tag):
            if ("0.9m" in tag) or ("1.2m" in tag):
                tag = tag.replace("0.9m", "0.6m")
                tag = tag.replace("1.2m", "0.6m")

        return tag

    def change_mesh_type(self, active, full_type):

        m_name = full_type

        self.load_mesh(m_name)

        active.data = bpy.data.meshes[m_name]
        active.poleprops.signboard_type = full_type

        active.name = active.data.name

        print("Change mesh type", full_type)
        print("active.poleprops.signboard_type = ", active.poleprops.signboard_type)
        
        return

    def change_material_slot_type(self, active, full_type):

        name = active.data.name

        #print("name=", name)

        if "Rhombus" in name:
            active.material_slots[1].material = self.load_material(".Pole.Signal.Rhombus")

        if "TriangleA" in name:
            active.material_slots[1].material = self.load_material(".Pole.Signal.Triangle")

        if "Circle" in name:
            active.material_slots[1].material = self.load_material(".Pole.Signal.Circle")

        if "TriangleB" in name:
            active.material_slots[1].material = self.load_material(".Pole.Signal.Crossing")
            
        if "SquareA" in name:
            active.material_slots[1].material = self.load_material(".Pole.Signal.SquareA")

    def set_texID(self, context, active, gpname):

        if not ".GN.Pole.SignTexID" in active.modifiers:
            return {'FINISHED'}

        mod = active.modifiers['.GN.Pole.SignTexID']

        udim_id = self.TexIDDict[gpname]

        paramList = [("UseTexture", True),
                     ("UDIM_ID", udim_id),
                     ("scale", 1.0)]

        self.set_GN_inputs(mod, paramList)

        #print("active.update_tag()")
        active.data.update()
        active.update_tag()
        
        return {'FINISHED'}

    def execute(self, context):

        active = context.view_layer.objects.active

        if (not active):
            return {'FINISHED'}

        pole_type = active.poleprops.pole_type

        if (not "SIGNBOARD" in pole_type):
            return {'FINISHED'}
        
        full_type = self.get_type_full()

        print("full_type", full_type)
        
        if (active.poleprops.signboard_type != full_type):
            self.change_mesh_type(active, full_type)
            
            print("A", active.poleprops.signboard_type)
            signboard_type = self.SignboardMappingDict[active.poleprops.signboard_type]

        #別のマテリアルにした後、戻る場合に if 文内だと戻らないので
        #毎回実行
        self.change_material_slot_type(active, full_type)

        signboard_type = self.SignboardMappingDict[active.poleprops.signboard_type]
        #print(signboard_type)
        gpname = ""
        
        if (signboard_type == "TRIANGLEA"):
            gpname = self.MarkTriangleDict[self.mark_triangle_type]
            
        if (signboard_type == "TRIANGLEB"):
            gpname = self.MarkCrossingDict[self.mark_crossing_type]
            
        if ("RHOMBUS" in signboard_type):
            gpname = self.MarkRhombusDict[self.mark_rhombus_type]
            #if (self.mark_oblique_category == "WARNING"):
            #    gpname = self.MarkWarningDict[self.mark_warning_type]
            #    
            #if (self.mark_oblique_category == "ROAD"):
            #    gpname = self.MarkRoadDict[self.mark_road_type]
                
        if ("CIRCLE" in signboard_type):
            gpname = self.MarkCircularDict[self.mark_circular_type]
            
        if (signboard_type == "SQUARE"):
            gpname = self.MarkSquareDict[self.mark_square_type]

            
        if (self.mark_use_grease_pencil == False):
            return self.set_texID(context, active, gpname)

        if (gpname == ""):
            return {"FINISHED"}
        
        self.load_GP(gpname)

        objMark = self.make_GP(context, gpname, self.use_collection)

        objMark.parent = active

        objMark.rotation_euler.y = -3.14159/2

        if "0.4m" in full_type:
            objMark.scale = Vector((0.666, 0.666, 0.666))
            objMark.location.x = -0.105

            if "OV" in full_type:
                objMark.location.y = 0.1

        if "0.6m" in full_type:
            objMark.location.x = -0.105

        if "0.9m" in full_type:
            objMark.scale = Vector((1.5, 1.5, 1.5))
            objMark.location.x = -0.19
            objMark.location.z = -0.082

        if "1.2m" in full_type:
            objMark.scale = Vector((2, 2, 2))
            objMark.location.x = -0.19
            objMark.location.z = -0.123

        self.add_GP_offset_modifier(objMark, Vector((0, 0, self.mark_depth_offset)))
        
        return {'FINISHED'}

class OBJECT_OT_Preload(Operator, mesh_setting_module.MeshSettingModule):
    bl_idname = "object.pole_maker_preload"
    bl_label = "Preload"
    bl_description = "Preload mesh data for Pole Maker"
    bl_options = {'REGISTER', 'UNDO'}

    def progress_begin(self):
        wm = bpy.context.window_manager
        wm.progress_begin(0, 100)
        wm.progress_update(100)

    def progress_update(self, progress):
        wm = bpy.context.window_manager
        wm.progress_update(progress)

    def progress_end(self):
        wm = bpy.context.window_manager
        wm.progress_end()


    def preload_all(self):

        dict_list = [MESH_OT_AddPoleAppendix.TrafficLightDict,
                     MESH_OT_AddPoleAppendix.TrafficLightDoubleDict,
                     MESH_OT_AddPoleAppendix.TrafficLightWalkerDictA,
                     MESH_OT_AddPoleAppendix.TrafficLightWalkerDictB,
                     MESH_OT_AddPoleAppendix.TrafficLightArrowDict,
                     MESH_OT_AddPoleAppendix.TrafficLightHeadBoardDict,
                     MESH_OT_AddPoleAppendix.TrafficLightSideBoardDict,
                     MESH_OT_AddPoleAppendix.TrafficLightSideBoardWalkerDictA,
                     MESH_OT_AddPoleAppendix.TrafficLightSideBoardWalkerDictB,
                     MESH_OT_AddPoleAppendix.CurveMirrorBaseDict,
                     MESH_OT_AddPoleAppendix.SignBoardBaseDict,
                     MESH_OT_AddPoleAppendix.ControlBoxDict,
                     MESH_OT_AddPoleAppendix.ControlButtonDict,
                     MESH_OT_AddPoleAppendix.BirdFenceDict,
                     MESH_OT_AddPoleAppendix.InsulatorDict,
                     PoleMakerCommons.FootageBoltDict,
                     PoleMakerCommons.ConnectorDict]
        
        dict_list += [MESH_OT_PoleMaker.ArmHVDict,
                      MESH_OT_PoleMaker.ArmLVDict,
                      MESH_OT_PoleMaker.ArmComDict,
                      MESH_OT_PoleMaker.LampDict,
                      MESH_OT_PoleMaker.CapDict,
                      MESH_OT_PoleMaker.TransDict,
                      MESH_OT_PoleMaker.StreetLightSidewalkDict,
                      MESH_OT_PoleMaker.StreetLightDrivewayDict,
                      MESH_OT_PoleMaker.StreetLightParkDict,
                      MESH_OT_PoleMaker.PolesignDict,
                      #MESH_OT_PoleMaker.CurveMirrorDict,
                      MESH_OT_PoleMaker.CurveMirrorConnectorDict,
                      MESH_OT_PoleMaker.CurvePoleDict,
                      MESH_OT_PoleMaker.GuideSignLArmDict,
                      MESH_OT_PoleMaker.GuideSignLArmDict2,
                      MESH_OT_PoleMaker.GuideSignLDict,
                      MESH_OT_PoleMaker.GuideSignSDict,
                      MESH_OT_PoleMaker.AdPoleDict,
                      catenary_module.POLEMAKEROBJ_OT_CatenaryConnect.ClosureDict,]

        key_list = [PoleMakerCommons.SignboardMappingDict,]

        num = len(dict_list)

        self.progress_begin()

        meshname_list = []
        for dict in dict_list:
            
            for value in dict.values():
                
                if type(value) == tuple:
                    value = value[0]

                meshname_list.append(value)

                ADD = MESH_OT_AddPoleAppendix

                if dict in [ADD.TrafficLightDict, ADD.TrafficLightDoubleDict, ADD.TrafficLightArrowDict, ADD.TrafficLightHeadBoardDict, ADD.ControlBoxDict]:

                    meshname_list.append(value + ".mir")
                
                if (dict == ADD.TrafficLightSideBoardDict):
                    if (not 'Double' in value):
                        meshname_list.append(value + ".mir")

        for dict in key_list:
            for key in dict.keys():
                meshname_list.append(key)

        percentage = 10
        self.progress_update(percentage)

        meshname_list += ["EmptyMesh",]

        self.load_mesh_list(meshname_list)

        nodegroup_list = [".GN.Pole.CurveToMesh", ".GN.Pole.BaseColor", ".GN.Pole.BorderYScale", ".GN.Pole.ModifyHSV", ".GN.Pole.SignTexID", ".GN.GuideSignLargeB", ]
        nodegroup_list += [".GN.Pole.InstanceOffset", ".GN.Pole.AddRingConnector", ".GN.Pole.OffsetFromRadius", ".GN.Pole.AddRing", ".GN.TrafficArrow", ".GN.TrafficLight", ".GN.Pole.AddConnector", ".GN.Pole.AddConnector.Full", ".GN.Pole.AddConnector.Board.V", ".GN.Pole.AddConnector.Board.H", ".GN.Pole.CrossingLight", ".GN.Pole.Displace", ]
        
        nodegroup_list += [".GN.Pole.Catenary", ".GN.Pole.FootageBolt", ".GN.Pole.PartsOffset", ".GN.Pole.AdjustParts", ".GN.Pole.AdjustParts.Full", ".GN.Pole.Mir.CrossingLigt"]
        nodegroup_list += [".GN.Pole.SignboardConnector", ".GN.Pole.SignboardConnector.Hx2",]
        nodegroup_list += [".GN.Pole.MirrorConnector.Simple", ".GN.Pole.MirrorConnector.H", ".GN.Pole.CurveMirrorConnector", ]
        nodegroup_list += [".GN.Pole.ColorReplacer", ".GN.Pole.Mask", ]
        nodegroup_list += [".GN.SetAlongCurve", ".GN.MultiPartsAlongCurve", ".GN.Pole.AddSlack"]
        
        
        self.load_NodeGroup_list(nodegroup_list)

        material_list = [".Pole.Concrete",]
        self.load_material_list(material_list)

        self.load_curve("curve_spiral")

        self.progress_end()


    def execute(self, context):

        self.preload_all()

        return {'FINISHED'}
        
class POLEMAKER_PT_PoleObjectMode(Panel):
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_label = "Pole Maker"
    bl_category = "Create"
    bl_context = "objectmode"

    def draw_header(self, context):
        layout = self.layout
        layout.label(icon="PLUGIN")

    def pgettext_dummy(self, text):
        return text

    def draw(self, context):

        use_translate = context.preferences.view.use_translate_interface
        if (use_translate == True):
            pgettext = bpy.app.translations.pgettext
        else:
            pgettext = self.pgettext_dummy

        icons = preview_collections["icons"]

        layout = self.layout

        col = layout.column(align = True)

        col.operator(base_module.POLEMAKEROBJ_OT_BaseObject.bl_idname, text=pgettext("Base Object"), icon = "MESH_DATA").action = "DEFAULT"
        #col.operator(MESH_OT_PoleMaker.bl_idname, text=pgettext("Single Pole"), icon="CURSOR").action = "SINGLE"

        col = layout.column(align = True)
        col.operator(MESH_OT_PoleMaker.bl_idname, text=pgettext("Telegraph"), icon_value = icons["TELEGRAPH"].icon_id).action = "TELEGRAPH"
        
        col.operator(MESH_OT_PoleMaker.bl_idname, text=pgettext("Street Light"), icon="LIGHT").action = "STREETLIGHT"
        col.operator(MESH_OT_PoleMaker.bl_idname, text=pgettext("Signboard Pole"), icon="AUTO").action = "SIGNBOARDPOLE"
        col.operator(MESH_OT_PoleMaker.bl_idname, text=pgettext("Curve Mirror"), icon="PIVOT_ACTIVE").action = "CURVEMIRROR"
        col.operator(MESH_OT_PoleMaker.bl_idname, text=pgettext("Curve Pole"), icon="PLAY_REVERSE").action="CURVEPOLE"
        col.operator(MESH_OT_PoleMaker.bl_idname, text=pgettext("Guide Sign"), icon_value = icons["GUIDESIGN"].icon_id).action = "GUIDESIGN"
        col.operator(MESH_OT_PoleMaker.bl_idname, text=pgettext("Polesign"), icon_value = icons["POLESIGN"].icon_id).action = "POLESIGN"
        
        col = layout.column(align = True)
        col.operator(MESH_OT_AddPoleAppendix.bl_idname, text=pgettext("Add Appendix"), icon="APPEND_BLEND").action = "EXEC"
        col.operator(OBJECT_OT_AddMark.bl_idname, text=pgettext("Traffic Mark"), icon = "AUTO")

        col = layout.column(align = True)
        icon_name = "HAIR_DATA"
        if (bpy.app.version >= (3,2)):
            icon_name = "CURVES_DATA"
        col.operator(catenary_module.POLEMAKEROBJ_OT_CatenaryConnect.bl_idname, text=pgettext("Connect Wire"), icon=icon_name).action = 'CONNECT'
        
        col.operator(catenary_module.POLEMAKEROBJ_OT_JoinWires.bl_idname, text=pgettext("Join Wires"), icon=icon_name).action = 'JOIN'
        
        
        layout.operator(OBJECT_OT_Preload.bl_idname, text=pgettext("Preload"), icon="FILE_REFRESH")
        

class PoleProperties(bpy.types.PropertyGroup):

    pole_type : StringProperty(
        default = "NotDefined")

    pole_subtype : StringProperty(
        default = "NotDefined")

    pole_subsubtype : StringProperty(
        default = "NotDefined")

    telegraph_type : EnumProperty(
        default = "10.0m",
        items = MESH_OT_PoleMaker.TelegraphType)

    signboard_type : StringProperty(
        default = "CV0.6m_Circle)")
        
classes = (
    base_module.POLEMAKEROBJ_OT_BaseObject, 
    MESH_OT_PoleMaker,
    catenary_module.POLEMAKEROBJ_OT_CatenaryConnect,
    catenary_module.POLEMAKEROBJ_OT_JoinWires,
    MESH_OT_AddPoleAppendix,
    OBJECT_OT_Preload,
    OBJECT_OT_AddMark,
    POLEMAKER_PT_PoleObjectMode,
    
    PoleMakerPreferences,
    PoleProperties,
)

def load_icons():

    addon_path = os.path.dirname(__file__)
    icons_dir = os.path.join(addon_path, "icons")
    
    def get_index_dir(HeaderName, num, use_empty):
        if (use_empty == True):
            Dic = {"-1":"Empty.png"}
        else:
            Dic = {}
        for idx in range(1, num):
            Dic.update({"%02d" % idx:HeaderName+"_10%02d.png" % idx})
        return Dic

    def set_icons(icon_id, Dic):
        icons = bpy.utils.previews.new()
        for tag, file in Dic.items():
            icons.load(tag, os.path.join(icons_dir, file), 'IMAGE')
            #print("tag=", tag, icons[tag].icon_id)
            
        preview_collections[icon_id] = icons

    def SetIcons(HeaderName, num, use_empty = False):
        dir = get_index_dir(HeaderName, num, use_empty)
        set_icons(HeaderName+"Type", dir)

    # インデックスが1スタートなのでアイコン数は引数-1なことに注意
    SetIcons("SignCircular", 29)
    SetIcons("SignSquare", 18)
    SetIcons("SignTriangle", 5)
    SetIcons("SignCrossing", 5)
    SetIcons("SignRhombus", 19)
    SetIcons("PoleSign"   , 6)
    SetIcons("Transformer", 6, use_empty = True)
    SetIcons("ArmHV", 7)
    SetIcons("ArmLV", 6)
    SetIcons("ArmCom", 5)
    SetIcons("AppendixLamp", 4)
    SetIcons("AppendixSignboard", 5)
    SetIcons("AppendixMirror", 3)
    SetIcons("TrafficLight", 6)
    SetIcons("TrafficLightWalker", 5)
    SetIcons("Pelican", 4)
    SetIcons("ControllerBox", 5)
    SetIcons("StreetLightDriveWay", 5)
    SetIcons("StreetLightSideWalk", 5)
    SetIcons("StreetLightPark", 5)
    SetIcons("StreetLightGarden", 5)
    SetIcons("StreetLightFootLight", 5)
    SetIcons("CurvePole", 3)
    SetIcons("CurveMirrorPole", 5)
    SetIcons("CurveMirrorBoard", 4)
    SetIcons("BirdFence", 3)
    SetIcons("Insulator", 3)
    SetIcons("Closure", 3)
    SetIcons("AdCircle", 4)
    SetIcons("AdSquare", 4)
    SetIcons("AdLight", 3)
    SetIcons("GuidesignSmall", 4)
    SetIcons("GuidesignLarge", 3)
    
    def SetIconsCustomName(IconDic, collection_name):
        icons = bpy.utils.previews.new()
        for tag, file in IconDic.items():
            icons.load(tag, os.path.join(icons_dir, file), 'IMAGE')
        preview_collections[collection_name] = icons
    
    IconDic = {"TRIANGLEA"   :"TriangleA.png",
               "TRIANGLEB"   :"TriangleB.png",
               "RHOMBUS"     :"Rhombus.png",
               "TELEGRAPH"   :"Telegraph.png",
               "SIGNALPOLE"  :"Signalpole.png",
               "LIMITTING"   :"Limitting.png",
               "SPEED"       :"Speed.png",
               "TRAFFICLIGHT":"TrafficLight.png",
               "GUIDESIGN"   :"Guidesign.png",
               "POLESIGN"    :"Polesign.png",}
    SetIconsCustomName(IconDic, "icons")

    IconDic =  {"SLACK"    :"AppendixWire_Slack.png",
                "BIRDFENCE":"AppendixWire_BirdFence.png",
                "INSULATOR":"AppendixWire_Insulator.png",
                "CLOSURE"  :"AppendixWire_Closure.png",}
    SetIconsCustomName(IconDic, "WireAppendixType")

    IconDic = {"TRAFFICLIGHT":"AppendixType_TrafficLight.png",
               "CONTROLLER"  :"AppendixType_Controller.png",
               "ARM"         :"ArmLV_1002.png",
               "SIGNBOARD"   :"AppendixType_Signboard.png",
               "FOOTAGEBOLTS":"AppendixType_FootageBolt.png",
               "CURVE_MIRROR":"AppendixMirror_1001.png",
               "LAMP"        :"AppendixLamp_1001.png",
               "TRANS"       :"Transformer_1001.png",}
    SetIconsCustomName(IconDic, "AppendixType")

    

def register():

    load_icons()

    from bpy.utils import register_class
    for cls in classes:
        register_class(cls);

    bpy.types.Object.poleprops = PointerProperty(type = PoleProperties)

    bpy.app.translations.register(__name__, translate_module.translate_dict_PoleMaker)

    category_module.update_panel(None, bpy.context)

def unregister():
    
    for pcoll in preview_collections.values():
        bpy.utils.previews.remove(pcoll)

    from bpy.utils import unregister_class
    for cls in classes:
        unregister_class(cls);

    del bpy.types.Object.poleprops

    bpy.app.translations.unregister(__name__)

if __name__ == "__main__":
    register()
